@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -48,7 +48,7 @@ choice do_alias(char *name, Memman *mp, Alias *aliashead, char *dirsuffix,
   if (rc == FALSE)
     return(ERR);
 
-  do_aliasx(workspace, aliashead);
+  (void)do_aliasx(NULL, NULL, workspace, aliashead);
 
   if (!STREQ(workspace, name)) {
     len = strlen(workspace);
@@ -60,36 +60,36 @@ choice do_alias(char *name, Memman *mp, Alias *aliashead, char *dirsuffix,
     return(FALSE);
 }
 
-void do_aliasx(char *name, Alias *aliashead) {
-  /*  User aliases. name must be at least BLOCKSIZE long. */
+/* User-specified aliases.
+ *
+ * If outf != NULL, then the resultant name is printed to outf (even if it
+ * didn't get changed by any alias). od must also be set.
+ *
+ * If outf == NULL, then name is changed if necessary. In this case, name
+ * MUST be at least BLOCKSIZE long.
+ *
+ * In either case, the function returns whether any alias matched.
+ */
+logical do_aliasx(FILE *outf, Outchoices *od, char *name, Alias *aliashead) {
   ptrdiff_t pmatch[PMATCH_SIZE];
   Alias *ap;
-  logical done = FALSE;
+  logical done;
 
-  for (ap = aliashead; ap != NULL && !done; TO_NEXT(ap)) {
+  for (ap = aliashead, done = FALSE; ap != NULL && !done; TO_NEXT(ap)) {
     if (matchq(name,
 	       (ap->isregex)?((void *)(ap->pattern)):((void *)(ap->from)),
 	       ap->isregex, pmatch)) {
       done = TRUE;
-      wildalias(name, ap->to, pmatch);
+      if (outf != NULL)
+	printalias(outf, od, name, ap->to, pmatch);
+      else
+	wildalias(name, ap->to, pmatch);
     }
   }
-}
+  if (!done && outf != NULL)
+    htmlputs(outf, od, name, UNTRUSTED);
 
-void do_aliasrep(char *name, Alias *aliashead, logical multibyte) {
-  /* Call do_aliasx(), then convert unprintable chars to '?'. We sometimes rely
-     on the fact that this second operation doesn't change the length of the
-     string. name must be at least BLOCKSIZE long */
-  extern logical unprintable[256];
-  char *n;
-
-  do_aliasx(name, aliashead);
-  if (!multibyte) {
-    for (n = name; *n != '\0'; n++) {
-      if (unprintable[(unsigned char)(*n)])
-	*n = '?';
-    }
-  }
+  return(done);
 }
 
 logical do_aliasr(char *name,  char *dirsuffix, unsigned int dirsufflength) {
@@ -135,9 +135,9 @@ logical do_aliasr(char *name,  char *dirsuffix, unsigned int dirsufflength) {
 
   /* Otherwise, change backslashes to forward slashes. This is not strictly
      correct, but it's usually the right thing to do (to make the Directory
-     Report correct, and to avoid escaping following char. at print time). */
+     Report correct). */
 
-  for ( ; *c != '\0'; c++) {
+  for ( ; *c != '\0' && *c != '?'; c++) {
     if (*c == '\\')
       *c = '/';
   }
@@ -287,10 +287,10 @@ logical do_aliasf(char *name) {
     defaultport = 0;
   *c = ':';
 
-  /* Change backslash -> slash for reasons given in do_aliasr() */
+  /* Change backslash -> slash for the reason given in do_aliasr() */
 
   c++;
-  for (d = c; *d != '\0'; d++) {
+  for (d = c; *d != '\0' && *d != '?'; d++) {
     if (*d == '\\')
       *d = '/';
   }
@@ -402,7 +402,7 @@ logical do_aliasN(char *name, unsigned char convfloor, logical multibyte) {
     if (*c == '-') {  /* remove except at beginning of word */
       if ((c != name && *(c - 1) != ' ') ||
 	  *(c + 1) == '+' || *(c + 1) == '-' || *(c + 1) == '"' ||
-	  *(c + 1) == ',' || *(c + 1) == ';' || *(c + 1) == '(' ||
+	  *(c + 1) == ',' || *(c + 1) == '(' ||
 	  *(c + 1) == ')' || *(c + 1) == '.' || *(c + 1) == '\0' ||
 	  ISSPACE(*(c + 1)))
 	*c = ' ';
@@ -411,7 +411,7 @@ logical do_aliasN(char *name, unsigned char convfloor, logical multibyte) {
       if (c == name || !ISALNUM(*(c - 1)) || !ISALNUM(*(c + 1)))
 	*c = ' ';
     }
-    else if (*c == '+' || *c == '"' || *c == ',' || *c == ';' ||
+    else if (*c == '+' || *c == '"' || *c == ',' ||
 	     *c == '(' || *c == ')' || ISSPACE(*c))
       *c = ' ';
   }
@@ -479,15 +479,64 @@ logical do_aliasn(char *name, unsigned char convfloor, logical multibyte) {
 }
 
 logical do_aliasb(char *name) {
-  /* NB aliasB is at parse time, aliasb for compiling browsum */
-  /* not much left to do here coz most of the intelligence is in Bnextname() */
-  if (headmatch(name, "WebTV ") || headmatch(name, "Opera ")) {
-    if (ISDIGIT(*(name + 6)))
-      *(name + 5) = '/';
-  }
-  else if (headmatch(name, "MSIE ") && ISDIGIT(*(name + 5)))
-    *(name + 4) = '/';
+  /* NB aliasB is at parse time, aliasb for compiling Browser Summary. */
+  /* See Bnextname() for the role of this function. */
+  char *s, *t;
+  logical done = FALSE;
 
+  if (headmatch(name, "Mozilla")) {
+    if (*(name + 7) == '/' && *(name + 8) == '5') {
+      /* Mozilla/5 is messy. The real version number should be written at the
+	 end of the brackets. Furthermore, there are (so far) three different
+	 version number formats, "m18", "0.8" and "rv:0.9.1+".
+	 
+	 We've filtered out Mozilla/5.0 (compatible) in Bnextname() so these
+	 should all be genuine Mozilla. But if we can't find a version number
+	 we'll just leave it as Mozilla, not Mozilla/5.0, to avoid confusing
+	 the two different numbering schemes. */
+      if ((t = strstr(name, ") Gecko")) != NULL) {
+	/* There should only be one ')', but let's be cautious. */
+	for (s = t - 1; *s != ' ' && s != name; s--)
+	  ;  /* run back to space before last element inside brackets */
+	if (s != name && *(s - 1) == ';') {
+	  s++;
+	  if (headmatch(s, "rv:"))
+	    s += 3;
+	  if (ISDIGIT(*s) || (*s == 'm' && ISDIGIT(*(s + 1)))) { /* Found it */
+	    if (*s == 'm')
+	      *s = 'M';
+	    memcpy(name + 8, s, t - s);
+	    *(name + 8 + (t - s)) = '\0';
+	    done = TRUE;
+	  }
+	}
+      }
+      if (!done)
+	*(name + 7) = '\0';
+      return(TRUE);
+    }
+
+    /* All other Mozilla's are reckoned to be Netscape */
+    memmove(name + 8, name + 7, strlen(name + 7) + 1);
+    strncpy(name, "Netscape", 8);
+  }
+
+  /* Convert "Galeon; n.m" to "Galeon/n.m"; n a digit tested in Bnextname() */
+  else if (headmatch(name, "Galeon; ")) {
+    *(name + 6) = '/';
+    memmove(name + 7, name + 8, strlen(name + 8) + 1);
+    return(TRUE);
+  }
+
+  /* Netscape6 is actually Netscape */
+  else if (headmatch(name, "Netscape6"))
+    memmove(name + 8, name + 9, strlen(name + 9) + 1);
+
+  /* In general, convert "Name n.m" to "Name/n.m" if n a digit */
+  for (s = name; ISALNUM(*s); s++)
+    ;
+  if (*s == ' ' && ISDIGIT(*(s + 1)))
+    *s = '/';
   return(TRUE);
 }
 
@@ -531,12 +580,12 @@ logical dnsresolve(char *name, choice level) {
       signal(SIGALRM, sighandler);
       alarm(dnstimeout);
 #endif
-      tempp = gethostbyaddr(addrp, sizeof(struct in_addr), AF_INET);
+      tempp = gethostbyaddr(addrp, sizeof(IN_ADDR_T), AF_INET);
 #ifndef NOALARM
       alarm(0);
       signal(SIGALRM, SIG_DFL);
 #endif
-      if (tempp != NULL && tempp->h_name[0] != '\0') {
+      if (tempp != NULL && tempp->h_name != NULL && tempp->h_name[0] != '\0') {
 	/* hostnames shouldn't be 32,000 characters long, but... */
 	strncpy(name, tempp->h_name, BLOCKSIZE - 1);
 	name[BLOCKSIZE - 1] = '\0';
@@ -632,18 +681,37 @@ void do_dns(char *name, char *alias, choice level) {
 }
 #endif
 
+/* Print the alias of a name to outf. Matching has already been checked. */
+void printalias(FILE *outf, Outchoices *od, char *name, AliasTo *to,
+		ptrdiff_t pmatch[]) {
+  char c;
+
+  for ( ; to != NULL; TO_NEXT(to)) {
+    htmlputs(outf, od, to->string, FROM_CFG);
+    if (to->after >= 0 && pmatch[to->after] >= 0) {
+      c = *(name + pmatch[to->after + 1]);
+      /* temporarily set end of portion to \0 */
+      *(name + pmatch[to->after + 1]) = '\0';
+      htmlputs(outf, od, name + pmatch[to->after], UNTRUSTED);
+      *(name + pmatch[to->after + 1]) = c;
+    }
+  }
+}
+
+/* Change name to its alias. Matching has already been checked.
+   name MUST have length >= BLOCKSIZE. */
 void wildalias(char *name, AliasTo *to, ptrdiff_t pmatch[]) {
-  /* Matching has already been checked. name must have length >= BLOCKSIZE. */
   char *s, *ans;
   ptrdiff_t *pm;
-  size_t len;
+  size_t len, tolen;
 
   len = strlen(name);
   ans = name + len;
   /* The aliased name is built AFTER the orig name. */
   for (s = ans; to != NULL; TO_NEXT(to)) {
     pm = &(pmatch[to->after]);
-    len += strlen(to->string);
+    tolen = strlen(to->string);
+    len += tolen;
     if (to->after >= 0 && pm[0] >= 0)
       len += (size_t)(pm[1] - pm[0]);
     if (len >= BLOCKSIZE - 1) {
@@ -651,7 +719,7 @@ void wildalias(char *name, AliasTo *to, ptrdiff_t pmatch[]) {
       return;
     }
     strcpy(s, to->string);
-    s = strchr(s, '\0');
+    s += tolen;
     if (to->after >= 0 && pm[0] >= 0) {
       memcpy((void *)s, (void *)(name + pm[0]), (size_t)(pm[1] - pm[0]));
       s += pm[1] - pm[0];
@@ -773,8 +841,9 @@ void reverseonename(Hashindex *p) {
   size_t l, m = 0;
   logical done;
 
-  if (*(p->name) == '\f' || *(p->name) == '\b' || *(p->name) == '\v')
-    return;   /* special codes in Znextname() */
+  /* reversing done in own space, so pull off const char * cases first */
+  if (STREQ(p->name, LNGSTR_NODOMAIN) || STREQ(p->name, LNGSTR_UNKDOMAIN))
+    return;
   l = strlen(p->name);                             /* numerical host */
   if (ISDIGIT(*(p->name + l - 1))  && ISDIGIT(*(p->name))) {
     t1 = p->name;
