@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -44,7 +44,7 @@ int main(int argc, char *argv[])
     if (lf != NULL) {
       readcache(logfilep, lf, op->hash, op->no_cols, op->datacols,
 		op->data2cols, op->arraydata, op->wanthead, op->ispagehead,
-		op->aliashead, &(op->dman), op->outopts.tree, op->outopts.derv,
+		&(op->dman), op->outopts.tree, op->outopts.derv,
 		op->outopts.alltrees, op->outopts.alldervs,
 		op->miscopts.lowmem, op->miscopts.dirsuffix,
 		op->miscopts.dirsufflength, op->miscopts.usercase_insensitive,
@@ -106,6 +106,10 @@ int main(int argc, char *argv[])
 	  fprintf(errfile, "%s: [%02d:%02d:%02d] %lu lines read\n",
 		  commandname, pflt->tm_hour, pflt->tm_min, pflt->tm_sec,
 		  linesread);
+	  if (errfile != stderr)
+	    fprintf(stderr, "%s: [%02d:%02d:%02d] %lu lines read\n",
+		    commandname, pflt->tm_hour, pflt->tm_min, pflt->tm_sec,
+		    linesread);
 	  nextreport += progressfreq;
 	}
 #ifdef MAC_EVENTS
