@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -14,7 +14,7 @@
 
 /*** OS specific things ***/
 
-#define VNUMBER "5.03"    /* the version number of this program */
+#define VNUMBER "5.22"    /* the version number of this program */
 
 #undef VERSION /* If no OS set, compilation will fail by not knowing VERSION */
 
@@ -92,20 +92,32 @@
 #define VERSION VNUMBER"/OpenVMS"
 #endif
 
+#ifdef BEOS
+#define NOALARM
+#define NOFOLLOW
+#define PATHSEPS "/"
+#define DIRSEP '/'
+#define EXTSEP '.'
+#define VERSION VNUMBER"/BeOS"
+#endif
+
 #ifdef OSX
 #define PATHSEPS "/"
 #define DIRSEP '/'
 #define EXTSEP '.'
 #define VERSION VNUMBER"/OS X"
-#else
+#endif
 
 #ifdef UNIX
+#ifndef OSX
+#ifndef BEOS
 #define PATHSEPS "/"
 #define DIRSEP '/'
 #define EXTSEP '.'
 #define VERSION VNUMBER"/Unix"
 #endif
 #endif
+#endif
 
 #ifdef WIN32
 #define popen _popen
@@ -159,15 +171,6 @@ extern void Win32Cleanup(void);
 #define VERSION VNUMBER"/NeXTSTEP"
 #endif
 
-#ifdef BEOS
-#define NOALARM
-#define NOFOLLOW
-#define PATHSEPS "/"
-#define DIRSEP '/'
-#define EXTSEP '.'
-#define VERSION VNUMBER"/BeOS"
-#endif
-
 #ifdef MPEIX
 #define NOFOLLOW
 #define PATHSEPS "/"
@@ -345,8 +348,8 @@ extern void mac_free(void *ptr);
 #include <OpenTptInternet.h>
 #include <AddressXlation.h>
 #endif
-#else
-#ifdef HAVE_ADDR_T           /* Win32 and normal DNS */
+#else                        /* Win32 and normal DNS */
+#ifdef HAVE_ADDR_T   /* define in Makefile if in_addr_t != unsigned long */
 #define IN_ADDR_T in_addr_t
 #else
 typedef unsigned long IN_ADDR_T;
@@ -452,6 +455,26 @@ typedef unsigned long IN_ADDR_T;
 #define MAC_IDLE_FREQ (200)
 #endif
 
+/* Special codes for certain reports. We collect them here just to avoid
+   changing several instances if they change (although see Pnenxtname() too).
+   *
+   It would be nice to let the user use localised versions of these in e.g.
+   *EXCLUDE commands, but that doesn't scale to the case of reports in two
+   languages in one pass, because not all trees are built as late as output
+   time. */
+#define LNGSTR_UNRESOLVED "[unresolved numerical addresses]"
+#define LNGSTR_NODOMAIN "[domain not given]"
+#define LNGSTR_UNKDOMAIN "[unknown domain]"
+#define LNGSTR_ROOTDIR "[root directory]"
+#define LNGSTR_NODIR "[no directory]"
+#define LNGSTR_NOEXT "[no extension]"
+#define LNGSTR_BRKDIRS "[directories]"
+#define LNGSTR_UNKWIN "Unknown Windows"
+#define LNGSTR_UNKMAC "Unknown Macintosh"
+#define LNGSTR_UNKUX "Other Unix"
+#define LNGSTR_ROBOTS "Robots"
+#define LNGSTR_UNKOS "OS unknown"
+
 /*** logfile line types ***/
 
 #define LOG_COMMON1 "%S %j %u [%d/%M/%Y:%h:%n:%j] \"%j%w%r%wHTTP%j\" %c %b"
@@ -464,14 +487,10 @@ typedef unsigned long IN_ADDR_T;
   "%S %j %u [%d/%M/%Y:%h:%n:%j] \"%j%w%r\" %c %b \"%f\" \"%B\""
 #define LOG_COMBINED3 \
   "%S %j %u [%d/%M/%Y:%h:%n:%j] \"%r\" %c %b \"%f\" \"%B\""
-#define LOG_MS_INT2A "%S, %u, %d/%m/%y, %h:%n:%j, W3SVC%j, %j, %v, %T, %j, %b, %c, %j, %j, %r, %q,"
-#define LOG_MS_INT2B "%*S, %*u, %d/%m/%y, %h:%n:%j, %j"
-#define LOG_MS_INT4A "%S, %u, %d/%m/%Y, %h:%n:%j, W3SVC%j, %j, %v, %T, %j, %b, %c, %j, %j, %r, %q,"
-#define LOG_MS_INT4B "%*S, %*u, %d/%m/%Y, %h:%n:%j, %j"
-#define LOG_MS_NA2A "%S, %u, %m/%d/%y, %h:%n:%j, W3SVC%j, %j, %v, %T, %j, %b, %c, %j, %j, %r, %q,"
-#define LOG_MS_NA2B "%*S, %*u, %m/%d/%y, %h:%n:%j, %j"
-#define LOG_MS_NA4A "%S, %u, %m/%d/%Y, %h:%n:%j, W3SVC%j, %j, %v, %T, %j, %b, %c, %j, %j, %r, %q,"
-#define LOG_MS_NA4B "%*S, %*u, %m/%d/%Y, %h:%n:%j, %j"
+#define LOG_MS_INT1 "%S, %u, %d/%m/%Z, %h:%n:%j, W3SVC%j, %j, %v, %T, %j, %b, %c, %j, %j, %r, %q,"
+#define LOG_MS_INT2 "%*S, %*u, %d/%m/%Z, %h:%n:%j, %j"
+#define LOG_MS_NA1 "%S, %u, %m/%d/%Z, %h:%n:%j, W3SVC%j, %j, %v, %T, %j, %b, %c, %j, %j, %r, %q,"
+#define LOG_MS_NA2 "%*S, %*u, %m/%d/%Z, %h:%n:%j, %j"
 #define LOG_WEBSTAR1 "!!LOG_FORMAT%x2"
 #define LOG_WEBSTAR2 "!!%j"
 #define LOG_EXTENDED1 "#Fields:%x3"
@@ -482,6 +501,7 @@ typedef unsigned long IN_ADDR_T;
   "%m/%d/%y %h:%n:%j\t%S\t%v\t%j\t%u\t%j\t%r\t%f\t%j\t%B\t%c\t%b\t%T"
 #define LOG_WEBSITE_INT \
   "%d/%m/%y %h:%n:%j\t%S\t%v\t%j\t%u\t%j\t%r\t%f\t%j\t%B\t%c\t%b\t%T"
+#define LOG_MACHTTP "%m/%d/%y\t%h:%n:%j \t%C%w%S\t%r\t%b"
 #define LOG_NETSCAPE "format=%x4"
 #define LOG_REFERRER1 "[%d/%M/%Y:%h:%n:%j] %F -> %*r"
 #define LOG_REFERRER2 "%F -> %*r"
@@ -569,12 +589,17 @@ enum {SUNDAY, MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY};
 /* languages; see also country in globals.c */
 enum {ARMENIAN, /*BOSNIAN,*/ BULGARIAN, BULGARIAN_MIK, CATALAN,
       /*SIMP_CHINESE,*/ TRAD_CHINESE, /*CROATIAN,*/ CZECH, CZECH_1250, DANISH,
-      DUTCH, ENGLISH, US_ENGLISH, /*FINNISH,*/ FRENCH, GERMAN, /*GREEK,*/
+      DUTCH, ENGLISH, US_ENGLISH, FINNISH, FRENCH, GERMAN, /*GREEK,*/
       HUNGARIAN, /*ICELANDIC,*/ ITALIAN, JAPANESE_EUC, JAPANESE_JIS,
       JAPANESE_SJIS, JAPANESE_UTF, KOREAN, LATVIAN, /*LITHUANIAN,*/
       NORWEGIAN, NYNORSK, POLISH, PORTUGUESE, BR_PORTUGUESE,
       /*ROMANIAN, RUSSIAN,*/ SERBIAN, /*SLOVAK,*/ SLOVENE, SLOVENE_1250,
       SPANISH, SWEDISH, SWEDISH_ALT, TURKISH, UKRAINIAN};
+#define OLDLANG (-2)
+
+/* source of data passed to htmlputs(): see comments there. These represent
+   roughly increasing levels of security. */
+enum {TRUSTED, FROM_CFG, UNTRUSTED, IN_HREF};
 
 /* lngstrs; charset_ must be first */
 enum {charset_, weekbeg_, year_, quarteryr_, month_, day_, days_, hr_, minute_,
@@ -583,7 +608,9 @@ enum {charset_, weekbeg_, year_, quarteryr_, month_, day_, days_, hr_, minute_,
       file_, files_, host_, hosts_, vhost_, vhosts_, dir_, dirs_, dom_, doms_,
       org_, orgs_, ext_, exts_, url_, urls_, browser_, browsers_, os_, oss_,
       size_, search_, searches_, site_, sites_, user_, users_, code_, codes_,
-      webstatsfor_, gensum_, yearrep_, busyyear_, quarterlyrep_,
+      webstatsfor_, kilo_, mega_, giga_, tera_, peta_, exa_, zeta_, yotta_,
+      kiloabbr_, megaabbr_, gigaabbr_, teraabbr_, petaabbr_, exaabbr_,
+      zetaabbr_, yottaabbr_, gensum_, yearrep_, busyyear_, quarterlyrep_,
       busyquarteryr_, monthrep_, busymonth_, weekrep_, busyweek_, daysum_,
       dayrep_, busyday_, hourrep_, hoursum_, weekhoursum_, busyhour_,
       quarterrep_, quartersum_, busyquarterhr_, fiverep_, fivesum_, busyfive_,
@@ -642,6 +669,8 @@ enum {charset_, weekbeg_, year_, quarteryr_, month_, day_, days_, hr_, minute_,
       code414_, code415_, code416_, code417_, code499_, code500_, code501_,
       code502_, code503_, code504_, code505_, code506_, code599_, code999_,
       LNGSTR_NUMBER};
+#define byteprefix_ (kilo_ - 1)
+#define byteprefixabbr_ (kiloabbr_ - 1)
 
 /* return codes from strtoinfmt() */
 /* see also configlogfmt() in init2.c */
@@ -676,6 +705,8 @@ typedef struct aliasto {
   choice after;
   struct aliasto *next;
 } AliasTo;
+/* The target of an alias alternates a "string" and possibly a part of the
+   original name indexed by "after" (else after == -1) (see printalias()). */
 
 typedef struct alias {
   char *from;
@@ -710,6 +741,11 @@ typedef struct {
   void *opt;
 } Configfns;
 
+typedef struct {
+  char *name;
+  choice arg;
+} Choices;
+
 typedef struct {
   char code;
   choice type;
@@ -834,7 +870,7 @@ typedef struct {
   char *outfile, *cacheoutfile;
   char *domainsfile, *descfile, *headerfile, *footerfile;
   choice outstyle, gotos;
-  logical descriptions, multibyte, html, repspan;
+  logical descriptions, multibyte, html, repspan, pdflatex;
   unsigned int rsthresh;
   choice reporder[REP_NUMBER + 1];
   logical repq[REP_NUMBER], last7, runtime;
@@ -866,6 +902,7 @@ typedef struct {
   Include *wanthead[GENREP_NUMBER];
   Alias *aliashead[GENREP_NUMBER];
   Include *link[GENREP_NUMBER];
+  Strlist *expandhead[GENREP_NUMBER];
   char *baseurl;
   logical searchconv;
   unsigned char convfloor;
@@ -974,8 +1011,8 @@ typedef struct {
 #define ENTRY_BLANK(e) ((e) == blank_entry)
 #define LEN3(n, c) ((unsigned int)((c == '\0')?(n):((n) + (((n) - 1) / 3))))
 /* The length of an n digit number with seps; e.g. 56789 [5] -> 56,789 [6] */
-#define DEFAULTSTR(s, n) {s = NULL; \
-                          configstr((void *)(&(s)), NULL, n, NULL, -1);}
+#define COPYSTR(s, n) {char *z = n; s = (char *)xmalloc(strlen(z) + 1); \
+                       strcpy(s, z);}  /* z needed so n only evaluated once */
 #define TOO_FULL(n, z) ((n) > (z))   /* defn for hash table too full */
 #define TOO_FULL_TREE TOO_FULL
 #define NEW_SIZE(z) (2 * (z))        /* new size of the hash table */
