@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -16,8 +16,8 @@
 extern choice do_alias(char *name, Memman *mp, Alias *aliashead,
   char *dirsuffix, unsigned int dirsufflength, logical usercase_insensitive,
   unsigned char convfloor, logical multibyte, choice type);
-extern void do_aliasx(char *name, Alias *aliashead);
-extern void do_aliasrep(char *name, Alias *aliashead, logical multibyte);
+extern logical do_aliasx(FILE *outf, Outchoices *od, char *name,
+  Alias *aliashead);
 extern logical do_aliasr(char *name, char *dirsuffix,
   unsigned int dirsufflength);
 extern logical do_aliasS(char *name);
@@ -34,6 +34,8 @@ extern logical do_aliasb(char *name);
 extern logical dnsresolve(char *name, choice level);
 extern void do_dns(char *name, char *alias, choice level);
 #endif
+extern void printalias(FILE *outf, Outchoices *od, char *name, AliasTo *to,
+  ptrdiff_t pmatch[]);
 extern void wildalias(char *name, AliasTo *to, ptrdiff_t pmatch[]);
 extern choice prealias(Memman *mm, Memman *mmv, Hashentry *vhost, Memman *mmq,
   logical case_insensitive, char *prefix, size_t prefixlen, int pvpos,
@@ -51,7 +53,7 @@ extern void outputcache(char *outfile, Hashindex **gooditems, Dateman *dman,
 extern void hashbuild(choice code, unsigned long *data, double bys, Memman *mm,
   Hashtable **hash, unsigned int data_number,
   choice datacols[OUTCOME_NUMBER][DATACOLS_NUMBER][2], choice data2cols[],
-  Include *wanthead, Include *ispagehead, Alias *aliashead, Tree **tree,
+  Include *wanthead, Include *ispagehead, Tree **tree,
   Derv **derv, choice *alltrees, choice *alldervs, choice lowmem,
   char *dirsuffix, unsigned int dirsufflength, logical usercase_insensitive,
   unsigned char convfloor, logical multibyte);
@@ -66,7 +68,7 @@ extern void readcache(Filelist *cachefilep, FILE *lf, Hashtable **hash,
   choice datacols[ITEM_NUMBER][OUTCOME_NUMBER][DATACOLS_NUMBER][2],
   choice data2cols[ITEM_NUMBER][DATA_NUMBER],
   Arraydata **arraydata, Include **wanthead, Include *ispagehead,
-  Alias **aliashead, Dateman *dman, Tree **tree, Derv **derv, choice *alltrees,
+  Dateman *dman, Tree **tree, Derv **derv, choice *alltrees,
   choice *alldervs, choice *lowmem, char *dirsuffix,
   unsigned int dirsufflength, logical usercase_insensitive,
   unsigned char convfloor, logical multibyte, unsigned int granularity);
@@ -111,7 +113,7 @@ extern void unhashall(Hashtable **hash, Hashindex ***gooditems,
 /* in init.c */
 extern void initialise(int argc, char *argv[], Options *op);
 extern void confline(Options *op, char *cmd, char *arg1, char *arg2, int rc);
-extern choice config(char *filename, Options *op);
+extern choice config(char *filename, Options *op, logical fromcommandline);
 extern void settings(Options *op, int argc, char *argv[]);
 extern Inputformatlist *correctlogfmt(Filelist *lp, Include **wanthead,
   choice *code2type, Dateman *dman);
@@ -133,26 +135,15 @@ extern void configcall(void *opt, char *cmd, char *arg1, char *arg2, int rc);
 extern void configcols(void *opt, char *cmd, char *arg1, char *arg2, int rc);
 extern void configallcols(void *opt, char *cmd, char *arg1, char *arg2,
   int rc);
-extern void configsortby(void *opt, char *cmd, char *arg1, char *arg2, int rc);
+extern void configchoice(void *opt, char *cmd, char *arg1, char *arg2, int rc);
 #ifndef NOGRAPHICS
-extern void configchart(void *opt, char *cmd, char *arg1, char *arg2, int rc);
 extern void configallchart(void *opt, char *cmd, char *arg1, char *arg2,
   int rc);
 #endif
 extern void configdebug(void *opt, char *cmd, char *arg1, char *arg2, int rc);
-extern void configonoff(void *opt, char *cmd, char *arg1, char *arg2, int rc);
 extern void configall(void *opt, char *cmd, char *arg1, char *arg2, int rc);
 extern void configallback(void *opt, char *cmd, char *arg1, char *arg2,
   int rc);
-extern void configgotos(void *opt, char *cmd, char *arg1, char *arg2, int rc);
-extern void configcase(void *opt, char *cmd, char *arg1, char *arg2, int rc);
-extern void configweekday(void *opt, char *cmd, char *arg1, char *arg2,
-  int rc);
-extern void configoutstyle(void *opt, char *cmd, char *arg1, char *arg2,
-  int rc);
-#ifndef NODNS
-extern void configdns(void *opt, char *cmd, char *arg1, char *arg2, int rc);
-#endif
 extern void configlang(void *opt, char *cmd, char *arg1, char *arg2, int rc);
 extern void selectlang(char *country, Outchoices *op);
 extern void configlogfmt(void *opt, char *cmd, char *arg1, char *arg2, int rc);
@@ -165,12 +156,15 @@ extern void configstrlist(void *opt, char *cmd, char *arg1, char *arg2,
 extern void configerrfile(void *opt, char *cmd, char *arg1, char *arg2,
   int rc);
 #ifndef NODIRENT
-extern void configwildlogs(void *opt, char *arg1, char *arg2, int rc);
+extern Filelist *clonelogs(Filelist *from, char *name);
+extern Filelist *expandwildlogs(Filelist *lp, Filelist **pter);
 #endif
 extern void configlogfile(void *opt, char *cmd, char *arg1, char *arg2,
   int rc);
 extern void configcachefile(void *opt, char *cmd, char *arg1, char *arg2,
   int rc);
+extern void configoutfile(void *opt, char *cmd, char *arg1, char *arg2,
+  int rc);
 extern void configchar(void *opt, char *cmd, char *arg1, char *arg2, int rc);
 extern void configbarstyle(void *opt, char *cmd, char *arg1, char *arg2,
   int rc);
@@ -228,7 +222,8 @@ extern choice parseuint2strict(FILE *f, void *arg, char c);
 extern choice parsescode(FILE *f, void *arg, char c);
 extern choice parseuint4strict(FILE *f, void *arg, char c);
 extern choice parseudint(FILE *f, void *arg, char c);
-extern choice parseyear(FILE *f, void *arg, char c);
+extern choice parseyear2(FILE *f, void *arg, char c);
+extern choice parseyear2or4(FILE *f, void *arg, char c);
 extern choice parseam(FILE *f, void *arg, char c);
 extern choice parsecode(FILE *f, void *arg, char c);
 extern choice parsejunk(FILE *f, void *arg, char c);
@@ -315,8 +310,7 @@ extern size_t htmlstrlen_utf8(char *s, choice outstyle);
 extern size_t htmlstrlen_jis(char *s, choice outstyle);
 extern void matchlength(FILE *outf, choice outstyle, char *s, char c);
 extern void myputc(FILE *outf, char c, choice outstyle);
-extern void htmlfprintf(FILE *outf, Outchoices *od, logical multibyte,
-  char *s, logical userinput);
+extern void htmlputs(FILE *outf, Outchoices *od, char *s, choice source);
 extern void latexfprintf(FILE *outf, char *s);
 extern void escfprintf(FILE *outf, char *name);
 extern void hrule(FILE *outf, Outchoices *od);
@@ -382,11 +376,16 @@ extern void checkreps(Outchoices *od, Dateman *dman, Hashindex **gooditems,
   Arraydata **arraydata, choice data2cols[ITEM_NUMBER][DATA_NUMBER]);
 #ifndef NOGRAPHICS
 extern FILE *piechart_init(char *filename);
+extern void findwedges(Wedge wedge[], choice rep, Hashindex *items,
+  choice chartby, Strlist *expandlist, unsigned int level, Strlist *partname,
+  unsigned long tot, double totb, double totb7);
 extern void piechart_caption(FILE *outf, choice rep, choice chartby,
   char **lngstr);
-extern void piechart_wedge(FILE *outf, Outchoices *od, double angle, char *s,
-  char *extension);
+extern int piechart_wedge(FILE *outf, Outchoices *od, double angle, char *s);
+extern void piechart_key(FILE *outf, Outchoices *od, int col, char *name,
+  char *extension, Alias *aliashead);
 extern void piechart_write(FILE *pieoutf, char *filename, logical jpegcharts);
+extern void piechart_cleanup(Wedge wedge[]);
 extern void piechart(FILE *outf, Outchoices *od, choice rep, Hashindex *items,
   choice requests, choice requests7, choice pages, choice pages7,
   unsigned long totr, unsigned long totr7, unsigned long totp,
@@ -577,6 +576,7 @@ extern logical genwildmatch(char *s, char *s2, char *p);
 extern logical headmatch(char *s, char *p);
 extern logical headcasematch(char *s, char *p);
 extern logical included(char *name, logical ispage, Include *listhead);
+extern logical incstrlist(char *name, Strlist *listhead);
 extern logical pageq(char *name, Include *ispagehead, choice type);
 
 /* in win32.c */
