@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -58,6 +58,7 @@ unsigned int *col2colhead, *method2sing, *method2pl, *method2date;
 unsigned int *method2pc, *method2relpc, *method2sort;
 logical *repistree;
 char *byteprefix = " kMGTPEZY";  /* Room for growth :)  Note initial space */
+/* NB Output versions of the byte prefixes are in the language files */
 
 /* Domain levels; making this global is messy but easiest */
 Strpair **domlevels;
@@ -105,7 +106,7 @@ char *methodname[] = {"requests","7-day requests", "pages", "7-day pages", "",
 		      "bytes", "7-day bytes", "alphabetical", "random"};
 /* Filenames for each language, in order of enum in anlghea3.h */
 char *country[] = {"am", /*"ba",*/ "bg", "bgm", "cat", /*"cn",*/ "tw",
-		   /*"hr",*/ "cz", "cz1", "dk", "nl", "uk", "us", /*"fi",*/
+		   /*"hr",*/ "cz", "cz1", "dk", "nl", "uk", "us", "fi",
 		   "fr", "de", /*"gr",*/ "hu", /*"is",*/ "it", "jpe", "jpj",
 		   "jps", "jpu", "kr", "lv", /*"lt",*/ "no", "no2", "pl", "pt",
 		   "br", /*"ro", "ru",*/ "yu", /*"sk",*/ "si", "si1", "es",
@@ -147,13 +148,8 @@ char *repname[] = {"Yearly Report", "Quarterly Report", "Monthly Report",
 		   "Operating System Report", "File Size Report",
 		   "Status Code Report", "Processing Time Report"};
 
-/* EBCDIC character set */
-/* This character set was contributed to analog by
-   Martin Kraemer (Martin.Kraemer@Fujitsu-Siemens.com) and
-   Holger Schranz (schranz@munich.crosswinds.net). It was originally
-   developed by the Apache Group for use in the Apache HTTP server project
-   (http://httpd.apache.org/).
-   Copyright (c) 1998-2001 The Apache Group.  All rights reserved. */
+/* EBCDIC character set: see copyright notice in docs/Licapach.txt within the
+   analog distribution. */
 #ifdef EBCDIC
 /* EBCDIC-to-ASCII tables:
   These tables are bijective - there are no ambigous or duplicate characters.
@@ -315,7 +311,7 @@ logical convertchar[256] = {
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   /* 00 - 0F */
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,   /* 10 - 1F */
   1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,   /* 20 - 2F */
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0,   /* 30 - 3F */
+  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0,   /* 30 - 3F */
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,   /* 40 - 4F */
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,   /* 50 - 5F */
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,   /* 60 - 6F */
@@ -330,8 +326,8 @@ logical convertchar[256] = {
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};  /* F0 - FF */
 /* %nm escapes to convert in do_alias(r|f). We convert all ASCII printable
    characters (0x20 - 0x7E) with the exception of % (0x25) [to distinguish
-   %25ab from %ab]; and ? (0x3f), & (0x26) and = (0x3d) [to split query
-   strings up correctly]. Could argue for + (0x2b) in query strings, but
+   %25ab from %ab]; and ? (0x3f), & (0x26), ; (0x3b) and = (0x3d) [to split
+   query strings up correctly]. Could argue for + (0x2b) in query strings, but
    messes search reports up if we do this. */
 
 logical unprintable[256];
@@ -340,6 +336,9 @@ logical unprintable[256];
 
 Inputfns inpfns[] = { /* first the ITEM fns in any order */
   /* See also ITEMFNS_NUMBER in anlghea3.c */
+  /* Also, some letters, namely those with &parsestring and other codes that
+     take following separators (see list in strtoinfmt() in init.c), need to be
+     included in the list in report_logfmt() in settings.c. */
   {'v', ITEM_VHOST, &parsestring, &mm[ITEM_VHOST]},
   {'r', ITEM_FILE, &parsestring, &mm[ITEM_FILE]},
   {'u', ITEM_USER, &parsestring, &mm[ITEM_USER]},
@@ -356,8 +355,9 @@ Inputfns inpfns[] = { /* first the ITEM fns in any order */
   {'d', INP_DATE, &parseuint2, (void *)&date},
   {'m', INP_MONTH, &parsenmonth, (void *)&month},
   {'M', INP_MONTH, &parsemonth, (void *)&month},
-  {'y', INP_YEAR, &parseyear, (void *)&year},
+  {'y', INP_YEAR, &parseyear2, (void *)&year},
   {'Y', INP_YEAR, &parseuint4strict, (void *)&year},
+  {'Z', INP_YEAR, &parseyear2or4, (void *)&year},
   {'U', INP_UNIXTIME, &parseulong, (void *)&unixtime},
   {'T', INP_PROCTIME, &parseulong, (void *)&proctime},
   {'t', INP_PROCTIME, &parseproctime, (void *)&proctime},
@@ -459,61 +459,89 @@ Configfns cf[] = {
   {"ALLGRAPH", &configallgraph, (void *)(opts.outopts.graph)},
   {"ALLGRAPHS", &configallgraph, (void *)(opts.outopts.graph)},
 #ifndef NOGRAPHICS
-  {"REQCHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_REQ)])},
-  {"REDIRCHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_REDIR)])},
-  {"FAILCHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_FAIL)])},
-  {"TYPECHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_TYPE)])},
-  {"SIZECHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_SIZE)])},
-  {"PROCTIMECHART", &configchart,
+  {"REQCHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_REQ)])},
+  {"REDIRCHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_REDIR)])},
+  {"FAILCHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_FAIL)])},
+  {"TYPECHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_TYPE)])},
+  {"SIZECHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_SIZE)])},
+  {"PROCTIMECHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_PROCTIME)])},
-  {"DIRCHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_DIR)])},
-  {"HOSTCHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_HOST)])},
-  {"REDIRHOSTCHART", &configchart,
+  {"DIRCHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_DIR)])},
+  {"HOSTCHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_HOST)])},
+  {"REDIRHOSTCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_REDIRHOST)])},
-  {"FAILHOSTCHART", &configchart,
+  {"FAILHOSTCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_FAILHOST)])},
-  {"DOMCHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_DOM)])},
-  {"ORGCHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_ORG)])},
-  {"REFCHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_REF)])},
-  {"REFSITECHART", &configchart,
+  {"DOMCHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_DOM)])},
+  {"ORGCHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_ORG)])},
+  {"REFCHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_REF)])},
+  {"REFSITECHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_REFSITE)])},
-  {"REDIRREFCHART", &configchart,
+  {"REDIRREFCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_REDIRREF)])},
-  {"FAILREFCHART", &configchart,
+  {"FAILREFCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_FAILREF)])},
-  {"BROWREPCHART", &configchart,
+  {"BROWREPCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_BROWREP)])},
-  {"BROWSUMCHART", &configchart,
+  {"BROWSUMCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_BROWSUM)])},
-  {"OSCHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_OS)])},
-  {"VHOSTCHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_VHOST)])},
-  {"REDIRVHOSTCHART", &configchart,
+  {"OSCHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_OS)])},
+  {"VHOSTCHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_VHOST)])},
+  {"REDIRVHOSTCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_REDIRVHOST)])},
-  {"FAILVHOSTCHART", &configchart,
+  {"FAILVHOSTCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_FAILVHOST)])},
-  {"USERCHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_USER)])},
-  {"REDIRUSERCHART", &configchart,
+  {"USERCHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_USER)])},
+  {"REDIRUSERCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_REDIRUSER)])},
-  {"FAILUSERCHART", &configchart,
+  {"FAILUSERCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_FAILUSER)])},
-  {"SEARCHQUERYCHART", &configchart,
+  {"SEARCHQUERYCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_SEARCHREP)])},
-  {"SEARCHWORDCHART", &configchart,
+  {"SEARCHWORDCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_SEARCHSUM)])},
-  {"INTSEARCHQUERYCHART", &configchart,
+  {"INTSEARCHQUERYCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_INTSEARCHREP)])},
-  {"INTSEARCHWORDCHART", &configchart,
+  {"INTSEARCHWORDCHART", &configchoice,
    (void *)&(opts.outopts.chartby[G(REP_INTSEARCHSUM)])},
-  {"STATUSCHART", &configchart, (void *)&(opts.outopts.chartby[G(REP_CODE)])},
+  {"STATUSCHART", &configchoice, (void *)&(opts.outopts.chartby[G(REP_CODE)])},
   {"ALLCHART", &configallchart, (void *)(opts.outopts.chartby)},
   {"ALLCHARTS", &configallchart, (void *)(opts.outopts.chartby)},
+  {"REQCHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_REQ)])},
+  {"REDIRCHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_REDIR)])},
+  {"FAILCHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_FAIL)])},
+  {"TYPECHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_TYPE)])},
+  {"DIRCHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_DIR)])},
+  {"DOMCHARTEXPAND", &configstrlist,  /* NB special case in configstrlist */
+   (void *)&(opts.outopts.expandhead[G(REP_DOM)])},
+  {"ORGCHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_ORG)])},
+  {"REFCHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_REF)])},
+  {"REFSITECHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_REFSITE)])},
+  {"REDIRREFCHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_REDIRREF)])},
+  {"FAILREFCHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_FAILREF)])},
+  {"BROWSUMCHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_BROWSUM)])},
+  {"BROWCHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_BROWSUM)])},
+  {"OSCHARTEXPAND", &configstrlist,
+   (void *)&(opts.outopts.expandhead[G(REP_OS)])},
   {"CHARTDIR", &configstr, (void *)&(opts.outopts.chartdir)},
   {"LOCALCHARTDIR", &configstr, (void *)&(opts.outopts.localchartdir)},
 #ifdef HAVE_GD
-  {"JPEGCHARTS", &configonoff, (void *)&(opts.outopts.jpegcharts)},
+  {"JPEGCHARTS", &configchoice, (void *)&(opts.outopts.jpegcharts)},
 #endif
 #endif  /* NOGRAPHICS */
-  {"PNGIMAGES", &configonoff, (void *)&(opts.outopts.pngimages)},
+  {"PNGIMAGES", &configchoice, (void *)&(opts.outopts.pngimages)},
   {"BARSTYLE", &configbarstyle, (void *)&(opts.outopts.barstyle)},
   {"COMPSEP", &configstr, (void *)&(opts.outopts.compsep)},
   {"PRESEP", &configstr, (void *)&(opts.outopts.compsep)},
@@ -535,7 +563,7 @@ Configfns cf[] = {
   {"FOOTERFILE", &configstr, (void *)&(opts.outopts.footerfile)},
   {"LANGUAGE", &configlang, (void *)&(opts.outopts.lang)},
   {"LANGFILE", &configstr, (void *)&(opts.outopts.lang.file)},
-  {"OUTFILE", &configstr, (void *)&(opts.outopts.outfile)},
+  {"OUTFILE", &configoutfile, (void *)&(opts.outopts.outfile)},
   {"CACHEOUTFILE", &configstr, (void *)&(opts.outopts.cacheoutfile)},
   {"ERRFILE", &configerrfile, (void *)&errfile},
   {"DOMAINSFILE", &configstr, (void *)&(opts.outopts.domainsfile)},
@@ -574,6 +602,8 @@ Configfns cf[] = {
    (void *)&(opts.outopts.floor[G(REP_REFSITE)])},
   {"REFDIRFLOOR", &configfloor,
    (void *)&(opts.outopts.subfloor[G(REP_REFSITE)])},
+  {"SUBREFSITEFLOOR", &configfloor,
+   (void *)&(opts.outopts.subfloor[G(REP_REFSITE)])},
   {"REDIRREFFLOOR", &configfloor,
    (void *)&(opts.outopts.floor[G(REP_REDIRREF)])},
   {"REDIRREFARGSFLOOR", &configfloor,
@@ -722,6 +752,7 @@ Configfns cf[] = {
   {"SUBDIR", &configtree, (void *)&(opts.outopts.tree[G(REP_DIR)])},
   {"SUBTYPE", &configtree, (void *)&(opts.outopts.tree[G(REP_TYPE)])},
   {"REFDIR", &configtree, (void *)&(opts.outopts.tree[G(REP_REFSITE)])},
+  {"SUBREFSITE", &configtree, (void *)&(opts.outopts.tree[G(REP_REFSITE)])},
   {"SUBBROW", &configtree, (void *)&(opts.outopts.tree[G(REP_BROWSUM)])},
   {"SUBOS", &configtree, (void *)&(opts.outopts.tree[G(REP_OS)])},
   {"SUBORG", &configstrlist, (void *)&(opts.outopts.suborgs)},
@@ -741,7 +772,7 @@ Configfns cf[] = {
   {"VHOSTEXCLUDE", &configexc, (void *)&(opts.wanthead[ITEM_VHOST])},
   {"STATUSINCLUDE", &configscinc, (void *)(opts.code2type)},
   {"STATUSEXCLUDE", &configscexc, (void *)(opts.code2type)},
-  {"304ISSUCCESS", &configonoff, (void *)&(opts.succ304)},
+  {"304ISSUCCESS", &configchoice, (void *)&(opts.succ304)},
   {"REQINCLUDE", &configinc, (void *)&(opts.outopts.wanthead[G(REP_REQ)])},
   {"REQEXCLUDE", &configexc, (void *)&(opts.outopts.wanthead[G(REP_REQ)])},
   {"REDIRINCLUDE", &configinc, (void *)&(opts.outopts.wanthead[G(REP_REDIR)])},
@@ -862,169 +893,173 @@ Configfns cf[] = {
   {"PAGEEXCLUDE", &configexc, (void *)&(opts.ispagehead)},
   {"ROBOTINCLUDE", &configinc, (void *)&(opts.robots)},
   {"ROBOTEXCLUDE", &configexc, (void *)&(opts.robots)},
-  {"REQSORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_REQ)])},
-  {"REQARGSSORTBY", &configsortby,
+  {"REQSORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_REQ)])},
+  {"REQARGSSORTBY", &configchoice,
    (void *)&(opts.outopts.subsortby[G(REP_REQ)])},
-  {"REDIRSORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_REDIR)])},
-  {"REDIRARGSSORTBY", &configsortby,
+  {"REDIRSORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_REDIR)])},
+  {"REDIRARGSSORTBY", &configchoice,
    (void *)&(opts.outopts.subsortby[G(REP_REDIR)])},
-  {"FAILSORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_FAIL)])},
-  {"FAILARGSSORTBY", &configsortby,
+  {"FAILSORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_FAIL)])},
+  {"FAILARGSSORTBY", &configchoice,
    (void *)&(opts.outopts.subsortby[G(REP_FAIL)])},
-  {"TYPESORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_TYPE)])},
-  {"SUBTYPESORTBY", &configsortby,
+  {"TYPESORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_TYPE)])},
+  {"SUBTYPESORTBY", &configchoice,
    (void *)&(opts.outopts.subsortby[G(REP_TYPE)])},
-  {"DIRSORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_DIR)])},
-  {"SUBDIRSORTBY", &configsortby,
+  {"DIRSORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_DIR)])},
+  {"SUBDIRSORTBY", &configchoice,
    (void *)&(opts.outopts.subsortby[G(REP_DIR)])},
-  {"HOSTSORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_HOST)])},
-  {"REDIRHOSTSORTBY", &configsortby,
+  {"HOSTSORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_HOST)])},
+  {"REDIRHOSTSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_REDIRHOST)])},
-  {"FAILHOSTSORTBY", &configsortby,
+  {"FAILHOSTSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_FAILHOST)])},
-  {"DOMSORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_DOM)])},
-  {"SUBDOMSORTBY", &configsortby,
+  {"DOMSORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_DOM)])},
+  {"SUBDOMSORTBY", &configchoice,
    (void *)&(opts.outopts.subsortby[G(REP_DOM)])},
-  {"ORGSORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_ORG)])},
-  {"SUBORGSORTBY", &configsortby,
+  {"ORGSORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_ORG)])},
+  {"SUBORGSORTBY", &configchoice,
    (void *)&(opts.outopts.subsortby[G(REP_ORG)])},
-  {"REFSORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_REF)])},
-  {"REFARGSSORTBY", &configsortby,
+  {"REFSORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_REF)])},
+  {"REFARGSSORTBY", &configchoice,
    (void *)&(opts.outopts.subsortby[G(REP_REF)])},
-  {"REFSITESORTBY", &configsortby,
+  {"REFSITESORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_REFSITE)])},
-  {"REFDIRSORTBY", &configsortby,
+  {"REFDIRSORTBY", &configchoice,
    (void *)&(opts.outopts.subsortby[G(REP_REFSITE)])},
-  {"REDIRREFSORTBY", &configsortby,
+  {"SUBREFSITESORTBY", &configchoice,
+   (void *)&(opts.outopts.subsortby[G(REP_REFSITE)])},
+  {"REDIRREFSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_REDIRREF)])},
-  {"REDIRREFARGSSORTBY", &configsortby,
+  {"REDIRREFARGSSORTBY", &configchoice,
    (void *)&(opts.outopts.subsortby[G(REP_REDIRREF)])},
-  {"FAILREFSORTBY", &configsortby,
+  {"FAILREFSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_FAILREF)])},
-  {"FAILREFARGSSORTBY", &configsortby,
+  {"FAILREFARGSSORTBY", &configchoice,
    (void *)&(opts.outopts.subsortby[G(REP_FAILREF)])},
-  {"BROWREPSORTBY", &configsortby,
+  {"BROWREPSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_BROWREP)])},
-  {"FULLBROWSORTBY", &configsortby,
+  {"FULLBROWSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_BROWREP)])},
-  {"BROWSUMSORTBY", &configsortby,
+  {"BROWSUMSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_BROWSUM)])},
-  {"BROWSORTBY", &configsortby,
+  {"BROWSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_BROWSUM)])},
-  {"SUBBROWSORTBY", &configsortby,
+  {"SUBBROWSORTBY", &configchoice,
    (void *)&(opts.outopts.subsortby[G(REP_BROWSUM)])},
-  {"OSSORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_OS)])},
-  {"SUBOSSORTBY", &configsortby, (void *)&(opts.outopts.subsortby[G(REP_OS)])},
-  {"STATUSSORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_CODE)])},
-  {"VHOSTSORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_VHOST)])},
-  {"REDIRVHOSTSORTBY", &configsortby,
+  {"OSSORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_OS)])},
+  {"SUBOSSORTBY", &configchoice, (void *)&(opts.outopts.subsortby[G(REP_OS)])},
+  {"STATUSSORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_CODE)])},
+  {"VHOSTSORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_VHOST)])},
+  {"REDIRVHOSTSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_REDIRVHOST)])},
-  {"FAILVHOSTSORTBY", &configsortby,
+  {"FAILVHOSTSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_FAILVHOST)])},
-  {"USERSORTBY", &configsortby, (void *)&(opts.outopts.sortby[G(REP_USER)])},
-  {"REDIRUSERSORTBY", &configsortby,
+  {"USERSORTBY", &configchoice, (void *)&(opts.outopts.sortby[G(REP_USER)])},
+  {"REDIRUSERSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_REDIRUSER)])},
-  {"FAILUSERSORTBY", &configsortby,
+  {"FAILUSERSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_FAILUSER)])},
-  {"SEARCHQUERYSORTBY", &configsortby,
+  {"SEARCHQUERYSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_SEARCHREP)])},
-  {"SEARCHWORDSORTBY", &configsortby,
+  {"SEARCHWORDSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_SEARCHSUM)])},
-  {"INTSEARCHQUERYSORTBY", &configsortby,
+  {"INTSEARCHQUERYSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_INTSEARCHREP)])},
-  {"INTSEARCHWORDSORTBY", &configsortby,
+  {"INTSEARCHWORDSORTBY", &configchoice,
    (void *)&(opts.outopts.sortby[G(REP_INTSEARCHSUM)])},
   {"ALL", &configall, (void *)(opts.outopts.repq)},
   {"ALLBACK", &configallback, (void *)(opts.outopts.back)},
-  {"YEARLY", &configonoff, (void *)&(opts.outopts.repq[REP_YEAR])},
-  {"QUARTERLY", &configonoff, (void *)&(opts.outopts.repq[REP_QUARTERLY])},
-  {"MONTHLY", &configonoff, (void *)&(opts.outopts.repq[REP_MONTH])},
-  {"WEEKLY", &configonoff, (void *)&(opts.outopts.repq[REP_WEEK])},
-  {"DAILYREP", &configonoff, (void *)&(opts.outopts.repq[REP_DAYREP])},
-  {"FULLDAILY", &configonoff, (void *)&(opts.outopts.repq[REP_DAYREP])},
-  {"DAILYSUM", &configonoff, (void *)&(opts.outopts.repq[REP_DAYSUM])},
-  {"DAILY", &configonoff, (void *)&(opts.outopts.repq[REP_DAYSUM])},
-  {"HOURLYREP", &configonoff, (void *)&(opts.outopts.repq[REP_HOURREP])},
-  {"FULLHOURLY", &configonoff, (void *)&(opts.outopts.repq[REP_HOURREP])},
-  {"HOURLYSUM", &configonoff, (void *)&(opts.outopts.repq[REP_HOURSUM])},
-  {"HOURLY", &configonoff, (void *)&(opts.outopts.repq[REP_HOURSUM])},
-  {"WEEKHOUR", &configonoff, (void *)&(opts.outopts.repq[REP_WEEKHOUR])},
-  {"QUARTERREP", &configonoff, (void *)&(opts.outopts.repq[REP_QUARTERREP])},
-  {"QUARTER", &configonoff, (void *)&(opts.outopts.repq[REP_QUARTERREP])},
-  {"QUARTERSUM", &configonoff, (void *)&(opts.outopts.repq[REP_QUARTERSUM])},
-  {"FIVEREP", &configonoff, (void *)&(opts.outopts.repq[REP_FIVEREP])},
-  {"FIVE", &configonoff, (void *)&(opts.outopts.repq[REP_FIVEREP])},
-  {"FIVESUM", &configonoff, (void *)&(opts.outopts.repq[REP_FIVESUM])},
-  {"GENERAL", &configonoff, (void *)&(opts.outopts.repq[REP_GENSUM])},
-  {"LASTSEVEN", &configonoff, (void *)&(opts.outopts.last7)},
-  {"RUNTIME", &configonoff, (void *)&(opts.outopts.runtime)},
-  {"GOTOS", &configgotos, (void *)&(opts.outopts.gotos)},
-  {"REPORTSPAN", &configonoff, (void *)&(opts.outopts.repspan)},
+  {"YEARLY", &configchoice, (void *)&(opts.outopts.repq[REP_YEAR])},
+  {"QUARTERLY", &configchoice, (void *)&(opts.outopts.repq[REP_QUARTERLY])},
+  {"MONTHLY", &configchoice, (void *)&(opts.outopts.repq[REP_MONTH])},
+  {"WEEKLY", &configchoice, (void *)&(opts.outopts.repq[REP_WEEK])},
+  {"DAILYREP", &configchoice, (void *)&(opts.outopts.repq[REP_DAYREP])},
+  {"FULLDAILY", &configchoice, (void *)&(opts.outopts.repq[REP_DAYREP])},
+  {"DAILYSUM", &configchoice, (void *)&(opts.outopts.repq[REP_DAYSUM])},
+  {"DAILY", &configchoice, (void *)&(opts.outopts.repq[REP_DAYSUM])},
+  {"HOURLYREP", &configchoice, (void *)&(opts.outopts.repq[REP_HOURREP])},
+  {"FULLHOURLY", &configchoice, (void *)&(opts.outopts.repq[REP_HOURREP])},
+  {"HOURLYSUM", &configchoice, (void *)&(opts.outopts.repq[REP_HOURSUM])},
+  {"HOURLY", &configchoice, (void *)&(opts.outopts.repq[REP_HOURSUM])},
+  {"WEEKHOUR", &configchoice, (void *)&(opts.outopts.repq[REP_WEEKHOUR])},
+  {"QUARTERREP", &configchoice, (void *)&(opts.outopts.repq[REP_QUARTERREP])},
+  {"QUARTER", &configchoice, (void *)&(opts.outopts.repq[REP_QUARTERREP])},
+  {"QUARTERSUM", &configchoice, (void *)&(opts.outopts.repq[REP_QUARTERSUM])},
+  {"FIVEREP", &configchoice, (void *)&(opts.outopts.repq[REP_FIVEREP])},
+  {"FIVE", &configchoice, (void *)&(opts.outopts.repq[REP_FIVEREP])},
+  {"FIVESUM", &configchoice, (void *)&(opts.outopts.repq[REP_FIVESUM])},
+  {"GENERAL", &configchoice, (void *)&(opts.outopts.repq[REP_GENSUM])},
+  {"LASTSEVEN", &configchoice, (void *)&(opts.outopts.last7)},
+  {"RUNTIME", &configchoice, (void *)&(opts.outopts.runtime)},
+  {"GOTOS", &configchoice, (void *)&(opts.outopts.gotos)},
+  {"REPORTSPAN", &configchoice, (void *)&(opts.outopts.repspan)},
   {"REPORTSPANTHRESHOLD", &configuint, (void *)&(opts.outopts.rsthresh)},
   {"RSTHRESH", &configuint, (void *)&(opts.outopts.rsthresh)},
-  {"DESCRIPTIONS", &configonoff, (void *)&(opts.outopts.descriptions)},
-  {"REQUEST", &configonoff, (void *)&(opts.outopts.repq[REP_REQ])},
-  {"REDIR", &configonoff, (void *)&(opts.outopts.repq[REP_REDIR])},
-  {"REDIRECTION", &configonoff, (void *)&(opts.outopts.repq[REP_REDIR])},
-  {"FAILURE", &configonoff, (void *)&(opts.outopts.repq[REP_FAIL])},
-  {"FILETYPE", &configonoff, (void *)&(opts.outopts.repq[REP_TYPE])},
-  {"SIZE", &configonoff, (void *)&(opts.outopts.repq[REP_SIZE])},
-  {"PROCTIME", &configonoff, (void *)&(opts.outopts.repq[REP_PROCTIME])},
-  {"DIRECTORY", &configonoff, (void *)&(opts.outopts.repq[REP_DIR])},
-  {"HOST", &configonoff, (void *)&(opts.outopts.repq[REP_HOST])},
-  {"REDIRHOST", &configonoff, (void *)&(opts.outopts.repq[REP_REDIRHOST])},
-  {"FAILHOST", &configonoff, (void *)&(opts.outopts.repq[REP_FAILHOST])},
-  {"DOMAIN", &configonoff, (void *)&(opts.outopts.repq[REP_DOM])},
-  {"ORGANISATION", &configonoff, (void *)&(opts.outopts.repq[REP_ORG])},
-  {"ORGANIZATION", &configonoff, (void *)&(opts.outopts.repq[REP_ORG])},
-  {"REFERRER", &configonoff, (void *)&(opts.outopts.repq[REP_REF])},
-  {"REFERER", &configonoff, (void *)&(opts.outopts.repq[REP_REF])},
-  {"REFSITE", &configonoff, (void *)&(opts.outopts.repq[REP_REFSITE])},
-  {"REDIRREF", &configonoff, (void *)&(opts.outopts.repq[REP_REDIRREF])},
-  {"FAILREF", &configonoff, (void *)&(opts.outopts.repq[REP_FAILREF])},
-  {"BROWSERREP", &configonoff, (void *)&(opts.outopts.repq[REP_BROWREP])},
-  {"BROWREP", &configonoff, (void *)&(opts.outopts.repq[REP_BROWREP])},
-  {"FULLBROWSER", &configonoff, (void *)&(opts.outopts.repq[REP_BROWREP])},
-  {"BROWSERSUM", &configonoff, (void *)&(opts.outopts.repq[REP_BROWSUM])},
-  {"BROWSUM", &configonoff, (void *)&(opts.outopts.repq[REP_BROWSUM])},
-  {"BROWSER", &configonoff, (void *)&(opts.outopts.repq[REP_BROWSUM])},
-  {"OSREP", &configonoff, (void *)&(opts.outopts.repq[REP_OS])},
-  {"VHOST", &configonoff, (void *)&(opts.outopts.repq[REP_VHOST])},
-  {"REDIRVHOST", &configonoff, (void *)&(opts.outopts.repq[REP_REDIRVHOST])},
-  {"FAILVHOST", &configonoff, (void *)&(opts.outopts.repq[REP_FAILVHOST])},
-  {"USER", &configonoff, (void *)&(opts.outopts.repq[REP_USER])},
-  {"REDIRUSER", &configonoff, (void *)&(opts.outopts.repq[REP_REDIRUSER])},
-  {"FAILUSER", &configonoff, (void *)&(opts.outopts.repq[REP_FAILUSER])},
-  {"SEARCHQUERY", &configonoff, (void *)&(opts.outopts.repq[REP_SEARCHREP])},
-  {"SEARCHWORD", &configonoff, (void *)&(opts.outopts.repq[REP_SEARCHSUM])},
-  {"INTSEARCHQUERY", &configonoff,
+  {"DESCRIPTIONS", &configchoice, (void *)&(opts.outopts.descriptions)},
+  {"REQUEST", &configchoice, (void *)&(opts.outopts.repq[REP_REQ])},
+  {"REDIR", &configchoice, (void *)&(opts.outopts.repq[REP_REDIR])},
+  {"REDIRECTION", &configchoice, (void *)&(opts.outopts.repq[REP_REDIR])},
+  {"FAILURE", &configchoice, (void *)&(opts.outopts.repq[REP_FAIL])},
+  {"FILETYPE", &configchoice, (void *)&(opts.outopts.repq[REP_TYPE])},
+  {"SIZE", &configchoice, (void *)&(opts.outopts.repq[REP_SIZE])},
+  {"PROCTIME", &configchoice, (void *)&(opts.outopts.repq[REP_PROCTIME])},
+  {"DIRECTORY", &configchoice, (void *)&(opts.outopts.repq[REP_DIR])},
+  {"HOST", &configchoice, (void *)&(opts.outopts.repq[REP_HOST])},
+  {"REDIRHOST", &configchoice, (void *)&(opts.outopts.repq[REP_REDIRHOST])},
+  {"FAILHOST", &configchoice, (void *)&(opts.outopts.repq[REP_FAILHOST])},
+  {"DOMAIN", &configchoice, (void *)&(opts.outopts.repq[REP_DOM])},
+  {"ORGANISATION", &configchoice, (void *)&(opts.outopts.repq[REP_ORG])},
+  {"ORGANIZATION", &configchoice, (void *)&(opts.outopts.repq[REP_ORG])},
+  {"REFERRER", &configchoice, (void *)&(opts.outopts.repq[REP_REF])},
+  {"REFERER", &configchoice, (void *)&(opts.outopts.repq[REP_REF])},
+  {"REFSITE", &configchoice, (void *)&(opts.outopts.repq[REP_REFSITE])},
+  {"REDIRREF", &configchoice, (void *)&(opts.outopts.repq[REP_REDIRREF])},
+  {"FAILREF", &configchoice, (void *)&(opts.outopts.repq[REP_FAILREF])},
+  {"BROWSERREP", &configchoice, (void *)&(opts.outopts.repq[REP_BROWREP])},
+  {"BROWREP", &configchoice, (void *)&(opts.outopts.repq[REP_BROWREP])},
+  {"FULLBROWSER", &configchoice, (void *)&(opts.outopts.repq[REP_BROWREP])},
+  {"BROWSERSUM", &configchoice, (void *)&(opts.outopts.repq[REP_BROWSUM])},
+  {"BROWSUM", &configchoice, (void *)&(opts.outopts.repq[REP_BROWSUM])},
+  {"BROWSER", &configchoice, (void *)&(opts.outopts.repq[REP_BROWSUM])},
+  {"OSREP", &configchoice, (void *)&(opts.outopts.repq[REP_OS])},
+  {"VHOST", &configchoice, (void *)&(opts.outopts.repq[REP_VHOST])},
+  {"REDIRVHOST", &configchoice, (void *)&(opts.outopts.repq[REP_REDIRVHOST])},
+  {"FAILVHOST", &configchoice, (void *)&(opts.outopts.repq[REP_FAILVHOST])},
+  {"USER", &configchoice, (void *)&(opts.outopts.repq[REP_USER])},
+  {"REDIRUSER", &configchoice, (void *)&(opts.outopts.repq[REP_REDIRUSER])},
+  {"FAILUSER", &configchoice, (void *)&(opts.outopts.repq[REP_FAILUSER])},
+  {"SEARCHQUERY", &configchoice, (void *)&(opts.outopts.repq[REP_SEARCHREP])},
+  {"SEARCHWORD", &configchoice, (void *)&(opts.outopts.repq[REP_SEARCHSUM])},
+  {"INTSEARCHQUERY", &configchoice,
    (void *)&(opts.outopts.repq[REP_INTSEARCHREP])},
-  {"INTSEARCHWORD", &configonoff,
+  {"INTSEARCHWORD", &configchoice,
    (void *)&(opts.outopts.repq[REP_INTSEARCHSUM])},
-  {"STATUS", &configonoff, (void *)&(opts.outopts.repq[REP_CODE])},
-  {"YEARBACK", &configonoff, (void *)&(opts.outopts.back[REP_YEAR])},
-  {"QUARTERLYBACK", &configonoff, (void *)&(opts.outopts.back[REP_QUARTERLY])},
-  {"MONTHBACK", &configonoff, (void *)&(opts.outopts.back[REP_MONTH])},
-  {"WEEKBACK", &configonoff, (void *)&(opts.outopts.back[REP_WEEK])},
-  {"DAYBACK", &configonoff, (void *)&(opts.outopts.back[REP_DAYREP])},
-  {"DAYREPBACK", &configonoff, (void *)&(opts.outopts.back[REP_DAYREP])},
-  {"FULLDAYBACK", &configonoff, (void *)&(opts.outopts.back[REP_DAYREP])},
-  {"HOURBACK", &configonoff, (void *)&(opts.outopts.back[REP_HOURREP])},
-  {"HOURREPBACK", &configonoff, (void *)&(opts.outopts.back[REP_HOURREP])},
-  {"FULLHOURBACK", &configonoff, (void *)&(opts.outopts.back[REP_HOURREP])},
-  {"QUARTERBACK", &configonoff, (void *)&(opts.outopts.back[REP_QUARTERREP])},
-  {"QUARTERREPBACK", &configonoff,
+  {"STATUS", &configchoice, (void *)&(opts.outopts.repq[REP_CODE])},
+  {"YEARBACK", &configchoice, (void *)&(opts.outopts.back[REP_YEAR])},
+  {"QUARTERLYBACK", &configchoice,
+   (void *)&(opts.outopts.back[REP_QUARTERLY])},
+  {"MONTHBACK", &configchoice, (void *)&(opts.outopts.back[REP_MONTH])},
+  {"WEEKBACK", &configchoice, (void *)&(opts.outopts.back[REP_WEEK])},
+  {"DAYBACK", &configchoice, (void *)&(opts.outopts.back[REP_DAYREP])},
+  {"DAYREPBACK", &configchoice, (void *)&(opts.outopts.back[REP_DAYREP])},
+  {"FULLDAYBACK", &configchoice, (void *)&(opts.outopts.back[REP_DAYREP])},
+  {"HOURBACK", &configchoice, (void *)&(opts.outopts.back[REP_HOURREP])},
+  {"HOURREPBACK", &configchoice, (void *)&(opts.outopts.back[REP_HOURREP])},
+  {"FULLHOURBACK", &configchoice, (void *)&(opts.outopts.back[REP_HOURREP])},
+  {"QUARTERBACK", &configchoice, (void *)&(opts.outopts.back[REP_QUARTERREP])},
+  {"QUARTERREPBACK", &configchoice,
    (void *)&(opts.outopts.back[REP_QUARTERREP])},
-  {"FIVEBACK", &configonoff, (void *)&(opts.outopts.back[REP_FIVEREP])},
-  {"FIVEREPBACK", &configonoff, (void *)&(opts.outopts.back[REP_FIVEREP])},
-  {"RAWBYTES", &configonoff, (void *)&(opts.outopts.rawbytes)},
-  {"NOROBOTS", &configonoff, (void *)&(opts.outopts.norobots)},
-  {"SETTINGS", &configonoff, (void *)&vblesonly},
-  {"CGI", &configonoff, (void *)&cgi},
-  {"OUTPUT", &configoutstyle, (void *)&(opts.outopts.outstyle)},
-  {"CASE", &configcase, (void *)&(opts.miscopts.case_insensitive)},
-  {"USERCASE", &configcase, (void *)&(opts.miscopts.usercase_insensitive)},
-  {"SEARCHCHARCONVERT", &configonoff, (void *)&(opts.outopts.searchconv)},
+  {"FIVEBACK", &configchoice, (void *)&(opts.outopts.back[REP_FIVEREP])},
+  {"FIVEREPBACK", &configchoice, (void *)&(opts.outopts.back[REP_FIVEREP])},
+  {"RAWBYTES", &configchoice, (void *)&(opts.outopts.rawbytes)},
+  {"NOROBOTS", &configchoice, (void *)&(opts.outopts.norobots)},
+  {"SETTINGS", &configchoice, (void *)&vblesonly},
+  {"CGI", &configchoice, (void *)&cgi},
+  {"OUTPUT", &configchoice, (void *)&(opts.outopts.outstyle)},
+  {"PDFLATEX", &configchoice, (void *)&(opts.outopts.pdflatex)},
+  {"CASE", &configchoice, (void *)&(opts.miscopts.case_insensitive)},
+  {"USERCASE", &configchoice, (void *)&(opts.miscopts.usercase_insensitive)},
+  {"SEARCHCHARCONVERT", &configchoice, (void *)&(opts.outopts.searchconv)},
   {"YEARROWS", &configuint, (void *)&(opts.outopts.rows[REP_YEAR])},
   {"QUARTERLYROWS", &configuint, (void *)&(opts.outopts.rows[REP_QUARTERLY])},
   {"MONTHROWS", &configuint, (void *)&(opts.outopts.rows[REP_MONTH])},
@@ -1053,7 +1088,7 @@ Configfns cf[] = {
   {"MINGRAPHWIDTH", &configuint, (void *)&(opts.outopts.mingraphwidth)},
   {"ERRLINELENGTH", &configuint, (void *)&errwidth},
   {"PROGRESSFREQ", &configulong, (void *)&progressfreq},
-  {"WEEKBEGINSON", &configweekday, (void *)&(opts.outopts.weekbeginson)},
+  {"WEEKBEGINSON", &configchoice, (void *)&(opts.outopts.weekbeginson)},
   {"FROM", &configstr, (void *)&(opts.dman.fromstr)},
   {"TO", &configstr, (void *)&(opts.dman.tostr)},
   {"REPORTORDER", &configrepord, (void *)(opts.outopts.reporder)},
@@ -1062,7 +1097,7 @@ Configfns cf[] = {
   {"GENSUMLINES", &configdebug, (void *)&(opts.outopts.gensumlines)},
   {"DOMLEVEL", &configdomlevel, (void *)&domlevels},
 #ifndef NODNS
-  {"DNS", &configdns, (void *)&dnslevel},
+  {"DNS", &configchoice, (void *)&dnslevel},
   {"DNSFILE", &configstr, (void *)&dnsfile},
   {"DNSLOCKFILE", &configstr, (void *)&dnslockfile},
   {"DNSGOODHOURS", &configuint, (void *)&dnsgoodhrs},
@@ -1099,6 +1134,9 @@ void globals(char *comname) {
 #endif
 #ifdef SIGQUIT
   signal(SIGQUIT, sighandler);
+#endif
+#ifdef SIGPIPE
+  signal(SIGPIPE, SIG_IGN);
 #endif
   workspace = (char *)xmalloc(BLOCKSIZE);
   block_start = (char *)xmalloc(BLOCKSIZE + 1);  /* 1 for terminating char */
@@ -1128,9 +1166,9 @@ void globals(char *comname) {
   termchar[(unsigned char)'\0'] = TRUE;
 
   if (IS_EMPTY_STRING(comname))
-    DEFAULTSTR(commandname, "analog")  /* no semicolon by defn of DEFAULTSTR */
+    COPYSTR(commandname, "analog")  /* no semicolon by defn of COPYSTR */
   else
-    DEFAULTSTR(commandname, comname);
+    COPYSTR(commandname, comname);
 #ifndef NOFOLLOW
   commandpath = (char *)xmalloc(PATH_MAX);
   done = FALSE;
@@ -1493,13 +1531,13 @@ void defaults(Options *op) {
   confline(op, "CACHEFILE", CACHEFILE, NULL, -1);
   op->outopts.domainsfile = NULL;
   op->outopts.descfile = NULL;
-  DEFAULTSTR(op->miscopts.dirsuffix, DIRSUFFIX);
+  COPYSTR(op->miscopts.dirsuffix, DIRSUFFIX);
 #ifndef NODNS
   dnslevel = DNS;
   dnsgoodhrs = DNSGOODHOURS;
   dnsbadhrs = DNSBADHOURS;
-  DEFAULTSTR(dnsfile, DNSFILE);
-  DEFAULTSTR(dnslockfile, DNSLOCKFILE);
+  COPYSTR(dnsfile, DNSFILE);
+  COPYSTR(dnslockfile, DNSLOCKFILE);
 #ifndef NOALARM
   dnstimeout = DNSTIMEOUT;
 #endif
@@ -1509,12 +1547,13 @@ void defaults(Options *op) {
   op->outopts.searchconv = SEARCHCHARCONVERT;
   op->outopts.outstyle = OUTPUT;
   op->outopts.html = TRUE;
+  op->outopts.pdflatex = TRUE;
   htmlstrlen = &htmlstrlen_normal;
-  DEFAULTSTR(op->outopts.headerfile, HEADERFILE);
-  DEFAULTSTR(op->outopts.footerfile, FOOTERFILE);
-  DEFAULTSTR(op->outopts.outfile, OUTFILE);
-  DEFAULTSTR(op->outopts.cacheoutfile, CACHEOUTFILE);
-  DEFAULTSTR(op->outopts.imagedir, IMAGEDIR);
+  COPYSTR(op->outopts.headerfile, HEADERFILE);
+  COPYSTR(op->outopts.footerfile, FOOTERFILE);
+  COPYSTR(op->outopts.outfile, OUTFILE);
+  COPYSTR(op->outopts.cacheoutfile, CACHEOUTFILE);
+  COPYSTR(op->outopts.imagedir, IMAGEDIR);
   op->outopts.chartdir = NULL;
   op->outopts.localchartdir = NULL;
   op->outopts.jpegcharts = FALSE;
@@ -1774,11 +1813,11 @@ void defaults(Options *op) {
   op->outopts.markchar = MARKCHAR;
   op->outopts.rawbytes = RAWBYTES;
   op->outopts.norobots = NOROBOTS;
-  DEFAULTSTR(op->outopts.hostname, HOSTNAME);
-  DEFAULTSTR(op->outopts.hosturl, HOSTURL);
+  COPYSTR(op->outopts.hostname, HOSTNAME);
+  COPYSTR(op->outopts.hosturl, HOSTURL);
   op->outopts.baseurl = NULL;
-  DEFAULTSTR(op->outopts.logo, LOGO);
-  DEFAULTSTR(op->outopts.stylesheet, STYLESHEET);
+  COPYSTR(op->outopts.logo, LOGO);
+  COPYSTR(op->outopts.stylesheet, STYLESHEET);
   op->outopts.weekbeginson = WEEKBEGINSON;
   op->outopts.plainpagewidth = PLAINPAGEWIDTH;
   op->outopts.htmlpagewidth = HTMLPAGEWIDTH;
@@ -1787,11 +1826,12 @@ void defaults(Options *op) {
   op->outopts.sepchar = (char)UNSET;
   op->outopts.repsepchar = (char)UNSET;
   op->outopts.decpt = (char)UNSET;
-  DEFAULTSTR(op->outopts.compsep, COMPSEP);
+  COPYSTR(op->outopts.compsep, COMPSEP);
   for (i = 0; i < GENREP_NUMBER; i++) {
     op->outopts.link[i] = NULL;
     op->outopts.wanthead[i] = NULL;
     op->outopts.aliashead[i] = NULL;
+    op->outopts.expandhead[i] = NULL;
     op->outopts.tree[i] = (Tree *)xmalloc(sizeof(Tree));
     op->outopts.tree[i]->tree = NULL;
     op->outopts.tree[i]->space = (Memman *)xmalloc(sizeof(Memman));
