@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -28,19 +28,22 @@ void confline(Options *op, char *cmd, char *arg1, char *arg2, int rc) {
   extern Configfns cf[];
   extern char *pos;
 
-  char u[520];   /* see nextconfline(); why this high though? */
+  char u[265];   /* see nextconfline() and uses of u below */
   char *savepos;
   int i;
   logical done = FALSE;
 
   strtoupper(cmd);
-  if (rc >= 2 && STREQ(cmd, "SUBDOMAIN")) {   /* ugly but easy */
-    sprintf(u, "%s (%s)", arg1, arg2);
-    if (strchr(u, '$') != NULL || strchr(u, '*') != NULL)
-      warn('C', TRUE, "Can't have $ or * in argument to SUBDOMAIN");
-    else
+  if (rc >= 2 && (STREQ(cmd, "SUBDOMAIN") || STREQ(cmd, "SUBDOM"))) {
+    /* ugly but easy */
+    if (strchr(arg1, '*') != NULL)
+      warn('C', TRUE, "Can't have * in first of two arguments to SUBDOMAIN: "
+	   "ignoring second argument");
+    else {
+      sprintf(u, "PLAIN:%s (%s)", arg1, arg2);
       configalias((void *)&(op->outopts.aliashead[G(REP_DOM)]), "SUBDOMAIN",
 		  arg1, u, -1);
+    }
     rc--;  /* to avoid error message */
   }
   for (i = 0; cf[i].fn != NULL && !done; i++) {
@@ -52,7 +55,7 @@ void confline(Options *op, char *cmd, char *arg1, char *arg2, int rc) {
       if (cf[i].fn == &configcall && rc != '\0' && !IS_EMPTY_STRING(arg1)) {
 	savepos = pos;
 	strcpy(u, arg1); /* or name will get obliterated before fclose */
-	(void)config(u, op);
+	(void)config(u, op, FALSE);
 	pos = savepos;
       }
       done = TRUE;
@@ -62,7 +65,7 @@ void confline(Options *op, char *cmd, char *arg1, char *arg2, int rc) {
     unknownwarn(cmd, arg1, arg2);
 }
 
-choice config(char *filename, Options *op) {
+choice config(char *filename, Options *op, logical fromcommandline) {
   extern Inputformatlist *logformat;
   extern char *pos;
 
@@ -71,7 +74,7 @@ choice config(char *filename, Options *op) {
   char *fullname, *cmd, *arg1, *arg2;
   int rc;
 
-  if (IS_STDIN(filename))
+  if (fromcommandline || IS_STDIN(filename))
     fullname = filename;
   else
     fullname = buildfilename(CONFIGDIR, "", filename);
@@ -116,11 +119,11 @@ void settings(Options *op, int argc, char *argv[]) {
       if (!IS_EMPTY_STRING(argv[i]) && argv[i][1] == 'G' &&
 	  (argv[i][0] == '+' || argv[i][0] == '-')) {
 	done = TRUE;
-	CLLONGCHECK(if (argv[i][0] == '+') (void)config(DEFAULTCONFIGFILE, op);)
+	CLLONGCHECK(if (argv[i][0] == '+') (void)config(DEFAULTCONFIGFILE, op, FALSE);)
       }
     }
     if (!done)
-      (void)config(DEFAULTCONFIGFILE, op);
+      (void)config(DEFAULTCONFIGFILE, op, FALSE);
   }
 
   /* now read in rest of command line arguments */
@@ -133,7 +136,7 @@ void settings(Options *op, int argc, char *argv[]) {
   /* finally, read in mandatory config file, aborting if not found */
   newloglist = TRUE;
   if (!strcaseeq(MANDATORYCONFIGFILE, "none") &&
-      config(MANDATORYCONFIGFILE, op) == ERR)
+      config(MANDATORYCONFIGFILE, op, FALSE) == ERR)
     error("Cannot ignore mandatory configuration file");
 }
 
@@ -257,7 +260,7 @@ void correct(Options *op) {
   Filelist *lp;
   Include *incp, *lastincp;
   choice rep, *cols, floor, subfloor, sortby, subsortby, chartby;
-  char *subf, *subs;
+  char *subf, *subs, *c;
   logical colsinc[COL_NUMBER], istree, templ;
   char graph;
   int i, j, k;
@@ -583,6 +586,9 @@ void correct(Options *op) {
 	  if (sortby == REQUESTS || sortby == REQUESTS7 || sortby == PAGES ||
 	      sortby == PAGES7 || sortby == BYTES || sortby == BYTES7)
 	    od->chartby[G(rep)] = sortby;
+	  else if (floor == REQUESTS || floor == REQUESTS7 || floor == PAGES ||
+		   floor == PAGES7 || floor == BYTES || floor == BYTES7)
+	    od->chartby[G(rep)] = floor;
 	  else
 	    od->chartby[G(rep)] = REQUESTS;
 	  chartby = od->chartby[G(rep)];
@@ -623,11 +629,21 @@ void correct(Options *op) {
 	      (sortby == FIRSTDATE && floor == DATESORT))
 	    warn('D', TRUE, "In %s, SORTBY (%s) doesn't match FLOOR (%s)",
 		 repname[rep], methodname[sortby], methodname[floor]);
-	  if (chartby != CHART_NONE && sortby != chartby &&
-	      (sortby == REQUESTS || sortby == REQUESTS7 || sortby == PAGES ||
-	       sortby == PAGES7 || sortby == BYTES || sortby == BYTES7))
-	    warn('D', TRUE, "In %s, CHART (%s) doesn't match SORTBY (%s)",
-		 repname[rep], methodname[chartby], methodname[sortby]);
+	  if (chartby != CHART_NONE) {
+	    if (sortby == REQUESTS || sortby == REQUESTS7 || sortby == PAGES ||
+		sortby == PAGES7 || sortby == BYTES || sortby == BYTES7) {
+	      if (sortby != chartby) {
+		warn('D', TRUE, "In %s, CHART (%s) doesn't match SORTBY (%s)",
+		     repname[rep], methodname[chartby], methodname[sortby]);
+	      }
+	    }
+	    else if ((floor == REQUESTS || floor == REQUESTS7 ||
+		      floor == PAGES || floor == PAGES7 || floor == BYTES ||
+		      floor == BYTES7) && floor != chartby) {
+		warn('D', TRUE, "In %s, CHART (%s) doesn't match FLOOR (%s)",
+		     repname[rep], methodname[chartby], methodname[floor]);
+	    }
+	  }
 	  if (sortby == ALPHABETICAL && colsinc[COL_INDEX])
 	    warn('D', TRUE, "In %s, column N with SORTBY ALPHABETICAL",
 		 repname[rep]);
@@ -786,12 +802,16 @@ void correct(Options *op) {
 	lp->pvpos = UNSET;
       }
     }
-  }
+  }  /* end if (!vblesonly) */
   if (outstyle == COMPUTER) {
     od->sepchar = '\0';
     od->repsepchar = '\0';
     od->decpt = '.';
     od->rawbytes = TRUE;
+    while ((c = strstr(od->compsep, "\\t")) != NULL) {
+      *c = '\t';
+      memmove((void *)(c + 1), (void *)(c + 2), strlen(c + 2) + 1);
+    }
   }
   else if (od->repsepchar == (char)UNSET)
     od->repsepchar = '\0'; /* sepchar, decpt set from langfile below */
@@ -873,6 +893,7 @@ void finalinit(Options *op) {
   Outchoices *od = &(op->outopts);
   choice outstyle = od->outstyle;
 
+  Filelist *lp, *lastlp;
   logical wantcol[ITEM_NUMBER][DATA_NUMBER];
   choice *cols;
   char **lngstr = NULL;  /* just to keep compiler happy */
@@ -907,15 +928,34 @@ void finalinit(Options *op) {
     reversealias(&(op->aliashead[i]));
   for (i = 0; i < GENREP_NUMBER; i++)
     reversealias(&(od->aliashead[i]));
-  /* translate filenames to put files in the correct directories */
+  /* Translate filenames to put files in the correct directories */
+  /* Also interpret %date codes and wild cards where appropriate */
   if (!strcaseeq(od->headerfile, "none") && !IS_STDIN(od->headerfile))
     od->headerfile = buildfilename(HEADERDIR, "", od->headerfile);
   if (!strcaseeq(od->footerfile, "none") && !IS_STDIN(od->footerfile))
     od->footerfile = buildfilename(HEADERDIR, "", od->footerfile);
-  if (!STREQ(od->cacheoutfile, "none") && !IS_STDOUT(od->cacheoutfile))
-    od->cacheoutfile = buildfilename(CACHEDIR, "", od->cacheoutfile);
-  if (!IS_STDOUT(od->outfile))
-    od->outfile = buildfilename(OUTDIR, "", od->outfile);
+  if (!STREQ(od->cacheoutfile, "none") && !IS_STDOUT(od->cacheoutfile)) {
+    s = timesprintf(NULL, buildfilename(CACHEDIR, "", od->cacheoutfile),
+		    op->dman.last7to, TRUE);
+    COPYSTR(od->cacheoutfile, s);
+  }
+  if (!IS_STDOUT(od->outfile)) {
+    s = timesprintf(NULL, od->outfile, op->dman.last7to, TRUE);
+    /* outfile has already had buildfilename() in configoutfile() */
+    COPYSTR(od->outfile, s);
+  }
+  for (i = 0; i <= 1; i++) {
+    for (lp = op->miscopts.logfile[i], lastlp = NULL; lp != NULL;
+	 lastlp = lp, TO_NEXT(lp)) {
+      if (!IS_STDIN(lp->name)) {
+	s = timesprintf(NULL, lp->name, op->dman.last7to, TRUE);
+	COPYSTR(lp->name, s);
+	if (strchr(lp->name, '*') != NULL || strchr(lp->name, '?') != NULL)
+	  lp = expandwildlogs(lp, (lastlp == NULL)?\
+			      (&(op->miscopts.logfile[i])):(&(lastlp->next)));
+      }
+    }
+  }
 #ifndef NOGRAPHICS
   if (outstyle == HTML) {
     if (od->chartdir == NULL || od->localchartdir == NULL) {
@@ -942,9 +982,9 @@ void finalinit(Options *op) {
     /* convert any date codes in CHARTDIRs */
     if (od->chartdir != NULL && od->localchartdir != NULL) {
       s = timesprintf(NULL, od->localchartdir, op->dman.last7to, TRUE);
-      DEFAULTSTR(od->localchartdir, s);
+      COPYSTR(od->localchartdir, s);
       s = timesprintf(NULL, od->chartdir, op->dman.last7to, TRUE);
-      DEFAULTSTR(od->chartdir, s);
+      COPYSTR(od->chartdir, s);
     }
   }
 #endif
@@ -979,11 +1019,16 @@ void finalinit(Options *op) {
       od->lngstr = (char **)xmalloc(LNGSTR_NUMBER * sizeof(char *));
       lngstr = od->lngstr;  /* just for conciseness */
       pos = NULL;
-      DEFAULTSTR(lngstr[charset_], nextlngstr(tempf, od->lang.file, TRUE));
+      COPYSTR(lngstr[charset_], nextlngstr(tempf, od->lang.file, TRUE));
       if (outstyle == LATEX && !strcaseeq(lngstr[charset_], "US-ASCII") &&
-	  !strcaseeq(lngstr[charset_], "ISO-8859-1"))
-	error("OUTPUT LATEX only available with US-ASCII and ISO-8859-1 "
-	      "character sets");
+	  !(headcasematch(lngstr[charset_], "ISO-8859-") &&
+	    (lngstr[charset_][9] == '1' || lngstr[charset_][9] == '2') &&
+	    lngstr[charset_][10] == '\0'))
+	error("OUTPUT LATEX only available with US-ASCII, ISO-8859-1 and "
+	      "ISO-8859-2 character sets");
+      /* NB Latin-3 and -5 have LaTeX .def files too but I have no such
+	 language files, so no way to test them (and no need). Ditto some
+	 Windows codepages. */
       if (*(lngstr[charset_]) == '*') {
 	od->multibyte = TRUE;
 	(lngstr[charset_])++;
@@ -995,13 +1040,21 @@ void finalinit(Options *op) {
       else if (strcaseeq(lngstr[charset_], "ISO-2022-JP"))
 	htmlstrlen = &htmlstrlen_jis;
       for (i = 0; i < 7; i++)
-	DEFAULTSTR(od->dayname[i], nextlngstr(tempf, od->lang.file, TRUE));
+	COPYSTR(od->dayname[i], nextlngstr(tempf, od->lang.file, TRUE));
       for (i = 0; i < 12; i++)
-	DEFAULTSTR(od->monthname[i], nextlngstr(tempf, od->lang.file, TRUE));
+	COPYSTR(od->monthname[i], nextlngstr(tempf, od->lang.file, TRUE));
       for (i = 1; i < LNGSTR_NUMBER; i++)  /* charset_ was 0 */
-	DEFAULTSTR(lngstr[i], nextlngstr(tempf, od->lang.file, TRUE));
+	COPYSTR(lngstr[i], nextlngstr(tempf, od->lang.file, TRUE));
       (void)nextlngstr(tempf, od->lang.file, FALSE);
       /* This final nextlngstr() checks the language file isn't too long */
+      if (strchr(lngstr[xbytes_], '?') == NULL ||
+	  strchr(lngstr[nxbytes_], '?') == NULL ||
+	  strchr(lngstr[nxbytes7_], '?') == NULL ||
+	  strchr(lngstr[xbytestraffic_], '?') == NULL ||
+	  strchr(lngstr[xbytestraffic7_], '?') == NULL)
+	error("language file %s corrupt: translation of \"?bytes\" missing "
+	      "a question mark", od->lang.file);
+      /* Fatal error is harsh here: but it avoids all sorts of trouble later */
       od->plainmonthlen = arraymaxlen(od->monthname, 12, ASCII);
       /* plain is longer: by pretending to be ASCII it includes whole strlen */
       /* But note that even this is not enough for formatted dates: could need
@@ -1016,7 +1069,8 @@ void finalinit(Options *op) {
 	 convert A0-FF for ISO-8859-*, 80-FF o/wise. This may still include
 	 some non-printable characters! But we can't have a table for every
 	 charset. */
-      if (!od->searchconv || strcaseeq(lngstr[charset_], "US-ASCII"))
+      if (!od->searchconv || od->multibyte ||
+	  strcaseeq(lngstr[charset_], "US-ASCII"))
 	od->convfloor = 0;
       else if (substrcaseeq(lngstr[charset_], "iso-8859-") ||
 	       strcaseeq(lngstr[charset_], "armscii-8"))
@@ -1045,32 +1099,32 @@ void finalinit(Options *op) {
       if (od->baseurl != NULL && STREQ(od->baseurl, "none"))
 	od->baseurl = NULL;
       if (outstyle == COMPUTER) {
-	DEFAULTSTR(lngstr[succreqs_], "SRS7");
-	DEFAULTSTR(lngstr[totunknown_], "NCC7");
-	DEFAULTSTR(lngstr[totpages_], "PRP7");
-	DEFAULTSTR(lngstr[totfails_], "FLF7");
-	DEFAULTSTR(lngstr[totredirs_], "RRR7");
-	DEFAULTSTR(lngstr[inforeqs_], "NII7");
-	DEFAULTSTR(lngstr[distfiles_], "NFN7");
-	DEFAULTSTR(lngstr[disthosts_], "NHH7");
-	DEFAULTSTR(lngstr[corrupt_], "CL");
-	DEFAULTSTR(lngstr[unwanted_], "UL");
-	DEFAULTSTR(lngstr[totdata_], "BTB7");
-	DEFAULTSTR(lngstr[dayrepfmt_], "%Y%\b%M%\b%D");
-	DEFAULTSTR(lngstr[daysumfmt_], "%w");
-	DEFAULTSTR(lngstr[hourrepfmt_], "%Y%\b%M%\b%D%\b%H");
-	DEFAULTSTR(lngstr[hoursumfmt_], "%H");
-	DEFAULTSTR(lngstr[weekhoursumfmt_], "%w%\b%H");
-	DEFAULTSTR(lngstr[quarterrepfmt_], "%Y%\b%M%\b%D%\b%H%\b%n");
-	DEFAULTSTR(lngstr[quartersumfmt_], "%H%\b%n");
-	DEFAULTSTR(lngstr[weekfmt_], "%Y%\b%M%\b%D");
-	DEFAULTSTR(lngstr[monthfmt_], "%Y%\b%M");
-	DEFAULTSTR(lngstr[quarterlyfmt_], "%Y%\b%q");
-	DEFAULTSTR(lngstr[yearfmt_], "%Y");
-	DEFAULTSTR(lngstr[genrepdate_], "%Y%\b%M%\b%D");
-	DEFAULTSTR(lngstr[genreptime_], "%Y%\b%M%\b%D%\b%H%\b%n");
-	DEFAULTSTR(lngstr[datefmt1_], "%Y%\b%M%\b%D%\b%H%\b%n");
-	DEFAULTSTR(lngstr[datefmt2_], "%Y%\b%M%\b%D%\b%H%\b%n");
+	COPYSTR(lngstr[succreqs_], "SRS7");
+	COPYSTR(lngstr[totunknown_], "NCC7");
+	COPYSTR(lngstr[totpages_], "PRP7");
+	COPYSTR(lngstr[totfails_], "FLF7");
+	COPYSTR(lngstr[totredirs_], "RRR7");
+	COPYSTR(lngstr[inforeqs_], "NII7");
+	COPYSTR(lngstr[distfiles_], "NFN7");
+	COPYSTR(lngstr[disthosts_], "NHH7");
+	COPYSTR(lngstr[corrupt_], "CL");
+	COPYSTR(lngstr[unwanted_], "UL");
+	COPYSTR(lngstr[totdata_], "BTB7");
+	COPYSTR(lngstr[dayrepfmt_], "%Y%\b%M%\b%D");
+	COPYSTR(lngstr[daysumfmt_], "%w");
+	COPYSTR(lngstr[hourrepfmt_], "%Y%\b%M%\b%D%\b%H");
+	COPYSTR(lngstr[hoursumfmt_], "%H");
+	COPYSTR(lngstr[weekhoursumfmt_], "%w%\b%H");
+	COPYSTR(lngstr[quarterrepfmt_], "%Y%\b%M%\b%D%\b%H%\b%n");
+	COPYSTR(lngstr[quartersumfmt_], "%H%\b%n");
+	COPYSTR(lngstr[weekfmt_], "%Y%\b%M%\b%D");
+	COPYSTR(lngstr[monthfmt_], "%Y%\b%M");
+	COPYSTR(lngstr[quarterlyfmt_], "%Y%\b%q");
+	COPYSTR(lngstr[yearfmt_], "%Y");
+	COPYSTR(lngstr[genrepdate_], "%Y%\b%M%\b%D");
+	COPYSTR(lngstr[genreptime_], "%Y%\b%M%\b%D%\b%H%\b%n");
+	COPYSTR(lngstr[datefmt1_], "%Y%\b%M%\b%D%\b%H%\b%n");
+	COPYSTR(lngstr[datefmt2_], "%Y%\b%M%\b%D%\b%H%\b%n");
       }
       /* repsepchar done above. Also OUTPUT COMPUTER versions done above. */
       if (od->sepchar == (char)UNSET)
@@ -1221,21 +1275,20 @@ void finalinit(Options *op) {
     od->derv[REP_OS - FIRST_DERVREP]->arg = (void *)(op->robots);
     od->derv[REP_BROWSUM - FIRST_DERVREP]->arg = NULL;
     if (outstyle != OUT_NONE) {
-      ALIASP("DOMOUTPUTALIAS", "\f", lngstr[unresolved_]);
-      ALIASP("DOMOUTPUTALIAS", "\b", lngstr[nodomain_]);
-      ALIASP("DOMOUTPUTALIAS", "\v", lngstr[unkdomain_]);
-      ALIASP("ORGOUTPUTALIAS", "\f", lngstr[unresolved_]);
-      ALIASP("ORGOUTPUTALIAS", "\b", lngstr[nodomain_]);
-      ALIASP("ORGOUTPUTALIAS", "\v", lngstr[unkdomain_]);
-      ALIASP("DIROUTPUTALIAS", "/", lngstr[rootdir_]);
-      ALIASP("DIROUTPUTALIAS", "//", lngstr[nodir_]);
-      ALIASP("TYPEOUTPUTALIAS", ".", lngstr[noext_]);
-      ALIASP("TYPEOUTPUTALIAS", "./", lngstr[brkdirs_]);
-      ALIASP("OSOUTPUTALIAS", "unkwin", lngstr[unkwin_]);
-      ALIASP("OSOUTPUTALIAS", "unkmac", lngstr[unkmac_]);
-      ALIASP("OSOUTPUTALIAS", "unkux", lngstr[unkux_]);
-      ALIASP("OSOUTPUTALIAS", "unkos", lngstr[unkos_]);
-      ALIASP("OSOUTPUTALIAS", "\v", lngstr[robots_]);
+      ALIASP("DOMOUTPUTALIAS", LNGSTR_UNRESOLVED, lngstr[unresolved_]);
+      ALIASP("DOMOUTPUTALIAS", LNGSTR_NODOMAIN, lngstr[nodomain_]);
+      ALIASP("DOMOUTPUTALIAS", LNGSTR_UNKDOMAIN, lngstr[unkdomain_]);
+      ALIASP("ORGOUTPUTALIAS", LNGSTR_NODOMAIN, lngstr[nodomain_]);
+      ALIASP("ORGOUTPUTALIAS", LNGSTR_UNKDOMAIN, lngstr[unkdomain_]);
+      ALIASP("DIROUTPUTALIAS", LNGSTR_ROOTDIR, lngstr[rootdir_]);
+      ALIASP("DIROUTPUTALIAS", LNGSTR_NODIR, lngstr[nodir_]);
+      ALIASP("TYPEOUTPUTALIAS", LNGSTR_NOEXT, lngstr[noext_]);
+      ALIASP("TYPEOUTPUTALIAS", LNGSTR_BRKDIRS, lngstr[brkdirs_]);
+      ALIASP("OSOUTPUTALIAS", LNGSTR_UNKWIN, lngstr[unkwin_]);
+      ALIASP("OSOUTPUTALIAS", LNGSTR_UNKMAC, lngstr[unkmac_]);
+      ALIASP("OSOUTPUTALIAS", LNGSTR_UNKUX, lngstr[unkux_]);
+      ALIASP("OSOUTPUTALIAS", LNGSTR_UNKOS, lngstr[unkos_]);
+      ALIASP("OSOUTPUTALIAS", LNGSTR_ROBOTS, lngstr[robots_]);
     }
 #ifndef NODNS
     if (dnslevel != DNS_NONE) {
@@ -1764,13 +1817,12 @@ void clargs(Options *op, int argc, char *argv[]) {
 	  CLSHORTCHECK(confline(op, "FROM", argv[i] + 2, NULL, -2));
 	break;
       case 'g':
-	CLSHORTCHECK((void)config(argv[i] + 2, op));
+	CLSHORTCHECK((void)config(argv[i] + 2, op, TRUE));
 	break;
       case 'G':  /* mandatory config file: already dealt with */
 	break;
       case 'O':
-	CLSHORTCHECK(configstr((void *)&(op->outopts.outfile), NULL,
-			       argv[i] + 2, NULL, -1));
+	CLSHORTCHECK(confline(op, "OUTFILE", argv[i] + 2, NULL, -2));
 	break;
       case 'q':
 	cldebug(&warn_args, argv[i]);
