@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -174,73 +174,118 @@ void configallcols(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
     configcols((void *)&(cols[j * COL_NUMBER]), cmd, arg1, NULL, -3);
 }
 
-/* now a set of similar configs that use symbolic words */
-void configsortby(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
-  choice *sortby = (choice *)opt;
+/* A set of symbolic word choices for use with configchoice() below. Must all
+   end with "". */
+Choices sortbychoices[] = {{"REQUESTS", REQUESTS}, {"REQUESTS7", REQUESTS7},
+  {"PAGES", PAGES}, {"PAGES7", PAGES7}, {"BYTES", BYTES}, {"BYTES7", BYTES7},
+  {"LASTDATE", DATESORT}, {"DATE", DATESORT}, {"FIRSTDATE", FIRSTDATE},
+  {"ALPHABETICAL", ALPHABETICAL}, {"RANDOM", RANDOM}, {"", 0}};
+#ifndef NOGRAPHICS
+Choices chartchoices[] = {{"ON", CHART_SORTBY}, {"TRUE", CHART_SORTBY},
+  {"SORTBY", CHART_SORTBY}, {"OFF", CHART_NONE}, {"FALSE", CHART_NONE},
+  {"NONE", CHART_NONE}, {"REQUESTS", REQUESTS}, {"REQUESTS7", REQUESTS7},
+  {"PAGES", PAGES}, {"PAGES7", PAGES7}, {"BYTES", BYTES}, {"BYTES7", BYTES7},
+  {"", 0}};
+#endif
+Choices outstylechoices[] = {{"HTML", HTML}, {"PLAIN", PLAIN},
+  {"ASCII", ASCII}, {"LATEX", LATEX}, {"COMPUTER", COMPUTER},
+  {"PREFORMATTED", COMPUTER}, {"CRO", COMPUTER}, {"CACHE", OUT_NONE},
+  {"NONE", OUT_NONE}, {"", 0}};
+#ifndef NODNS
+Choices dnschoices[] = {{"NONE", DNS_NONE}, {"OFF", DNS_NONE},
+  {"READ", DNS_READ}, {"LOOKUP", DNS_LOOKUP}, {"WRITE", DNS_WRITE}, {"", 0}};
+#endif
+Choices gotochoices[] = {{"ON", TRUE}, {"TRUE", TRUE}, {"ALL", TRUE},
+  {"OFF", FALSE}, {"FALSE", FALSE}, {"NONE", FALSE}, {"FEW", FEW},
+  {"SOME", FEW}, {"", 0}};
+Choices casechoices[] = {{"INSENSITIVE", TRUE}, {"SENSITIVE", FALSE}, {"", 0}};
+Choices daychoices[] = {{"SUNDAY", SUNDAY}, {"MONDAY", MONDAY},
+  {"TUESDAY", TUESDAY}, {"WEDNESDAY", WEDNESDAY}, {"THURSDAY", THURSDAY},
+  {"FRIDAY", FRIDAY}, {"SATURDAY", SATURDAY}, {"", 0}};
+Choices langchoices[] = {{"ARMENIAN", ARMENIAN}, {"BOSNIAN", OLDLANG},
+  {"BULGARIAN", BULGARIAN}, {"BULGARIAN-MIK", BULGARIAN_MIK},
+  {"CATALAN", CATALAN}, {"SIMP-CHINESE", OLDLANG}, {"CHINESE", OLDLANG},
+  {"TRAD-CHINESE", TRAD_CHINESE}, {"TAIWANESE", TRAD_CHINESE},
+  {"CROATIAN", OLDLANG}, {"CZECH", CZECH}, {"CZECH-1250", CZECH_1250},
+  {"DANISH", DANISH}, {"DUTCH", DUTCH}, {"FLEMISH", DUTCH},
+  {"ENGLISH", ENGLISH}, {"US-ENGLISH", US_ENGLISH}, {"FINNISH", FINNISH},
+  {"FRENCH", FRENCH}, {"GERMAN", GERMAN}, {"GREEK", OLDLANG},
+  {"HUNGARIAN", HUNGARIAN}, {"ICELANDIC", OLDLANG}, {"ITALIAN", ITALIAN},
+  {"JAPANESE-EUC", JAPANESE_EUC}, {"JAPANESE-JIS", JAPANESE_JIS},
+  {"JAPANESE-SJIS", JAPANESE_SJIS}, {"JAPANESE-UTF", JAPANESE_UTF},
+  {"JAPANESE", JAPANESE_JIS}, {"KOREAN", KOREAN}, {"LATVIAN", LATVIAN},
+  {"LITHUANIAN", OLDLANG}, {"NORWEGIAN", NORWEGIAN}, {"BOKMAL", NORWEGIAN},
+  {"BOKMAAL", NORWEGIAN}, {"NYNORSK", NYNORSK}, {"POLISH", POLISH},
+  {"PORTUGUESE", PORTUGUESE}, {"PORTUGESE", PORTUGUESE},
+  {"BR-PORTUGUESE", BR_PORTUGUESE}, {"BR-PORTUGESE", BR_PORTUGUESE},
+  {"BRAZILIAN", BR_PORTUGUESE}, {"ROMANIAN", OLDLANG}, {"RUSSIAN", OLDLANG},
+  {"SERBIAN", SERBIAN}, {"SERBOCROATIAN", SERBIAN}, {"SERBOCROAT", SERBIAN},
+  {"SLOVAK", OLDLANG}, {"SLOVENE", SLOVENE}, {"SLOVENIAN", SLOVENE},
+  {"SLOVENE-1250", SLOVENE}, {"SLOVENIAN-1250", SLOVENE}, {"SPANISH", SPANISH},
+  {"SWEDISH", SWEDISH}, {"SWEDISH-ALT", SWEDISH_ALT}, {"TURKISH", TURKISH},
+  {"UKRAINIAN", UKRAINIAN}, {"UKRANIAN", UKRAINIAN}, {"", 0}};
+Choices onoffchoices[] = {{"ON", TRUE}, {"TRUE", TRUE}, {"OFF", FALSE},
+  {"FALSE", FALSE}, {"", 0}};
+
+void configchoice(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
+  Choices *choices;
+  choice *ans = (choice *)opt;
+  logical *ansl = (logical *)opt;
+  char *cmdend = strchr(cmd, '\0');
+  int i;
+  logical done, islog = FALSE;
 
   if (rc == 0) {
     shortwarn(cmd, arg1, rc);
     return;
   }
+
+  if (STREQ(cmdend - 6, "SORTBY"))
+    choices = sortbychoices;
+#ifndef NOGRAPHICS
+  else if (STREQ(cmdend - 5, "CHART"))
+    choices = chartchoices;
+#endif
+  else if (STREQ(cmd, "OUTPUT"))
+    choices = outstylechoices;
+#ifndef NODNS
+  else if (STREQ(cmd, "DNS"))
+    choices = dnschoices;
+#endif
+  else if (STREQ(cmd, "LANGUAGE"))
+    choices = langchoices;
+  else if (STREQ(cmd, "GOTOS"))
+    choices = gotochoices;
+  else if (STREQ(cmdend - 4, "CASE")) {
+    choices = casechoices;
+    islog = TRUE;
+  }
+  else if (STREQ(cmd, "WEEKBEGINSON"))
+    choices = daychoices;
+  else {
+    choices = onoffchoices;
+    islog = TRUE;
+  }
+
   if (rc > 1)
     longwarn(cmd, arg1, arg2, rc);
-  if (strcaseeq(arg1, "REQUESTS"))
-    *sortby = REQUESTS;
-  else if (strcaseeq(arg1, "REQUESTS7"))
-    *sortby = REQUESTS7;
-  else if (strcaseeq(arg1, "PAGES"))
-    *sortby = PAGES;
-  else if (strcaseeq(arg1, "PAGES7"))
-    *sortby = PAGES7;
-  else if (strcaseeq(arg1, "BYTES"))
-    *sortby = BYTES;
-  else if (strcaseeq(arg1, "BYTES7"))
-    *sortby = BYTES7;
-  else if (strcaseeq(arg1, "LASTDATE") || strcaseeq(arg1, "DATE"))
-    *sortby = DATESORT;
-  else if (strcaseeq(arg1, "FIRSTDATE"))
-    *sortby = FIRSTDATE;
-  else if (strcaseeq(arg1, "ALPHABETICAL"))
-    *sortby = ALPHABETICAL;
-  else if (strcaseeq(arg1, "RANDOM"))
-    *sortby = RANDOM;
-  else
+  for (i = 0, done = FALSE; !done && !IS_EMPTY_STRING(choices[i].name); i++) {
+    if (strcaseeq(arg1, choices[i].name)) {
+      if (islog)
+	*ansl = (logical)(choices[i].arg);
+      else
+	*ans = choices[i].arg;
+      /* I'm sure islog is unnecessary -- that we can safely pass a logical in
+	 to this function and pretend it's a choice in here. But let's pretend
+	 we're in a strongly-typed language. :-) */
+      done = TRUE;
+    }
+  }
+  if (!done)
     badwarn(cmd, TRUE, arg1, arg2, rc);
 }
 
 #ifndef NOGRAPHICS
-void configchart(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
-  choice *q = (choice *)opt;  /* see also configallchart() below */
-
-  if (rc == 0) {
-    shortwarn(cmd, arg1, rc);
-    return;
-  }
-  if (rc > 1)
-    longwarn(cmd, arg1, arg2, rc);
-
-  if (strcaseeq(arg1, "ON") || strcaseeq(arg1, "TRUE") ||
-      strcaseeq(arg1, "SORTBY"))
-    *q = CHART_SORTBY;
-  else if (strcaseeq(arg1, "OFF") || strcaseeq(arg1, "FALSE") ||
-	   strcaseeq(arg1, "NONE"))
-    *q = CHART_NONE;
-  else if (strcaseeq(arg1, "REQUESTS"))
-    *q = REQUESTS;
-  else if (strcaseeq(arg1, "REQUESTS7"))
-    *q = REQUESTS7;
-  else if (strcaseeq(arg1, "PAGES"))
-    *q = PAGES;
-  else if (strcaseeq(arg1, "PAGES7"))
-    *q = PAGES7;
-  else if (strcaseeq(arg1, "BYTES"))
-    *q = BYTES;
-  else if (strcaseeq(arg1, "BYTES7"))
-    *q = BYTES7;
-  else
-    badwarn(cmd, TRUE, arg1, arg2, rc);
-}
-
 void configallchart(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
   /* only takes ON and OFF to avoid invalid choices */
   choice *q = (choice *)opt;
@@ -304,298 +349,51 @@ void configdebug(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
   }
 }
 
-void configonoff(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
-  logical *q = (logical *)opt;
-
-  if (rc == 0) {
-    shortwarn(cmd, arg1, rc);
-    return;
-  }
-  if (rc > 1)
-    longwarn(cmd, arg1, arg2, rc);
-  if (strcaseeq(arg1, "ON") || strcaseeq(arg1, "TRUE"))
-    *q = TRUE;
-  else if (strcaseeq(arg1, "OFF") || strcaseeq(arg1, "FALSE"))
-    *q = FALSE;
-  else
-    badwarn(cmd, TRUE, arg1, arg2, rc);
-}
-
 void configall(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
   logical *q = (logical *)opt;
+  choice result = UNSET;
   int i;
-  logical result;
 
-  if (rc == 0) {
-    shortwarn(cmd, arg1, rc);
-    return;
-  }
-  if (rc > 1)
-    longwarn(cmd, arg1, arg2, rc);
-
-  if (strcaseeq(arg1, "ON") || strcaseeq(arg1, "TRUE"))
-    result = TRUE;
-  else if (strcaseeq(arg1, "OFF") || strcaseeq(arg1, "FALSE"))
-    result = FALSE;
-  else {
-    badwarn(cmd, TRUE, arg1, arg2, rc);
-    return;
-  }
-  for (i = 0; i < REP_NUMBER; i++) {
-    if (i != REP_GENSUM)
-      q[i] = result;
+  configchoice((void *)(&result), cmd, arg1, arg2, rc);
+  if (result != UNSET) {
+    for (i = 0; i < REP_NUMBER; i++) {
+      if (i != REP_GENSUM)
+	q[i] = result;
+    }
   }
 }
 
 void configallback(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
   logical *q = (logical *)opt;  /* same as configall(), but only DATEREPORTs */
+  choice result = UNSET;
   int i;
-  logical result;
 
-  if (rc == 0) {
-    shortwarn(cmd, arg1, rc);
-    return;
+  configchoice((void *)(&result), cmd, arg1, arg2, rc);
+  if (result != UNSET) {
+    for (i = 0; i < DATEREPORTS_NUMBER; i++)
+      q[i] = (logical)result;
   }
-  if (rc > 1)
-    longwarn(cmd, arg1, arg2, rc);
-
-  if (strcaseeq(arg1, "ON") || strcaseeq(arg1, "TRUE"))
-    result = TRUE;
-  else if (strcaseeq(arg1, "OFF") || strcaseeq(arg1, "FALSE"))
-    result = FALSE;
-  else {
-    badwarn(cmd, TRUE, arg1, arg2, rc);
-    return;
-  }
-  for (i = 0; i < DATEREPORTS_NUMBER; i++)
-    q[i] = result;
 }
 
-void configgotos(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
-  choice *q = (choice *)opt;
-
-  if (rc == 0) {
-    shortwarn(cmd, arg1, rc);
-    return;
-  }
-  if (rc > 1)
-    longwarn(cmd, arg1, arg2, rc);
-  if (strcaseeq(arg1, "ON") || strcaseeq(arg1, "TRUE") ||
-      strcaseeq(arg1, "ALL"))
-    *q = TRUE;
-  else if (strcaseeq(arg1, "OFF") || strcaseeq(arg1, "FALSE") ||
-	   strcaseeq(arg1, "NONE"))
-    *q = FALSE;
-  else if (strcaseeq(arg1, "FEW") || strcaseeq(arg1, "SOME"))
-    *q = FEW;
-  else
-    badwarn(cmd, TRUE, arg1, arg2, rc);
-}
-
-void configcase(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
-  logical *q = (logical *)opt;
-
-  if (rc == 0) {
-    shortwarn(cmd, arg1, rc);
-    return;
-  }
-  if (rc > 1)
-    longwarn(cmd, arg1, arg2, rc);
-  if (strcaseeq(arg1, "INSENSITIVE"))
-    *q = TRUE;
-  else if (strcaseeq(arg1, "SENSITIVE"))
-    *q = FALSE;
-  else
-    badwarn(cmd, TRUE, arg1, arg2, rc);
-}
-
-void configweekday(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
-  choice *d = (choice *)opt;
-
-  if (rc == 0) {
-    shortwarn(cmd, arg1, rc);
-    return;
-  }
-  if (rc > 1)
-    longwarn(cmd, arg1, arg2, rc);
-  if (strcaseeq(arg1, "SUNDAY"))
-    *d = SUNDAY;
-  else if (strcaseeq(arg1, "MONDAY"))
-    *d = MONDAY;
-  else if (strcaseeq(arg1, "TUESDAY"))
-    *d = TUESDAY;
-  else if (strcaseeq(arg1, "WEDNESDAY"))
-    *d = WEDNESDAY;
-  else if (strcaseeq(arg1, "THURSDAY"))
-    *d = THURSDAY;
-  else if (strcaseeq(arg1, "FRIDAY"))
-    *d = FRIDAY;
-  else if (strcaseeq(arg1, "SATURDAY"))
-    *d = SATURDAY;
-  else
-    badwarn(cmd, TRUE, arg1, arg2, rc);
-}
-
-void configoutstyle(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
-  choice *s = (choice *)opt;
-
-  if (rc == 0) {
-    shortwarn(cmd, arg1, rc);
-    return;
-  }
-  if (rc > 1)
-    longwarn(cmd, arg1, arg2, rc);
-  if (strcaseeq(arg1, "HTML"))
-    *s = HTML;
-  else if (strcaseeq(arg1, "PLAIN"))
-    *s = PLAIN;
-  else if (strcaseeq(arg1, "ASCII"))
-    *s = ASCII;
-  else if (strcaseeq(arg1, "LATEX"))
-    *s = LATEX;
-  else if (strcaseeq(arg1, "COMPUTER") || strcaseeq(arg1, "PREFORMATTED") ||
-	   strcaseeq(arg1, "CRO"))
-    *s = COMPUTER;
-  else if (strcaseeq(arg1, "CACHE") || strcaseeq(arg1, "NONE"))
-    *s = OUT_NONE;
-  else
-    badwarn(cmd, TRUE, arg1, arg2, rc);
-}
-
-#ifndef NODNS
-void configdns(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
-  choice *s = (choice *)opt;
-
-  if (rc == 0) {
-    shortwarn(cmd, arg1, rc);
-    return;
-  }
-  if (rc > 1)
-    longwarn(cmd, arg1, arg2, rc);
-  if (strcaseeq(arg1, "NONE") || strcaseeq(arg1, "OFF"))
-    *s = DNS_NONE;
-  else if (strcaseeq(arg1, "READ"))
-    *s = DNS_READ;
-  else if (strcaseeq(arg1, "LOOKUP"))
-    *s = DNS_LOOKUP;
-  else if (strcaseeq(arg1, "WRITE"))
-    *s = DNS_WRITE;
-  else
-    badwarn(cmd, TRUE, arg1, arg2, rc);
-}
-#endif
-
 void configlang(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
   Lang *lang = (Lang *)opt;
-  logical oldlang = FALSE;
+  choice code = UNSET;
 
+  configchoice((void *)(&code), cmd, arg1, arg2, rc);
   if (rc == 0) {
     shortwarn(cmd, arg1, rc);
     return;
   }
   if (rc > 1)
     longwarn(cmd, arg1, arg2, rc);
-  if (strcaseeq(arg1, "ARMENIAN"))
-    lang->code = ARMENIAN;
-  else if (strcaseeq(arg1, "BOSNIAN"))
-    /* lang->code = BOSNIAN; */ oldlang = TRUE;
-  else if (strcaseeq(arg1, "BULGARIAN"))
-    lang->code = BULGARIAN;
-  else if (strcaseeq(arg1, "BULGARIAN-MIK"))
-    lang->code = BULGARIAN_MIK;
-  else if (strcaseeq(arg1, "CATALAN"))
-    lang->code = CATALAN;
-  else if (strcaseeq(arg1, "SIMP-CHINESE") || strcaseeq(arg1, "CHINESE"))
-    /* lang->code = SIMP_CHINESE; */ oldlang = TRUE;
-  else if (strcaseeq(arg1, "TRAD-CHINESE") || strcaseeq(arg1, "TAIWANESE"))
-    lang->code = TRAD_CHINESE;
-  else if (strcaseeq(arg1, "CROATIAN"))
-    /* lang->code = CROATIAN; */ oldlang = TRUE;
-  else if (strcaseeq(arg1, "CZECH"))
-    lang->code = CZECH;
-  else if (strcaseeq(arg1, "CZECH-1250"))
-    lang->code = CZECH_1250;
-  else if (strcaseeq(arg1, "DANISH"))
-    lang->code = DANISH;
-  else if (strcaseeq(arg1, "FLEMISH") || strcaseeq(arg1, "DUTCH"))
-    lang->code = DUTCH;
-  else if (strcaseeq(arg1, "ENGLISH"))
-    lang->code = ENGLISH;
-  else if (strcaseeq(arg1, "US-ENGLISH"))
-    lang->code = US_ENGLISH;
-  else if (strcaseeq(arg1, "FINNISH"))
-    /* lang->code = FINNISH; */ oldlang = TRUE;
-  else if (strcaseeq(arg1, "FRENCH"))
-    lang->code = FRENCH;
-  else if (strcaseeq(arg1, "GERMAN"))
-    lang->code = GERMAN;
-  else if (strcaseeq(arg1, "GREEK"))
-    /* lang->code = GREEK; */ oldlang = TRUE;
-  else if (strcaseeq(arg1, "HUNGARIAN"))
-    lang->code = HUNGARIAN;
-  else if (strcaseeq(arg1, "ICELANDIC"))
-    /* lang->code = ICELANDIC; */ oldlang = TRUE;
-  else if (strcaseeq(arg1, "ITALIAN"))
-    lang->code = ITALIAN;
-  else if (strcaseeq(arg1, "JAPANESE-EUC"))
-    lang->code = JAPANESE_EUC;
-  else if (strcaseeq(arg1, "JAPANESE-JIS") || strcaseeq(arg1, "JAPANESE"))
-    lang->code = JAPANESE_JIS;
-  else if (strcaseeq(arg1, "JAPANESE-SJIS"))
-    lang->code = JAPANESE_SJIS;
-  else if (strcaseeq(arg1, "JAPANESE-UTF"))
-    lang->code = JAPANESE_UTF;
-  else if (strcaseeq(arg1, "KOREAN"))
-    lang->code = KOREAN;
-  else if (strcaseeq(arg1, "LATVIAN"))
-    lang->code = LATVIAN;
-  else if (strcaseeq(arg1, "LITHUANIAN"))
-    /* lang->code = LITHUANIAN */ oldlang = TRUE;
-  else if (strcaseeq(arg1, "NORWEGIAN"))
-    lang->code = NORWEGIAN;
-  else if (strcaseeq(arg1, "NYNORSK"))
-    lang->code = NYNORSK;
-  else if (strcaseeq(arg1, "POLISH"))
-    lang->code = POLISH;
-  else if (strcaseeq(arg1, "PORTUGUESE") || strcaseeq(arg1, "PORTUGESE"))
-    lang->code = PORTUGUESE;
-  else if (strcaseeq(arg1, "BR-PORTUGUESE") || strcaseeq(arg1, "BR-PORTUGESE")
-	   || strcaseeq(arg1, "BRAZILIAN"))
-    lang->code = BR_PORTUGUESE;
-  else if (strcaseeq(arg1, "ROMANIAN"))
-    /* lang->code = ROMANIAN; */ oldlang = TRUE;
-  else if (strcaseeq(arg1, "RUSSIAN"))
-    /* lang->code = RUSSIAN; */ oldlang = TRUE;
-  else if (strcaseeq(arg1, "SERBIAN") || strcaseeq(arg1, "SERBOCROATIAN") ||
-	   strcaseeq(arg1, "SERBOCROAT"))
-    lang->code = SERBIAN;
-  else if (strcaseeq(arg1, "SLOVAK"))
-    /* lang->code = SLOVAK; */ oldlang = TRUE;
-  else if (strcaseeq(arg1, "SLOVENE") || strcaseeq(arg1, "SLOVENIAN"))
-    lang->code = SLOVENE;
-  else if (strcaseeq(arg1, "SLOVENE-1250") ||
-	   strcaseeq(arg1, "SLOVENIAN-1250"))
-    lang->code = SLOVENE_1250;
-  else if (strcaseeq(arg1, "SPANISH"))
-    lang->code = SPANISH;
-  else if (strcaseeq(arg1, "SWEDISH"))
-    lang->code = SWEDISH;
-  else if (strcaseeq(arg1, "SWEDISH-ALT"))
-    lang->code = SWEDISH_ALT;
-  else if (strcaseeq(arg1, "TURKISH"))
-    lang->code = TURKISH;
-  else if (strcaseeq(arg1, "UKRAINIAN") || strcaseeq(arg1, "UKRANIAN"))
-    lang->code = UKRAINIAN;
-  else {
-    badwarn(cmd, TRUE, arg1, arg2, rc);
-    return;
-  }
-  if (oldlang) {
+  if (code == OLDLANG) {
     badwarn(cmd, FALSE, arg1, arg2, rc);
     warn('C', CONTINUATION, "  (language not yet translated for version 5)");
-    return;
   }
-  lang->file = NULL;   /* if successful */
+  else if (code != UNSET) {  /* code == UNSET warning in configchoice() */
+    lang->code = code;
+    lang->file = NULL;
+  }
 }
 
 void selectlang(char *country, Outchoices *op) {
@@ -619,7 +417,7 @@ void selectlang(char *country, Outchoices *op) {
   }
   else
     fclose(f);
-  DEFAULTSTR(op->lang.file, filename);
+  COPYSTR(op->lang.file, filename);
 
   if (op->domainsfile == NULL) {
     if (op->outstyle == HTML || op->outstyle == ASCII) {
@@ -639,7 +437,7 @@ void selectlang(char *country, Outchoices *op) {
 	fclose(f);
     }
     if (filename != NULL)
-      DEFAULTSTR(op->domainsfile, filename);
+      COPYSTR(op->domainsfile, filename);
   }
 
   if (op->descriptions && op->descfile == NULL) {
@@ -660,7 +458,7 @@ void selectlang(char *country, Outchoices *op) {
 	fclose(f);
     }
     if (filename != NULL)
-      DEFAULTSTR(op->descfile, filename);
+      COPYSTR(op->descfile, filename);
   }
 }
 
@@ -671,6 +469,7 @@ void configlogfmt(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
   Inputformat *form;
   choice count[INPUT_NUMBER];
   choice rc2;
+  char code;
   int i;
 
   if (rc == 0) {
@@ -706,22 +505,16 @@ void configlogfmt(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
     configlogfmt(opt, NULL, LOG_COMBINED3, NULL, -1);
   }
   else if (strcaseeq(arg1, "MICROSOFT-NA") ||
-	   strcaseeq(arg1, "MICROSOFT-NA2")) {
-    configlogfmt(opt, NULL, LOG_MS_NA2A, NULL, -1);
-    configlogfmt(opt, NULL, LOG_MS_NA2B, NULL, -1);
-  }
-  else if (strcaseeq(arg1, "MICROSOFT-NA4")) {
-    configlogfmt(opt, NULL, LOG_MS_NA4A, NULL, -1);
-    configlogfmt(opt, NULL, LOG_MS_NA4B, NULL, -1);
+	   strcaseeq(arg1, "MICROSOFT-NA2") ||
+	   strcaseeq(arg1, "MICROSOFT-NA4")) {  /* NA2 and NA4 are legacy */
+    configlogfmt(opt, NULL, LOG_MS_NA1, NULL, -1);
+    configlogfmt(opt, NULL, LOG_MS_NA2, NULL, -1);
   }
   else if (strcaseeq(arg1, "MICROSOFT-INT") ||
-	   strcaseeq(arg1, "MICROSOFT-INT2")) {
-    configlogfmt(opt, NULL, LOG_MS_INT2A, NULL, -1);
-    configlogfmt(opt, NULL, LOG_MS_INT2B, NULL, -1);
-  }
-  else if (strcaseeq(arg1, "MICROSOFT-INT4")) {
-    configlogfmt(opt, NULL, LOG_MS_INT4A, NULL, -1);
-    configlogfmt(opt, NULL, LOG_MS_INT4B, NULL, -1);
+	   strcaseeq(arg1, "MICROSOFT-INT2") ||
+	   strcaseeq(arg1, "MICROSOFT-INT4")) {
+    configlogfmt(opt, NULL, LOG_MS_INT1, NULL, -1);
+    configlogfmt(opt, NULL, LOG_MS_INT2, NULL, -1);
   }
   else if (strcaseeq(arg1, "WEBSITE-NA"))
     configlogfmt(opt, NULL, LOG_WEBSITE_NA, NULL, -1);
@@ -743,6 +536,8 @@ void configlogfmt(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
     configlogfmt(opt, NULL, LOG_WEBSTAR_EXTENDED1, NULL, -1);
     configlogfmt(opt, NULL, LOG_EXTENDED2, NULL, -1);
   }
+  else if (strcaseeq(arg1, "MACHTTP"))
+    configlogfmt(opt, NULL, LOG_MACHTTP, NULL, -1);
   else if (strcaseeq(arg1, "NETSCAPE"))
     configlogfmt(opt, NULL, LOG_NETSCAPE, NULL, -1);
   else if (strcaseeq(arg1, "BROWSER")) 
@@ -752,27 +547,29 @@ void configlogfmt(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
     configlogfmt(opt, NULL, LOG_REFERRER2, NULL, -1);
   }
   else if ((rc2 = strtoinfmt(&form, arg1, count)) != FMT_OK) {
+    code = (rc == -3)?'F':'C';
     if (rc == -3)
-      warn('C', FALSE, "Ignoring corrupt format line in logfile");
+      warn(code, FALSE, "Ignoring corrupt format line in logfile");
     else
       badwarn(cmd, FALSE, arg1, arg2, rc);
     if (rc2 == FMT_DUP)
-      warn('C', CONTINUATION, "  (reason: one item occurs twice in format)");
+      warn(code, CONTINUATION, "  (reason: one item occurs twice in format)");
     else if (rc2 == FMT_PARTTIME)
-      warn('C', CONTINUATION, "  (reason: time without date or vice versa)");
+      warn(code, CONTINUATION, "  (reason: time without date or vice versa)");
     else if (rc2 == FMT_BADPC)
-      warn('C', CONTINUATION, "  (reason: an unknown item code is present)");
+      warn(code, CONTINUATION, "  (reason: an unknown item code is present)");
     else if (rc2 == FMT_NOPC)
-      warn('C', CONTINUATION, "  (reason: no item codes are present)");
+      warn(code, CONTINUATION, "  (reason: no item codes are present)");
     else if (rc2 == FMT_BADCHAR)
-      warn('C', CONTINUATION,
+      warn(code, CONTINUATION,
 	   "  (reason: an unknown escape sequence is present)");
     else if (rc2 == FMT_NOTERM)
-      warn('C', CONTINUATION, "  (reason: an unterminated string is present)");
+      warn(code, CONTINUATION,
+	   "  (reason: an unterminated string is present)");
     else if (rc2 == FMT_BADBUILTIN)
-      warn('C', CONTINUATION, "  (reason: non-existent built-in format)");
+      warn(code, CONTINUATION, "  (reason: non-existent built-in format)");
     else if (rc2 == FMT_QBUTNOR)
-      warn('C', CONTINUATION, "  (reason: query string without filename)");
+      warn(code, CONTINUATION, "  (reason: query string without filename)");
     /* despite finishing "else if", that should be all the rc2's */
     return;
   }
@@ -906,16 +703,22 @@ void configstrlist(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
       ;  /* find end of list */
   }
   for ( ; t != NULL; t = ((rc == -1)?NULL:strtok((char *)NULL, ","))) {
-    if (*list == NULL) {
-      lp = (Strlist *)xmalloc(sizeof(Strlist));
-      *list = lp;
-    }
+    if (strcaseeq(t, "none"))
+      *list = NULL;
     else {
-      lp->next = (Strlist *)xmalloc(sizeof(Strlist));
-      TO_NEXT(lp);
+      if (*list == NULL) {
+	lp = (Strlist *)xmalloc(sizeof(Strlist));
+	*list = lp;
+      }
+      else {
+	lp->next = (Strlist *)xmalloc(sizeof(Strlist));
+	TO_NEXT(lp);
+      }
+      if (STREQ(cmd, "DOMCHARTEXPAND") && *t == '.')  /* ugly special case */
+	t++;
+      COPYSTR(lp->name, t);
+      lp->next = NULL;
     }
-    DEFAULTSTR(lp->name, t);
-    lp->next = NULL;
   }
   return;
 }
@@ -955,60 +758,67 @@ void configerrfile(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
   }
 }
 
-/* NODIRENT excludes all wildcards in logfile names.
-   Otherwise, if VMSDIRENT, MACDIRENT or WIN32DIRENT is defined, use that.
-   Otherwise use POSIX.2 glob, unless NOGLOB is defined when use POSIX.1 stuff,
-   not allowing wildcards in directory names.
-   MACDIRENT does actually include the POSIX.1 dirent code here, but implements
-   all the functions itself in macstuff.c. */
+/* There are several versions of expandwildlogs(), according to the following
+   flags:
+     NODIRENT excludes all wildcards in logfile names.
+     Otherwise, if VMSDIRENT, MACDIRENT or WIN32DIRENT is defined, use that.
+     Otherwise use POSIX.2 glob, unless NOGLOB is defined when use POSIX.1
+     stuff, not allowing wildcards in directory names.
+     MACDIRENT does actually include the POSIX.1 dirent code here, but
+     implements all the functions itself in macstuff.c.
+   In all cases, expandwildlogs() takes an argument, Filelist *lp, and clones
+   it within the list (using clonelogs()) as it expands the wildcards. It
+   returns a pointer to the last clone. E.g. expandwildlogs(log2*) turns
+   log1 -> log2* -> log3 into log1 -> log2a -> log2b -> log3 returning log2b */
 #ifndef NODIRENT
+Filelist *clonelogs(Filelist *from, char *name) {
+  Filelist *ans = (Filelist *)xmalloc(sizeof(Filelist));
+
+  memcpy(ans, from, sizeof(Filelist));
+  COPYSTR(ans->name, name);
+  ans->next = from->next;
+  from->next = ans;
+  return(ans);
+} 
+
 #ifndef VMSDIRENT
 #ifndef WIN32DIRENT
 #ifndef NOGLOB
 /* Glob code is due to Owen Cliffe Feb 2001, slightly modified by
    Stephen Turner */
-void configwildlogs(void *opt, char *arg1, char *arg2, int rc) {
-  extern logical iscache;
-
+Filelist *expandwildlogs(Filelist *lp, Filelist **pter) {
+  Filelist *ans = lp;
   glob_t globbuf;
-  char *arg1d;
   struct stat statbuf;
-  logical matched = FALSE;
   int i;
 
-  arg1d = buildfilename(iscache?(char *)CACHEDIR:(char *)LOGSDIR, "", arg1);
   globbuf.gl_offs = 0;
-  if(glob(arg1d,GLOB_NOCHECK,NULL,&globbuf) != 0){
-    warn('F', TRUE, "Error while expanding wildcards: ignoring %sfiles %s",
-	 iscache?"cache ":"log", arg1);
-    return;
+  if(glob(lp->name,GLOB_NOCHECK,NULL,&globbuf) != 0){
+    warn('F', TRUE, "Failed to expand wildcards %s", lp->name);
+    return(ans);
   }
    
    for(i=0;i<globbuf.gl_pathc;i++){
       char * fname= globbuf.gl_pathv[i];
-      if (stat(fname, &statbuf) == 0 && S_ISREG(statbuf.st_mode)){
-	 configlogfile(opt,"LOGFILE",fname,arg2,rc);
-	 matched = TRUE;
-      }
+      if (stat(fname, &statbuf) == 0 && S_ISREG(statbuf.st_mode))
+	ans = clonelogs(ans, fname);
    }
-   if (!matched)
-     warn('F', TRUE, "Cannot find %sfile %s: ignoring it",
-	  iscache?"cache ":"log", arg1);
    globfree(&globbuf);
+   if (ans != lp)  /* remove expanded wildcard lp from list */
+     *pter = lp->next;
+   return(ans);
 }
 #else   /* not GLOB; use POSIX dirent */
-void configwildlogs(void *opt, char *arg1, char *arg2, int rc) {
-  extern logical iscache;
+Filelist *expandwildlogs(Filelist *lp, Filelist **pter) {
+  Filelist *ans = lp;
   struct dirent *filep;
   struct stat buf;
   DIR *dirp;
-  char *arg1d, *dirname, *filename, *c;
+  char *dirname, *filename, *c;
   size_t dirlen, len = 0;
-  logical matched = FALSE;
 
-  arg1d = buildfilename(iscache?(char *)CACHEDIR:(char *)LOGSDIR, "", arg1);
-  if ((c = strrpbrk(arg1d, PATHSEPS)) == NULL) {
-    filename = arg1d;
+  if ((c = strrpbrk(lp->name, PATHSEPS)) == NULL) {
+    filename = lp->name;
     dirname = (char *)xmalloc(3);
 #ifdef MAC
     dirname[0] = '\0';
@@ -1018,19 +828,19 @@ void configwildlogs(void *opt, char *arg1, char *arg2, int rc) {
   }
   else {
     filename = c + 1;
-    dirname = (char *)xmalloc((size_t)(c - arg1d) + 2);
-    memcpy((void *)dirname, (void *)arg1d, (size_t)(c - arg1d) + 1);
-    dirname[(c - arg1d) + 1] = '\0';
+    dirname = (char *)xmalloc((size_t)(c - lp->name) + 2);
+    memcpy((void *)dirname, (void *)(lp->name), (size_t)(c - lp->name) + 1);
+    dirname[(c - lp->name) + 1] = '\0';
   }
 
   if ((dirp = opendir(dirname)) == NULL) {
     if (strchr(dirname, '*') != NULL || strchr(dirname, '?') != NULL)
-      warn('F', TRUE, "Cannot open directory %s: ignoring logfiles %s\n"
-	   "(wildcards not allowed in directory name)", dirname, arg1);
+      warn('F', TRUE, "Cannot open directory %s: won't expand wildcards %s\n"
+	   "(wildcards not allowed in directory name)", dirname, lp->name);
     else
-      warn('F', TRUE, "Cannot open directory %s: ignoring logfiles %s",
-	   dirname, arg1);
-    return;
+      warn('F', TRUE, "Cannot open directory %s: won't expand wildcards %s",
+	   dirname, lp->name);
+    return(ans);
   }
 
   dirlen = strlen(dirname);
@@ -1042,40 +852,36 @@ void configwildlogs(void *opt, char *arg1, char *arg2, int rc) {
       }
       memcpy((void *)(dirname + dirlen), (void *)(filep->d_name), len + 1);
       stat(dirname, &buf);  /* dirname now contains complete filename */
-      if (S_ISREG(buf.st_mode)) {
-	configlogfile(opt, "LOGFILE", dirname, arg2, rc);
-	matched = TRUE;
-      }
+      if (S_ISREG(buf.st_mode))
+	ans = clonelogs(ans, dirname);
     }
   }
   closedir(dirp);
-  if (!matched)
-    warn('F', TRUE, "Cannot find %sfile %s: ignoring it",
-	 iscache?"cache ":"log", arg1);
   free((void *)dirname);
+  if (ans != lp)
+    *pter = lp->next;
+  return(ans);
 }
 #endif   /* NOGLOB */
 #else    /* WIN32DIRENT */
-void configwildlogs(void *opt, char *arg1, char *arg2, int rc) {
-  extern logical iscache;
+Filelist *expandwildlogs(Filelist *lp, Filelist **pter) {
+  Filelist *ans = lp;
   struct _stat buf;
-  char *arg1d, *dirname, *c;
+  char *dirname, *c;
   struct _finddata_t file;
   long code;
   int rc2;
   size_t dirlen, len = 0;
-  logical matched = FALSE;
 
-  arg1d = buildfilename(iscache?(char *)CACHEDIR:(char *)LOGSDIR, "", arg1);
-  if ((code = _findfirst(arg1d, &file)) != (long)(-1)) {
-    if ((c = strrpbrk(arg1d, PATHSEPS)) == NULL) {
+  if ((code = _findfirst(lp->name, &file)) != (long)(-1)) {
+    if ((c = strrpbrk(lp->name, PATHSEPS)) == NULL) {
       dirname = (char *)xmalloc(1);
       dirname[0] = '\0';
     }
     else {
-      dirname = (char *)xmalloc((size_t)(c - arg1d) + 2);
-      memcpy((void *)dirname, (void *)arg1d, (size_t)(c - arg1d) + 1);
-      dirname[(c - arg1d) + 1] = '\0';
+      dirname = (char *)xmalloc((size_t)(c - lp->name) + 2);
+      memcpy((void *)dirname, (void *)(lp->name), (size_t)(c - lp->name) + 1);
+      dirname[(c - lp->name) + 1] = '\0';
     }
     dirlen = strlen(dirname);
     for (rc2 = 0; rc2 == 0; rc2 = _findnext(code, &file)) {
@@ -1085,64 +891,61 @@ void configwildlogs(void *opt, char *arg1, char *arg2, int rc) {
       }
       memcpy((void *)(dirname + dirlen), (void *)(file.name), len + 1);
       _stat(dirname, &buf);  /* dirname now contains complete filename */
-      if (_S_IFREG & buf.st_mode) {  /* bitwise & */
-	configlogfile(opt, "LOGFILE", dirname, arg2, rc);
-	matched = TRUE;
-      }
+      if (_S_IFREG & buf.st_mode)    /* bitwise & */
+	ans = clonelogs(ans, dirname);
     }
     _findclose(code);
     free((void *)dirname);
   }
-  if (!matched)
-    warn('F', TRUE, "Cannot find %sfile %s: ignoring it",
-	 iscache?"cache ":"log", arg1);
+  if (ans != lp)
+    *pter = lp->next;
+  return(ans);
 }
-#endif
+#endif   /* WIN32DIRENT */
 #else    /* VMSDIRENT */
-/* This function is due to Dave Jones */
-void configwildlogs(void *opt, char *arg1, char *arg2, int rc) {
-  extern logical iscache;
+/* This function is due to Dave Jones (except for any errors I introduced when
+   converting from old configwildlogs() to new expandwildlogs()) */
+Filelist *expandwildlogs(Filelist *lp, Filelist **pter) {
+  Filelist *ans = lp;
   static char fspec[VMS_FSPEC_MAX], related[VMS_FSPEC_MAX];
   static char result[VMS_FSPEC_MAX];
   static $DESCRIPTOR(fspec_dx,fspec);
   static $DESCRIPTOR(related_dx,"");
   static $DESCRIPTOR(default_dx,".log");
   static $DESCRIPTOR(result_dx,result);
-  char *arg1d, *space, *ques;
+  char *space, *ques;
   long int context;
   int status, stsval, length, LIB$FIND_FILE(), LIB$FIND_FILE_END();
-  logical matched;
   
-  arg1d = buildfilename(iscache?(char *)CACHEDIR:(char *)LOGSDIR, "", arg1);
-  length = strlen (arg1d);
+  length = strlen (lp->name);
   if ( length >= VMS_FSPEC_MAX ) length = VMS_FSPEC_MAX - 1;
-  strncpy ( fspec, arg1d, length ); fspec[length] = '\0';
+  strncpy ( fspec, lp->name, length ); fspec[length] = '\0';
   while ( ques = strchr(fspec,'?') ) *ques = '%';
   fspec_dx.dsc$w_length = length;
-  for ( matched = FALSE, context = 0; 
+  for ( context = 0; 
 	1&(status=LIB$FIND_FILE ( &fspec_dx, &result_dx, &context, &default_dx,
-				  &related_dx, &stsval, (long *) 0 )); 
-	matched = TRUE) {
+				  &related_dx, &stsval, (long *) 0 )); ) {
     space = strchr ( result, ' ' );
     if ( !space ) space = &result[VMS_FSPEC_MAX-1];
     *space = '\0';
-    configlogfile ( opt, "LOGFILE", result, arg2, rc );
+    ans = clonelogs(ans, result);
     /* Save last result to use as default for next lookup */
     strcpy ( related, result );
     related_dx.dsc$w_length = strlen(result);
     related_dx.dsc$a_pointer = related;
   }
   if ( context ) LIB$FIND_FILE_END ( &context );
-  if (!matched)
-    warn('F', TRUE, "Cannot find %sfile %s: ignoring it",
-	 iscache?"cache ":"log", arg1);
+  if (ans != lp)
+    *pter = lp->next;
+  return(ans);
 }
-#endif
-#endif
+#endif  /* VMSDIRENT */
+#endif  /* NODIRENT */
 
 void configlogfile(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
   extern Inputformatlist *logformat;
   extern logical newloglist, iscache;
+  /* see note in configcachefile() below re iscache */
   extern int tz;
 
   Filelist **logfile = (Filelist **)opt;
@@ -1171,12 +974,6 @@ void configlogfile(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
     free((void *)s);
     return;
   }
-#ifndef NODIRENT
-  if (strchr(arg1, '*') || strchr(arg1, '?')) {
-    configwildlogs(opt, arg1, arg2, rc);
-    return;
-  }
-#endif
   if (strcaseeq(arg1, "NONE")) {
     /* from LOGFILE not CACHEFILE because CACHEFILE NONE caught earlier */
     configlogfmt((void *)&logformat, "LOGFORMAT", "DEFAULT", NULL, -1);
@@ -1198,8 +995,8 @@ void configlogfile(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
     lf->next = (Filelist *)xmalloc(sizeof(Filelist));
     TO_NEXT(lf);
   }
-  if (IS_STDIN(arg1)) {
-    DEFAULTSTR(lf->name, arg1);
+  if (IS_STDIN(arg1) || rc == -2) {
+    COPYSTR(lf->name, arg1);
   }
   else
     lf->name = buildfilename(iscache?(char *)CACHEDIR:(char *)LOGSDIR, "",
@@ -1212,8 +1009,8 @@ void configlogfile(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
   lf->bytes = 0;
   lf->bytes7 = 0;
   lf->tz = tz;
-  if (rc >= 2) {  /* not from CACHEFILE: caught earlier */
-    DEFAULTSTR(lf->prefix, arg2);
+  if (rc >= 2) {  /* not from CACHEFILE: those are caught earlier */
+    COPYSTR(lf->prefix, arg2);
     lf->prefixlen = strlen(lf->prefix);
     if (strstr(arg2, "%v") == NULL)
       lf->pvpos = UNSET;
@@ -1248,6 +1045,23 @@ void configcachefile(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
   iscache = FALSE;
 }
 
+void configoutfile(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
+  char **file = (char **)opt;
+
+  if (rc == 0) {
+    shortwarn(cmd, arg1, rc);
+    return;
+  }
+  if (rc > 1)
+    longwarn(cmd, arg1, arg2, rc);
+
+  if (IS_STDOUT(arg1) || rc == -2) {
+    COPYSTR(*file, arg1);
+  }
+  else
+    *file = buildfilename(OUTDIR, "", arg1);
+}
+
 void configchar(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
   char *c = (char *)opt;
 
@@ -1278,7 +1092,7 @@ void configbarstyle(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
     longwarn(cmd, arg1, arg2, rc);
   h = TOLOWER(arg1[0]);
   if (h != 'a' && h != 'b' && h != 'c' && h != 'd' && h != 'e' && h != 'f' &&
-      h != 'g' && h != 'h')
+      h != 'g' && h != 'h' && h != 'i' && h != 'j')
     badwarn(cmd, TRUE, arg1, arg2, rc);
   else
     *g = h;
@@ -1662,7 +1476,7 @@ void configalias(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
 
   /* add new one to front for speed; swap round in reversealias() later */
   ap = (Alias *)xmalloc(sizeof(Alias));
-  DEFAULTSTR(ap->from, arg1);   /* save string even for regex: might l.c. */
+  COPYSTR(ap->from, arg1);   /* save string even for regex: might l.c. */
   ap->to = configaliasto(arg2, is_regex);
   ap->isregex = is_regex;
   if (is_regex)
@@ -1703,8 +1517,8 @@ void configstrpair(void *opt, char *cmd, char *arg1, char *arg2, int rc) {
     ap->next = (Strpair *)xmalloc(sizeof(Strpair));
     TO_NEXT(ap);
   }
-  DEFAULTSTR(ap->name, arg1);
-  DEFAULTSTR(ap->data, arg2);
+  COPYSTR(ap->name, arg1);
+  COPYSTR(ap->data, arg2);
   ap->next = NULL;
 }
 
@@ -1781,7 +1595,7 @@ void configinex(void *opt, char *cmd, char *arg1, char *arg2, int rc,
   if (omittrailslash && !IS_EMPTY_STRING(arg1) &&
       arg1[strlen(arg1) - 1] == '/')
     arg1[strlen(arg1) - 1] = '\0';
-  DEFAULTSTR((*include)->name, arg1);
+  COPYSTR((*include)->name, arg1);
   (*include)->next = ip;   /* save name even for regex: might l.c. */
   if (headcasematch(arg1, "REGEXP:") || headcasematch(arg1, "REGEXPI:")) {
     if (((*include)->pattern =
