@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -117,7 +117,7 @@ int my_lfclose(FILE *f, Filelist *p, Filelist *allc, Filelist *allp,
 	 data[LOGDATA_INFO] + data[LOGDATA_UNKNOWN] +
 	 data[LOGDATA_UNWANTED]) / 10 && data[LOGDATA_CORRUPT] > 10) {
       warn('L', TRUE, "Large number of corrupt lines in %s %s: "
-	   "try different LOGFORMAT", filetype,
+	   "turn debugging on or try different LOGFORMAT", filetype,
 	   IS_STDIN(p->name)?"stdin":(p->name));
       if (strchr(warn_args, 'L') != NULL)
 	report_logformat(errfile, p->format, TRUE);
@@ -528,7 +528,7 @@ choice parseudint(FILE *f, void *arg, char c) {
   return(TRUE);
 }
 
-choice parseyear(FILE *f, void *arg, char c) {
+choice parseyear2(FILE *f, void *arg, char c) {
   unsigned int *y = (unsigned int *)arg;
   logical rc;
 
@@ -540,6 +540,15 @@ choice parseyear(FILE *f, void *arg, char c) {
   return(rc);
 }
 
+choice parseyear2or4(FILE *f, void *arg, char c) {
+  if (*pos == '\0' || (*(pos + 1) == '\0'))
+    return(FALSE);
+  else if (ISDIGIT(*(pos + 2)))
+    return(parseuint4strict(f, arg, c));
+  else
+    return(parseyear2(f, arg, c));
+}
+
 choice parseam(FILE *f, void *arg, char c) {
   char *d = (char *)arg;
 
@@ -555,13 +564,16 @@ choice parseam(FILE *f, void *arg, char c) {
 
 choice parsecode(FILE *f, void *arg, char c) {
   unsigned int *x = (unsigned int *)arg;
-  char *d = pos, e;
+  char *d = pos, e, *g;
 
   *block_end = '\0';
   if (parsejunk(f, NULL, c) == FALSE || pos == d)
     return(FALSE);
-  e = *(pos - 1);
-  *(pos - 1) = '\0';
+  g = (c == WHITESPACE)?pos:(pos - 1);
+  /* parsejunk normally leaves pos AFTER delimiter, but ON delimiter if
+     delimiter == WHITESPACE */
+  e = *g;
+  *g = '\0';
   *x = IGNORE_CODE;
   if (d[0] == 'O' && d[1] == 'K')
     *x = 200;
@@ -578,7 +590,7 @@ choice parsecode(FILE *f, void *arg, char c) {
     else if (STREQ(d, "ailed"))
       *x = 499;
   }
-  *(pos - 1) = e;
+  *g = e;
   return(TRUE);
 }
 
@@ -642,6 +654,7 @@ choice parsejunk(FILE *f, void *arg, char c) { /* NB allows empty strings */
   }
   else if (c != WHITESPACE)
     p++;
+  /* Leave pos AFTER delimiter, except ON delimiter if delim == WHITESPACE */
   pos = p;
   return(TRUE);
 }
@@ -948,43 +961,39 @@ choice parselogfmt(FILE *f, void *arg, char c) {
     else if (chrn(pos, ',') == 15) {
       e = strchr(strchr(pos, ',') + 1, ',') + 2;
       x = (*(e++) - '0');
-      if (*e != '/')
+      if (ISDIGIT(*e))
 	x = x * 10 + (*(e++) - '0');
-      y = (*(++e) - '0');
-      if (*(++e) != '/')
+      if (*(e++) != '/') {
+	warn('F', TRUE, "Logfile seems to be in Microsoft format "
+	     "but I can't parse the date: use LOGFORMAT command");
+	return(BADLOGFMT);
+      }
+      y = (*(e++) - '0');
+      if (ISDIGIT(*e))
 	y = y * 10 + (*(e++) - '0');
-      e += 3;
+      if (*e != '/') {
+	warn('F', TRUE, "Logfile seems to be in Microsoft format "
+	     "but I can't parse the date: use LOGFORMAT command");
+	return(BADLOGFMT);
+      }
       if (x <= 12 && y > 12) {
-	if (*e == ',') {
-	  debug('F', "  Detect that it's in Microsoft format "
-		"(North American dates, two-digit years)");
-	  configlogfmt((void *)&logformat, NULL, "MICROSOFT-NA2", NULL, -1);
-	}
-	else {
-	  debug('F', "  Detect that it's in Microsoft format "
-		"(North American dates, four-digit years)");
-	  configlogfmt((void *)&logformat, NULL, "MICROSOFT-NA4", NULL, -1);
-	}
+	debug('F',
+	      "  Detect that it's in Microsoft format (North American dates)");
+	configlogfmt((void *)&logformat, NULL, "MICROSOFT-NA", NULL, -1);
       }
       else if (x > 12 && y <= 12) {
-	if (*e == ',') {
-	  debug('F', "  Detect that it's in Microsoft format "
-		"(international dates, two-digit years)");
-	  configlogfmt((void *)&logformat, NULL, "MICROSOFT-INT2", NULL, -1);
-	}
-	else {
-	  debug('F', "  Detect that it's in Microsoft format "
-		"(international dates, four-digit years)");
-	  configlogfmt((void *)&logformat, NULL, "MICROSOFT-INT4", NULL, -1);
-	}
+	debug('F',
+	      "  Detect that it's in Microsoft format (international dates)");
+	configlogfmt((void *)&logformat, NULL, "MICROSOFT-INT", NULL, -1);
+      }
+      else if (x > 12 && y > 12) {
+	warn('F', TRUE, "Logfile seems to be in Microsoft format "
+	     "but I can't parse the date: use LOGFORMAT command");
+	return(BADLOGFMT);
       }
       else {
-	if (*e == ',')
-	  warn('F', TRUE, "Microsoft logfile with ambiguous dates: "
-	       "use LOGFORMAT MICROSOFT-NA2 or LOGFORMAT MICROSOFT-INT2");
-	else
-	  warn('F', TRUE, "Microsoft logfile with ambiguous dates: "
-	       "use LOGFORMAT MICROSOFT-NA4 or LOGFORMAT MICROSOFT-INT4");
+	warn('F', TRUE, "Microsoft logfile with ambiguous dates: "
+	     "use LOGFORMAT MICROSOFT-NA or LOGFORMAT MICROSOFT-INT");
 	*d = '\n';
 	return(BADLOGFMT);
       }
@@ -996,23 +1005,32 @@ choice parselogfmt(FILE *f, void *arg, char c) {
       configlogfmt((void *)&logformat, NULL, "NETSCAPE", NULL, -1);
     }
     else if (ISDIGIT(*pos) && ISDIGIT(*(pos + 3)) && *(pos + 5) == '/' &&
-	     *(pos + 14) == ':' && ISDIGIT(*(pos + 16)) &&
-	     chrn(pos, '\t') == 12) {
-      x = (*pos - '0') * 10 + (*(pos + 1) - '0');
-      y = (*(pos + 3) - '0') * 10 + (*(pos + 4) - '0');
-      if (x <= 12 && y > 12) {
-	debug('F',
-	      "  Detect that it's in WebSite format (North American dates)");
-	configlogfmt((void *)&logformat, NULL, "WEBSITE-NA", NULL, -1);
+	     *(pos + 14) == ':' && ISDIGIT(*(pos + 16))) {
+      if (chrn(pos, '\t') == 12) {
+	x = (*pos - '0') * 10 + (*(pos + 1) - '0');
+	y = (*(pos + 3) - '0') * 10 + (*(pos + 4) - '0');
+	if (x <= 12 && y > 12) {
+	  debug('F',
+		"  Detect that it's in WebSite format (North American dates)");
+	  configlogfmt((void *)&logformat, NULL, "WEBSITE-NA", NULL, -1);
+	}
+	else if (x > 12 && y <= 12) {
+	  debug('F',
+		"  Detect that it's in WebSite format (international dates)");
+	  configlogfmt((void *)&logformat, NULL, "WEBSITE-INT", NULL, -1);
+	}
+	else {
+	  warn('F', TRUE, "WebSite logfile with ambiguous dates: "
+	       "use LOGFORMAT WEBSITE-NA or LOGFORMAT WEBSITE-INT");
+	  *d = '\n';
+	  return(BADLOGFMT);
+	}
       }
-      else if (x > 12 && y <= 12) {
-	debug('F',
-	      "  Detect that it's in Website format (international dates)");
-	configlogfmt((void *)&logformat, NULL, "WEBSITE-INT", NULL, -1);
+      else if (chrn(pos, '\t') == 5) {
+	debug('F', "  Detect that it's in MacHTTP format");
+	configlogfmt((void *)&logformat, NULL, "MACHTTP", NULL, -1);
       }
       else {
-	warn('F', TRUE, "WebSite logfile with ambiguous dates: "
-	     "use LOGFORMAT WEBSITE-NA or LOGFORMAT WEBSITE-INT");
 	*d = '\n';
 	return(BADLOGFMT);
       }
@@ -1098,7 +1116,7 @@ choice parselogfmt(FILE *f, void *arg, char c) {
       if (STREQ(pos, "COMMON_LOG_FORMAT"))
 	strcpy(fmt, "COMMON");
       else if (STREQ(pos, "DATE"))
-	strcat(fmt, "%m/%d/%y");
+	strcat(fmt, "%m/%d/%Z");
       else if (STREQ(pos, "TIME"))
 	strcat(fmt, "%W%h:%n:%j");
       else if (STREQ(pos, "RESULT"))
@@ -1296,10 +1314,12 @@ choice parsenextrecord(FILE *f, Inputformat *format) {
   Inputformat *ipf, *ipf2;
   logical rc;
 
-  if (pos == NULL) {
+  if (pos == NULL) {   /* new logfile */
     if (getmoredata(f, block_start, BLOCKSIZE) == EOF)
       return(EOF);
     pos = block_start;
+    if (*pos == '\n' || *pos == '\r')
+      parsenewline(f, NULL, '\0');   /* ignore completely blank lines at top */
     record_start = pos;
   }
 
@@ -1444,17 +1464,17 @@ choice nextdnsline(FILE *f, timecode_t *timec, char **name, char **alias) {
     return(EOF);
   if (strlen(block_start) >= 255) {
     *(block_start + 70) = '\0';
-    warn('C', TRUE, "Ignoring long line in DNS file starting\n%s",
+    warn('G', TRUE, "Ignoring long line in DNS file starting\n%s",
 	 block_start);
     return(FALSE);
   }
-  if ((timestr = strtok(block_start, " ")) == NULL || !ISDIGIT(*timestr) ||
+  if ((timestr = strtok(block_start, " ")) == NULL ||
       (*name = strtok((char *)NULL, " ")) == NULL ||
       (*alias = strtok((char *)NULL, " ")) == NULL ||
-      strtok((char *)NULL, " ") != NULL ||
+      strtok((char *)NULL, " ") != NULL || !ISDIGIT(*timestr) ||
       (*timec = strtoul(timestr, (char **)NULL, 10)) == 0) {
-    warn('C', TRUE, "Ignoring corrupt line in DNS file looking like\n%s",
-	 block_start);
+    warn('G', TRUE, "Ignoring corrupt line in DNS file\n%s %s %s", block_start,
+	 (*name == NULL)?"":(*name), (*alias == NULL)?"":(*alias));
     return(FALSE);
   }
   return(TRUE);
@@ -1471,7 +1491,7 @@ void process_domainsfile(FILE *f, Options *op) {
       ;  /* ignore comment line */
     else if (strlen(block_start) >= 250) {
       *(block_start + 70) = '\0';
-      warn('C', TRUE, "Ignoring long line in domains file starting\n%s",
+      warn('G', TRUE, "Ignoring long line in domains file starting\n%s",
 	   block_start);
     }
     else {
@@ -1480,13 +1500,13 @@ void process_domainsfile(FILE *f, Options *op) {
       for (t = s; *t != ' ' && *t != '\t' && *t != '\0'; t++)
 	;
       if (*t == '\0')
-	warn('C', TRUE, "Ignoring incomplete line in domains file\n%s",
+	warn('G', TRUE, "Ignoring incomplete line in domains file\n%s",
 	     block_start);
       else {
 	for (c = t + 1; *c == ' ' || *c == '\t'; c++)
 	  ;
 	if (*c < '1' || *c > '9' || (*(c + 1) != ' ' && *(c + 1) != '\t'))
-	  warn('C', TRUE, "Ignoring corrupt line in domains file\n%s",
+	  warn('G', TRUE, "Ignoring corrupt line in domains file\n%s",
 	       block_start);
 	else {
 	  *t = '\0';
@@ -1522,11 +1542,11 @@ void process_descfile(FILE *f, char *name, Options *op) {
     else {
       if (strlen(block_start) > 500) {
 	*(block_start + 500) = '\0';
-	warn('C', TRUE, "Long line in report descriptions file: truncating it "
+	warn('G', TRUE, "Long line in report descriptions file: truncating it "
 	     "at 500 characters");
 	/* Limit is because of prettyprintf() */
       }
-      DEFAULTSTR(op->outopts.descstr[i], block_start);
+      COPYSTR(op->outopts.descstr[i], block_start);
     }
   }
   while (getnextline(f, block_start, 0) != EOF) {
