@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -16,6 +16,7 @@
 #ifdef MAC
 #ifdef USE_ZLIB
 #include <unix.h>
+#include <unistd.h>
 #include <fcntl.h>
 #include "my_bzlib.h"
 #include "my_zlib.h"
@@ -201,6 +202,44 @@ DoAEQuit(const AppleEvent *theAppleEvent,AppleEvent *reply,long handlerRefcon)
 	return(err);
 }
 
+#ifndef _MSL_USE_NEW_FILE_APIS				// These are Pro 7 concepts!!
+#ifndef _MSL_USE_OLD_FILE_APIS				// If neither is defined, this is Pro 6
+#define _MSL_USE_OLD_FILE_APIS		1		// In Pro 6 do it the old way
+#endif
+#endif
+
+#if _MSL_USE_OLD_AND_NEW_FILE_APIS
+extern _MSL_IMP_EXP_C char __msl_system_has_new_file_apis(void);
+#endif
+
+static OSErr
+OpenFork(FSSpec *spec,char permission,short *refNum)
+{
+#if _MSL_USE_NEW_FILE_APIS
+	FSRef theRef;
+#endif
+	OSErr err;
+
+#if _MSL_USE_OLD_AND_NEW_FILE_APIS
+	if (__msl_system_has_new_file_apis()) {
+#endif /* _MSL_USE_OLD_AND_NEW_FILE_APIS */
+#if _MSL_USE_NEW_FILE_APIS
+		err=FSpMakeFSRef(spec,&theRef);
+		if (!err) err=FSOpenFork(&theRef,0,NULL,permission,refNum);
+#endif
+#if _MSL_USE_OLD_AND_NEW_FILE_APIS
+		}
+	else {
+#endif /* _MSL_USE_OLD_AND_NEW_FILE_APIS */
+#if _MSL_USE_OLD_FILE_APIS
+		err=FSpOpenDF(spec,permission,refNum);
+#endif
+#if _MSL_USE_OLD_AND_NEW_FILE_APIS
+		}
+#endif
+	return(err);
+	}
+
 char TryAEOpen(void);
 
 char
@@ -242,10 +281,9 @@ TryAEOpen(void)
 				}
 			}
 		}
-	if (haveFSS && !HOpenDF(myFSS.vRefNum,myFSS.parID,myFSS.name,fsRdPerm,&refNum)) {
-		lvalue=1;
-		FSRead(refNum,&lvalue,&c);
-		FSClose(refNum);
+	if (haveFSS && !OpenFork(&myFSS,fsRdPerm,&refNum)) {
+		read(refNum,&c,1);
+		close(refNum);
 		return(c);
 		}
         return(0);
@@ -395,8 +433,8 @@ mac_fopen(const char *name,const char *mode)
 	int len;
 	short refNum;
 
-	if (!strcmp(name,"%%%") && !strcmp(mode,"r") && haveFSS) {
-		if (!HOpenDF(myFSS.vRefNum,myFSS.parID,myFSS.name,fsRdPerm,&refNum)) {
+	if (!strcmp(name,"~~~") && !strcmp(mode,"r") && haveFSS) {
+		if (!OpenFork(&myFSS,fsRdPerm,&refNum)) {
 			if (in_fp) return(fdopen(refNum,"r"));
 			in_fp=fdopen(refNum,"rb");
 			}
