@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -31,7 +31,7 @@
 static char *cmd_vect[64] = {
 	"analog",
 	"-G",
-	"+g%%%",
+	"+g~~~",
 	0
 	};
 
@@ -70,7 +70,7 @@ void MacInit(int *argc,char **argv[])
   setvbuf(stdout,NULL,_IOLBF,BUFSIZ);
   if ((c=TryAEOpen())!=0) {
     if (c!='#') {
-      cmd_vect[1]="%%%";
+      cmd_vect[1]="~~~";
       *argc=2;
     }
     else *argc=3;
@@ -193,7 +193,6 @@ opendir (const char *name)           /* Open a directory stream on NAME. */
 	dir_volume=0;										// Default to the application directory
 	dir_ID=0;
 	while (name[0]=='.') ++name;						// Don't allow leading '.', avoids device names
-	if (*name) {
 		CToPCpy(fname,name);
 		if (!FSMakeFSSpec(0,0,fname,&spec)) {			// Figure out where user is pointing
 			ResolveAliasFile(&spec, true, &targetIsFolder, &wasAliased);
@@ -209,7 +208,6 @@ opendir (const char *name)           /* Open a directory stream on NAME. */
 			dir_ID=spec.parID;
 			}
 		else return ((DIR *)0);
-		}
 	indx=1;
 	return((DIR *)1);
 	}
