@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -19,14 +19,14 @@ void pagetop(FILE *outf, Outchoices *od, Dateman *dman) {
   choice outstyle = od->outstyle;
   char *compsep = od->compsep;
   char **lngstr = od->lngstr;
+  char *charset = lngstr[charset_];
 
   double t0;
   int t1, t2;
 
   if (cgi) {
     if (outstyle == HTML)
-      fprintf(outf, "Content-Type: text/html; charset=%s\n\n",
-	      lngstr[charset_]);
+      fprintf(outf, "Content-Type: text/html; charset=%s\n\n", charset);
     else
       fprintf(outf, "Content-Type: text/plain\n\n");
   }
@@ -34,17 +34,17 @@ void pagetop(FILE *outf, Outchoices *od, Dateman *dman) {
     fputs("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\n", outf);
     fputs("<html>\n<head>\n", outf);
     fprintf(outf, "<meta http-equiv=\"Content-Type\" "
-	    "content=\"text/html; charset=%s\">\n", lngstr[charset_]);
+	    "content=\"text/html; charset=%s\">\n", charset);
     if (od->norobots)
       fputs("<meta name=\"robots\" content=\"noindex,nofollow\">\n", outf);
     fprintf(outf, "<meta name=\"GENERATOR\" content=\"analog %s\">\n",
 	    VERSION);
     fprintf(outf, "<title>%s ", lngstr[webstatsfor_]);
-    htmlfprintf(outf, od, od->multibyte, od->hostname, TRUE);
+    htmlputs(outf, od, od->hostname, FROM_CFG);
     fputs("</title>\n", outf);
     if (!strcaseeq(od->stylesheet, "none")) {
       fputs("<link href=\"", outf);
-      htmlfprintf(outf, od, FALSE, od->stylesheet, FALSE);
+      htmlputs(outf, od, od->stylesheet, IN_HREF);
       fputs("\" rel=\"stylesheet\">\n", outf);
     }
     fputs("</head>\n", outf);
@@ -52,21 +52,21 @@ void pagetop(FILE *outf, Outchoices *od, Dateman *dman) {
     if (!strcaseeq(od->logo, "none")) {
       fputs("<IMG src=\"", outf);
       if (od->logo[0] != '/' && strstr(od->logo, "://") == NULL)
-	htmlfprintf(outf, od, FALSE, od->imagedir, FALSE);
-      htmlfprintf(outf, od, FALSE, od->logo, FALSE);
+	htmlputs(outf, od, od->imagedir, IN_HREF);
+      htmlputs(outf, od, od->logo, IN_HREF);
       if (STREQ(od->logo, "analogo"))
 	fprintf(outf, "%c%s", EXTSEP, od->pngimages?"png":"gif");
       fputs("\" alt=\"\"> ", outf);
     }
     if (strcaseeq(od->hosturl, "none")) {
       fprintf(outf, "%s</a> ", lngstr[webstatsfor_]);
-      htmlfprintf(outf, od, od->multibyte, od->hostname, TRUE);
+      htmlputs(outf, od, od->hostname, FROM_CFG);
     }
     else {
       fprintf(outf, "%s</a> <a HREF=\"", lngstr[webstatsfor_]);
-      htmlfprintf(outf, od, FALSE, od->hosturl, FALSE);
+      htmlputs(outf, od, od->hosturl, IN_HREF);
       fputs("\">", outf);
-      htmlfprintf(outf, od, od->multibyte, od->hostname, TRUE);
+      htmlputs(outf, od, od->hostname, FROM_CFG);
       fputs("</a>", outf);
     }
     fputs("</h1>\n\n", outf);
@@ -79,8 +79,15 @@ void pagetop(FILE *outf, Outchoices *od, Dateman *dman) {
   }
   else if (outstyle == LATEX) {
     fputs("\\documentclass{article}\n", outf);
-    fputs("\\usepackage{isolatin1}\n", outf);  /* safe even in ASCII charset */
-    fputs("\\usepackage[dvips]{color}\n", outf);
+    if (!strcaseeq(lngstr[charset_], "US-ASCII")) {
+      /* Charset US-ASCII or ISO-8859-[12] enforced in init.c */
+      fprintf(outf, "\\usepackage[latin%c]{inputenc}\n", charset[9]);
+      fputs("\\usepackage[T1]{fontenc}\n", outf);
+    }
+    if (od->pdflatex)
+      fputs("\\usepackage[pdftex]{color}\n", outf);
+    else
+      fputs("\\usepackage[dvips]{color}\n", outf);
     fputs("\\definecolor{barcolour}{rgb}{0.75,0.2,0.2}\n", outf);
     fputs("\\newsavebox{\\ttchar}\n", outf);
     fputs("\\sbox{\\ttchar}{\\mbox{\\tt -}}\n", outf);
@@ -190,7 +197,7 @@ void pagebot(FILE *outf, Outchoices *od) {
   if (outstyle == HTML && od->html) {
     fputs("<p><a href=\"http://validator.w3.org/\">\n", outf);
     fputs("<img src=\"", outf);
-    htmlfprintf(outf, od, FALSE, od->imagedir, FALSE);
+    htmlputs(outf, od, od->imagedir, IN_HREF);
     fprintf(outf, "html2%c%s\"\n", EXTSEP, od->pngimages?"png":"gif");
     fputs("alt=\"HTML 2.0 Conformant!\"></a>\n", outf);
   }
@@ -544,7 +551,9 @@ void printtree(FILE *outf, Outchoices *od, choice rep, Hashtable *tree,
       need += strlen(pn->name);
     for (p = tree->head[0]; p != NULL; TO_NEXT(p)) {
       name = maketreename(partname, p, &pn, &s, need, rep, TRUE);
-      if (name[0] == '\0' || name[1] == '\0')  /* in REP_ORG, a code */
+      /* name construction also in findwedges() */
+      if (STREQ(name, LNGSTR_NODOMAIN) || STREQ(name, LNGSTR_UNKDOMAIN) ||
+	  ISDIGIT(name[strlen(name) - 1]))
 	rightalign = FALSE;
       else
 	rightalign = possrightalign;
@@ -662,9 +671,7 @@ void genrep(FILE *outf, Outchoices *od, choice rep, Hashindex **gooditems,
       tw = alphatreewidth(od, rep, tree, 0, NULL);
     else for (p = *gooditems; p != NULL; TO_NEXT(p)) {
       strcpy(workspace, p->name);
-      do_aliasx(workspace, od->aliashead[G(rep)]);
-      /* Only use do_aliasx() not do_aliasrep(), because it's just for
-	 measuring width. */
+      do_aliasx(NULL, NULL, workspace, od->aliashead[G(rep)]);
       tw = MAX(tw, htmlstrlen(workspace, outstyle));
     }
     width[COL_TITLE] = MIN(tw, width[COL_TITLE]);
@@ -849,24 +856,23 @@ void output(Outchoices *od, Hashindex **gooditems, Hashindex **baditems,
 	    choice datacols[ITEM_NUMBER][OUTCOME_NUMBER][DATACOLS_NUMBER][2],
 	    choice data2cols[ITEM_NUMBER][DATA_NUMBER], unsigned int *no_cols,
 	    unsigned int granularity) {
-  char *filename = NULL;
+  char *outfile = od->outfile;
+
   FILE *outf;
   int ro;
   choice rep;
 
   /* first open output file */
   
-  if (IS_STDOUT(od->outfile)) {
+  if (IS_STDOUT(outfile)) {
     outf = stdout;
     debug('F', "Opening stdout as output file");
   }
   else {
-    filename = timesprintf(NULL, od->outfile, dman->last7to, TRUE);
-    /* filename is used later, so must not use timesprintf in the mean time */
-    if ((outf = FOPENW(filename)) == NULL)
-      error("failed to open output file %s for writing", filename);
+    if ((outf = FOPENW(outfile)) == NULL)
+      error("failed to open output file %s for writing", outfile);
     else
-      debug('F', "Opening %s as output file", filename);
+      debug('F', "Opening %s as output file", outfile);
   }
 
   /* remove any reports not wanted */
@@ -954,8 +960,8 @@ void output(Outchoices *od, Hashindex **gooditems, Hashindex **baditems,
   /*** Bit at the bottom of the page ***/
 
   pagebot(outf, od);
-  if (!IS_STDOUT(od->outfile)) {
-    debug('F', "Closing %s", filename);
+  if (!IS_STDOUT(outfile)) {
+    debug('F', "Closing %s", outfile);
     fclose(outf);
   }
 }
