@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -228,25 +228,55 @@ void myputc(FILE *outf, char c, choice outstyle) {
     putc(c, outf);
 }
 
-/* Quicker than using myputc() */
-/* See also latexfprintf() below */
-void htmlfprintf(FILE *outf, Outchoices *od, logical multibyte, char *s,
-		 logical userinput) {
-  /* NB We don't do anything when outstyle == LATEX. The reason for this is
-     that we are always inside a \verb so it would be wrong to do the
-     conversions. If we ever call this not from inside a \verb, we may have
-     to change this, and pass in outstyle == ASCII from inside \verb. */
-  /* NB multibyte is not necessarily od->multibyte here, because sometimes we
-     know the text is a URL or something that we definitely want to convert and
-     we pass in multibyte == FALSE. */
-  /* userinput really means "allow \& escape sequences". */
+/* htmlputs(): print a string with an appropriate amount of HTML encoding.
+   Much quicker than using myputc(). */
+
+/* We don't do anything when outstyle != HTML, even when outstyle == LATEX. The
+   reason for this is that we are always inside a \verb so it would be wrong to
+   do the conversions. If we ever call this not from inside a \verb, we may
+   have to change this, and pass in outstyle == ASCII from inside \verb.
+   But see also latexfprintf() below. */
+
+/* multibyte is occasionally not od->multibyte here, because sometimes we know
+   that the text is in English and we pass in multibyte == FALSE. */
+
+/** What to convert has SECURITY IMPLICATIONS. An attacker must not be allowed
+ ** to insert abitrary data in the output.
+ **
+ ** So data is categorised according to its source, via an enum in anlghea3.h.
+ ** In the following descriptions of the security levels, "convert" means
+ ** converting e.g. < to &lt; and "escape" means using \< to prevent this
+ ** happening.
+ **
+ ** 1) TRUSTED: E.g. a string from a language file. Completely trusted. In
+ **    single-byte mode, convert characters (for convenience not security), but
+ **    allow any of the special characters to be escaped, even \< .
+ **    In multibyte mode, output the string as-is.
+ ** 2) FROM_CFG: An item read in from configuration. Unless we're in CGI mode,
+ **    treat as case 1. In CGI mode, the input could have come from the form,
+ **    so be more cautious to avoid cross-site scripting attacks. Specifically,
+ **    convert all characters, allow only \& and \\ escapes, and use '?' in
+ **    place of unprintable characters.
+ ** 3) UNTRUSTED: E.g. data from the logfile. Do all conversions, don't allow
+ **    any escapes, and use '?' in place of all unprintable characters.
+ ** 4) IN_HREF: Special case for data from the config file which is being put
+ **    inside an <a href=""> or <img src="">. As 3), but use %nm in place of
+ **    unprintable characters. (NB data from the logfile which is put inside
+ **    an href uses escfprintf() instead of this function.)
+ **/
+void htmlputs(FILE *outf, Outchoices *od, char *s, choice source) {
+  extern logical cgi;
+  extern logical unprintable[256];
 
   char w1[64];
   char *c;
   char *w = w1;
   int len = 0;
 
-  if (od->outstyle != HTML || multibyte)
+  if (source == FROM_CFG && !cgi)
+    source = TRUSTED;
+
+  if (od->outstyle != HTML || (source == TRUSTED && od->multibyte))
     fputs(s, outf);
   else {
     for (c = s; *c != '\0'; c++) {
@@ -266,12 +296,33 @@ void htmlfprintf(FILE *outf, Outchoices *od, logical multibyte, char *s,
 	PUTs(w, "&quot;", 0);
 	len += 6;
       }
-      else if (userinput && *c == '\\' && *(c + 1) != '\0') {
+      else if (*c == '\\' &&  /* escape these chars in these circumstances: */
+	       ((source == TRUSTED && (*(c + 1) == '<' || *(c + 1) == '>' ||
+				       *(c + 1) == '&' || *(c + 1) == '"' ||
+				       *(c + 1) == '\\'))
+		|| (source == FROM_CFG && (*(c + 1) == '&' ||
+					   *(c + 1) == '\\')))) {
 	od->html = FALSE;
 	PUTc(w, *(++c));
 	len += 1;
       }
-      else {
+      else if (unprintable[(unsigned char)(*c)] && source != TRUSTED) {
+	/* unprintable chars */
+	if (source == IN_HREF) {
+#ifdef EBCDIC
+	sprintf(w, "%%%.2X", os_toascii[*c]);
+#else
+	sprintf(w, "%%%.2X", (unsigned char)(*c));
+#endif
+	w += 3;
+	len += 3;
+	}
+	else {  /* source == FROM_CFG or UNTRUSTED */
+	  PUTc(w, '?');
+	  len += 1;
+	}
+      }
+      else {  /* output non-special characters as-is */
 	PUTc(w, *c);
 	len += 1;
       }
@@ -288,7 +339,7 @@ void htmlfprintf(FILE *outf, Outchoices *od, logical multibyte, char *s,
 }
 
 void latexfprintf(FILE *outf, char *s) {
-  /* Modelled after htmlfprintf(). But this time assume outstyle == LATEX has
+  /* Modelled after htmlputs(). But this time assume outstyle == LATEX has
      already been tested. */
   char w1[64];
   char *c;
@@ -338,7 +389,7 @@ void latexfprintf(FILE *outf, char *s) {
 }
 
 void escfprintf(FILE *outf, char *name) {
-  /* Escape names for use in hyperlinks. As with htmlfprintf(), don't try and
+  /* Escape names for use in hyperlinks. As with htmlputs(), don't try and
      print character by character. Assume outstyle == HTML already tested. */
 #ifdef EBCDIC
   extern unsigned char os_toascii[];
@@ -788,9 +839,11 @@ void calcsizes(Outchoices *od, choice rep, unsigned int width[],
       *bmult = 0;
     }
     else {
-      width[COL_BYTES] = MAX(7, htmlstrlen(lngstr[col2colhead[COL_BYTES] + 1],
-					   outstyle));
       *bmult = findbmult(maxb);
+      width[COL_BYTES] = MAX(7, htmlstrlen(lngstr[col2colhead[COL_BYTES] + 1],
+					   outstyle)
+			     + htmlstrlen(lngstr[byteprefixabbr_ + *bmult],
+					  outstyle) - 1);
     }
     /* I have some misgivings about allowing the bmult7 to be different from
        the bmult. It's less immediately readable. But I think it's necessary,
@@ -802,10 +855,12 @@ void calcsizes(Outchoices *od, choice rep, unsigned int width[],
       *bmult7 = 0;
     }
     else {
+      *bmult7 = findbmult(maxb7);
       width[COL_BYTES7] = MAX(7,
 			      htmlstrlen(lngstr[col2colhead[COL_BYTES7] + 1],
-					 outstyle));
-      *bmult7 = findbmult(maxb7);
+					 outstyle)
+			      + htmlstrlen(lngstr[byteprefixabbr_ + *bmult7],
+					   outstyle) - 1);
     }
     width[COL_PREQS] = MAX(6, htmlstrlen(lngstr[col2colhead[COL_PREQS]],
 					 outstyle));
@@ -872,11 +927,10 @@ unsigned int alphatreewidth(Outchoices *od, choice rep, Hashtable *tree,
     return(0);
   for (p = tree->head[0]; p != NULL; TO_NEXT(p)) {
     name = maketreename(partname, p, &pn, &s, need, rep, TRUE);
-    if (name[0] != '\0' && name[1] != '\0') {   /* i.e. ignore special */
-      strcpy(workspace, name);                  /* left-aligned codes */
-      do_aliasx(workspace, od->aliashead[G(rep)]);
-      /* Only use do_aliasx() not do_aliasrep(), because it's just for
-	 measuring width. */
+    if (!STREQ(name, LNGSTR_NODOMAIN) && !STREQ(name, LNGSTR_UNKDOMAIN) &&
+	!ISDIGIT(name[strlen(name) - 1])) { /* ignore left-aligned ones */
+      strcpy(workspace, name);
+      do_aliasx(NULL, NULL, workspace, od->aliashead[G(rep)]);
       tmp = htmlstrlen(workspace, od->outstyle) + 2 * level;
                        /* will be printed with 2 trailing spaces per level */
       tw = MAX(tw, tmp);
@@ -891,10 +945,8 @@ unsigned int alphatreewidth(Outchoices *od, choice rep, Hashtable *tree,
 
 void declareunit(FILE *outf, Outchoices *od, char graphby, double unit,
 		 unsigned int bmult) {
-  /* NB Number, or string "kbytes", can still overflow pagewidth, but only if
-     pagewidth is small, and will wrap straight after. pagewidth is never
-     guaranteed anyway. */
-  extern char *byteprefix;
+  /* NB Number can still overflow pagewidth, but only if pagewidth is small,
+     and will wrap straight after. pagewidth is never guaranteed anyway. */
   extern unsigned int ppcol;
 
   choice outstyle = od->outstyle;
@@ -902,6 +954,8 @@ void declareunit(FILE *outf, Outchoices *od, char graphby, double unit,
   unsigned int pagewidth = od->pagewidth;
   char **lngstr = od->lngstr;
 
+  char *s;
+
   if (outstyle != COMPUTER) {
     if (outstyle == HTML)
       fputs("<p>\n", outf);
@@ -918,7 +972,7 @@ void declareunit(FILE *outf, Outchoices *od, char graphby, double unit,
     }
     else if (outstyle == HTML) {
       mprintf(outf, pagewidth, "<img src=\"");
-      htmlfprintf(outf, od, FALSE, od->imagedir, FALSE);
+      htmlputs(outf, od, od->imagedir, IN_HREF);
       mprintf(outf, pagewidth, "bar%c1%c%s\" alt=\"%c\">", od->barstyle,
 	      EXTSEP, od->pngimages?"png":"gif", markchar);
     }
@@ -927,31 +981,33 @@ void declareunit(FILE *outf, Outchoices *od, char graphby, double unit,
     mprintf(outf, pagewidth, ") %s ", lngstr[represents_]);
     if (graphby == 'R' || graphby == 'r') {
       ppcol += f3printf(outf, outstyle, unit, 0, od->sepchar);
-      mprintf(outf, pagewidth, " ");
       if (unit == 1.)
-	mprintf(outf, pagewidth, "%s.", lngstr[request_]);
+	mprintf(outf, pagewidth, " %s.", lngstr[request_]);
       else
-	mprintf(outf, pagewidth, "%s %s.", lngstr[requests_], lngstr[partof_]);
+	mprintf(outf, pagewidth, " %s %s.", lngstr[requests_],
+		lngstr[partof_]);
     }
     else if (graphby == 'P' || graphby == 'p') {
       ppcol += f3printf(outf, outstyle, unit, 0, od->sepchar);
-      mprintf(outf, pagewidth, " ");
       if (unit == 1.)
-	mprintf(outf, pagewidth, "%s.", lngstr[pagereq_]);
+	mprintf(outf, pagewidth, " %s.", lngstr[pagereq_]);
       else
-	mprintf(outf, pagewidth, "%s %s.", lngstr[pagereqs_], lngstr[partof_]);
+	mprintf(outf, pagewidth, " %s %s.", lngstr[pagereqs_],
+		lngstr[partof_]);
     }
     else {
       if (bmult > 0) {
 	doublemprintf(outf, outstyle, pagewidth, unit, od->decpt);
-	mprintf(outf, pagewidth, " ");
-	mprintf(outf, pagewidth, "%c", byteprefix[bmult]);
+	s = strchr(lngstr[xbytes_], '?');  /* checked in initialisation */
+	*s = '\0';
+	mprintf(outf, pagewidth, " %s%s%s %s.", lngstr[xbytes_],
+		lngstr[byteprefix_ + bmult], s + 1, lngstr[partof_]);
+	*s = '?';
       }
       else {
 	ppcol += f3printf(outf, outstyle, unit, 0, od->sepchar);
-	mprintf(outf, pagewidth, " ");
+	mprintf(outf, pagewidth, " %s %s.", lngstr[bytes_], lngstr[partof_]);
       }
-      mprintf(outf, pagewidth, "%s %s.", lngstr[bytes_], lngstr[partof_]);
     }
     mprintf(outf, 0, NULL);
   }
@@ -959,6 +1015,7 @@ void declareunit(FILE *outf, Outchoices *od, char graphby, double unit,
 
 void whatincluded(FILE *outf, Outchoices *od, choice rep, unsigned long n,
 		  Dateman *dman) {
+  extern char *byteprefix;
   extern unsigned int *method2sing, *method2pl, *method2date, *method2pc;
   extern unsigned int *method2relpc, *method2sort;
 
@@ -977,8 +1034,10 @@ void whatincluded(FILE *outf, Outchoices *od, choice rep, unsigned long n,
   choice date = rep2date[G(rep)];
   choice firstd = rep2firstd[G(rep)];
 
-  int firsts, firstds, alls, sorted, alphsort, unsort;
+  int firsts, firstds, alls, sorted, alphsort, unsort, bmult;
   char *lngs, *c;
+  static char *t = NULL;
+  static size_t tlen = 0;
   unsigned long temp = 0;
   timecode_t tempd;
 
@@ -1065,41 +1124,53 @@ void whatincluded(FILE *outf, Outchoices *od, choice rep, unsigned long n,
 	else {  /* floorqual != '\0' */
 	  doublemprintf(outf, outstyle, pagewidth, floormin, od->decpt);
 	  if (floorqual == '%') {
-	    if (outstyle == LATEX)
-	      fputc('\\', outf);
-	    /* The above is a real cheat. It translates % to \% by relying on
-	       the fact that all the following strings _start_ with %. In fact,
-	       this isn't true in Hungarian, but as the LaTeX output style only
-	       accepts ASCII and ISO-8859-1, this shouldn't be a problem.
-	       Unless someone specifies LANGFILE hua.lng. Or until another
-	       language file comes along and breaks this assumption. Oh well.
-	       The proper fix is much too difficult, unless this becomes a real
-	       problem. */
 	    if (floorby == REQUESTS)
-	      mprintf(outf, pagewidth, "%s", lngstr[method2pc[requests]]);
+	      c = lngstr[method2pc[requests]];
 	    else if (floorby == REQUESTS7)
-	      mprintf(outf, pagewidth, "%s", lngstr[method2pc[requests7]]);
+	      c = lngstr[method2pc[requests7]];
 	    else
-	      mprintf(outf, pagewidth, "%s", lngstr[method2pc[floorby]]);
+	      c = lngstr[method2pc[floorby]];
+	    if (outstyle == LATEX) { /* then change one % to \% -- yuk. */
+	      ENSURE_LEN(t, tlen, strlen(c) + 2);
+	      strcpy(t, c);
+	      if ((c = strchr(t, '%')) != NULL) {
+		memmove(c + 1, c, strlen(c) + 1);
+		*c = '\\';
+	      }
+	      c = t;
+	    }
+	    mprintf(outf, pagewidth, "%s", c);
 	  }
 	  else if (floorqual == ':') {
-	    if (outstyle == LATEX)
-	      fputc('\\', outf);   /* see comment above */
 	    if (floorby == REQUESTS)
-	      mprintf(outf, pagewidth, "%s", lngstr[method2relpc[requests]]);
+	      c = lngstr[method2relpc[requests]];
 	    else if (floorby == REQUESTS7)
-	      mprintf(outf, pagewidth, "%s", lngstr[method2relpc[requests7]]);
+	      c = lngstr[method2relpc[requests7]];
 	    else
-	      mprintf(outf, pagewidth, "%s", lngstr[method2relpc[floorby]]);
+	      c = lngstr[method2relpc[floorby]];
+	    if (outstyle == LATEX) { /* and again, change one % to \%. */
+	      ENSURE_LEN(t, tlen, strlen(c) + 2);
+	      strcpy(t, c);
+	      if ((c = strchr(t, '%')) != NULL) {
+		memmove(c + 1, c, strlen(c) + 1);
+		*c = '\\';
+	      }
+	      c = t;
+	    }
+	    mprintf(outf, pagewidth, "%s", c);
 	  }
-	  else { /* if qual is anything else, must be (k|M|G|T)bytes */
+	  else { /* if qual is anything else, must be (k|M|G|T|etc.)bytes */
 	    lngs = (floorby == BYTES)?lngstr[xbytestraffic_]:\
 	      lngstr[xbytestraffic7_];
-	    if ((c = strchr(lngs, '?')) != NULL)
-	      *c = floorqual;
-	    mprintf(outf, pagewidth, " %s", lngs);
-	    if (c != NULL)
-	      *c = '?';
+	    if (strchr(byteprefix, floorqual) == NULL)  /* shouldn't happen */
+	      bmult = 1;
+	    else
+	      bmult = strchr(byteprefix, floorqual) - byteprefix;
+	    c = strchr(lngs, '?');  /* checked during initialisation */
+	    *c = '\0';
+	    mprintf(outf, pagewidth, " %s%s%s", lngs,
+		    lngstr[byteprefix_ + bmult], c + 1);
+	    *c = '?';
 	  }
 	}   /* end floorqual != '\0' */
       }     /* end floormin > EPSILON */
@@ -1195,8 +1266,6 @@ void busyprintf(FILE *outf, Outchoices *od, choice rep, char *datefmt,
 		datecode_t date, unsigned int hr, unsigned int min,
 		datecode_t newdate, unsigned int newhr, unsigned int newmin,
 		char graphby) {
-  extern char *byteprefix;
-
   choice outstyle = od->outstyle;
   char **lngstr = od->lngstr;
   char sepchar = od->sepchar;
@@ -1204,6 +1273,7 @@ void busyprintf(FILE *outf, Outchoices *od, choice rep, char *datefmt,
   char *busystr = lngstr[rep2busystr[rep]];
 
   unsigned int bmult;
+  char *s;
 
   if (outstyle == ASCII)
     putc('\n', outf);
@@ -1238,9 +1308,15 @@ void busyprintf(FILE *outf, Outchoices *od, choice rep, char *datefmt,
     printbytes(outf, od, bys, bmult, 0, sepchar);
     if (outstyle != COMPUTER) {
       putc(' ', outf);
-      if (bmult >= 1)
-	putc(byteprefix[bmult], outf);
-      fprintf(outf, "%s).\n", lngstr[bytes_]);
+      if (bmult >= 1) {
+	s = strchr(lngstr[xbytes_], '?');  /* checked in initialisation */
+	*s = '\0';
+	fprintf(outf, "%s%s%s).\n", lngstr[xbytes_],
+		lngstr[byteprefix_ + bmult], s + 1);
+	*s = '?';
+      }
+      else
+	fprintf(outf, "%s).\n", lngstr[bytes_]);
     }
   }
   if (outstyle == COMPUTER)
@@ -1306,7 +1382,7 @@ void barchart(FILE *outf, Outchoices *od, char graphby, unsigned long reqs,
     for (j = 32; j >= 1; j /= 2) {
       while (y >= j) {
 	fputs("<img src=\"", outf);
-	htmlfprintf(outf, od, FALSE, od->imagedir, FALSE);
+	htmlputs(outf, od, od->imagedir, IN_HREF);
 	fprintf(outf, "bar%c%d%c%s\" alt=\"", od->barstyle, j, EXTSEP,
 		od->pngimages?"png":"gif");
 	if (first) {
@@ -1324,7 +1400,6 @@ void barchart(FILE *outf, Outchoices *od, char graphby, unsigned long reqs,
 void colheads(FILE *outf, Outchoices *od, choice rep, unsigned int width[],
 	      unsigned int bmult, unsigned int bmult7, logical name1st) {
   extern unsigned int *col2colhead, *rep2colhead;
-  extern char *byteprefix;
 
   char **lngstr = od->lngstr;
   char *name = lngstr[rep2colhead[rep]];
@@ -1351,14 +1426,19 @@ void colheads(FILE *outf, Outchoices *od, choice rep, unsigned int width[],
 	bm = (cols[c] == COL_BYTES)?bmult:bmult7;
 	lngs = lngstr[col2colhead[cols[c]] + (int)(bm != 0)];
 	len = (int)htmlstrlen(lngs, outstyle);
+	if (bm > 0)
+	  len += (int)htmlstrlen(lngstr[byteprefixabbr_ + bm], outstyle) - 1;
 	for (i = width[cols[c]] - len; i > 0; i--)
 	  putc(' ', outf);
-	d = NULL;
-	if (bmult != 0 && (d = strchr(lngs, '?')) != NULL)
-	  *d = byteprefix[bm];
-	fprintf(outf, "%s: ", lngs);
-	if (d != NULL)
+	if (bm > 0) {
+	  d = strchr(lngs, '?');  /* checked during initialisation */
+	  *d = '\0';
+	  fprintf(outf, "%s%s%s: ", lngs, lngstr[byteprefixabbr_ + bm],
+		  d + 1);
 	  *d = '?';
+	}
+	else
+	  fprintf(outf, "%s: ", lngs);
       }
       else
 	fprintf(outf, "%*s: ", width[cols[c]]
@@ -1414,10 +1494,6 @@ char printcols(FILE *outf, Outchoices *od, choice rep, unsigned long reqs,
   logical timerep = (rep < DATEREP_NUMBER);
   char graphby = timerep?(od->graph[rep]):'\0';
   char repsepchar = od->repsepchar;
-  logical multibyte =
-    (rep == REP_SIZE || rep == REP_PROCTIME)?FALSE:(od->multibyte);
-  /* For these two reports, we know the texts are in English, and we want to
-     convert > and < */
 
   char *datestr;
   /* Choice of a verbchar: can't be number, letter, colon, space, percent
@@ -1428,8 +1504,9 @@ char printcols(FILE *outf, Outchoices *od, choice rep, unsigned long reqs,
      verbchar at the end for the cases in which calling functions need to
      terminate the \verb */
   char *verbchar = "|+/!^#-=?_\\~'$\"()<>[]{}.,&;";
-  char lastditch[] = {(char)(0xBE)};   /* the character 3/4 in ISO-8859-1 */
+  char lastditch[] = {(char)(0xF7)};   /* the division sign in ISO-8859-[12] */
   choice outstyle, saveoutstyle;
+  logical aliased, multibyte = FALSE;
   int c, i;
 
   saveoutstyle = od->outstyle;
@@ -1514,7 +1591,15 @@ char printcols(FILE *outf, Outchoices *od, choice rep, unsigned long reqs,
     for (i = (int)width[COL_TITLE] - (int)htmlstrlen(name, outstyle); i > 0;
 	 i--)
       putc(' ', outf);
-    htmlfprintf(outf, od, multibyte, name, FALSE);
+    if (rep == REP_SIZE || rep == REP_PROCTIME) {
+      /* Kludge: for these two reports, we know the texts are things like
+	 "< 1" and we want to convert > and < */
+      multibyte = od->multibyte;
+      od->multibyte = FALSE;
+    }
+    htmlputs(outf, od, name, TRUSTED);
+    if (rep == REP_SIZE || rep == REP_PROCTIME)
+      od->multibyte = multibyte;
     fputs(": ", outf);
   }
 
@@ -1598,12 +1683,18 @@ char printcols(FILE *outf, Outchoices *od, choice rep, unsigned long reqs,
   else if (!name1st) {
     if (name == NULL)
       return(*verbchar);   /* calling function supplies name and newline */
-    strcpy(workspace, name);
-    do_aliasrep(workspace, od->aliashead[G(rep)], multibyte);
+    aliased = TRUE;  /* see aliased = do_aliasx() comment below */
     if (outstyle != COMPUTER) {
-      if (rightalign)
+      if (rightalign) {
+	strcpy(workspace, name);
+	aliased = do_aliasx(NULL, NULL, workspace, od->aliashead[G(rep)]);
+	/* In this case we have to do the alias twice, once into a string to
+	   calculate the number of spaces required, and once to actually print
+	   it. But at least we remember if an alias wasn't required, so that we
+	   don't need to do it again. */
 	i = (int)width[COL_TITLE] - (int)htmlstrlen(workspace, outstyle) -
 	  (int)spaces;
+      }
       else
 	i = (int)spaces;
       for ( ; i > 0; i--)
@@ -1615,16 +1706,21 @@ char printcols(FILE *outf, Outchoices *od, choice rep, unsigned long reqs,
 	 the nature of an annotation. */
       fputs("<a href=\"", outf);
       if (baseurl != NULL)
-	htmlfprintf(outf, od, FALSE, baseurl, TRUE);
+	htmlputs(outf, od, baseurl, IN_HREF);
       escfprintf(outf, name);
       fputs("\">", outf);
-      htmlfprintf(outf, od, multibyte, workspace, TRUE);
+      if (aliased)
+	do_aliasx(outf, od, name, od->aliashead[G(rep)]);  /* alias & print */
+      else
+	htmlputs(outf, od, name, UNTRUSTED);
       fputs("</a>", outf);
     }
+    else if (aliased)
+      do_aliasx(outf, od, name, od->aliashead[G(rep)]);    /* alias & print */
     else
-      htmlfprintf(outf, od, multibyte, workspace, TRUE);
+      htmlputs(outf, od, name, UNTRUSTED);
   }
-  /* The previous htmlfprintf's have userinput = TRUE because of aliases,
+  /* The previous htmlputs's have userinput = TRUE because of aliases,
      particularly accents in domains: cf do_aliasr(). Note also that for
      multibyte charsets, some necessary conversions may not take place.
      There's not much we can do about this because the source of the name may
@@ -1725,16 +1821,14 @@ void gensumline(FILE *outf, Outchoices *od, char codeletter, int namecode,
 void gensumlineb(FILE *outf, Outchoices *od, char codeletter, int namecode,
 		 double x, double x7) {
   /* same as gensumline() but for bytes */
-  extern char *byteprefix;
-
   choice outstyle = od->outstyle;
   char **lngstr = od->lngstr;
   char *name = lngstr[namecode];
   char *colon = lngstr[colon_];
   char *compsep = od->compsep;
 
-  char *lngs, *c;
-  unsigned int j;
+  char *c;
+  unsigned int bm;
 
   if (strchr(od->gensumlines, codeletter) == NULL)
     return;
@@ -1747,16 +1841,18 @@ void gensumlineb(FILE *outf, Outchoices *od, char codeletter, int namecode,
       fprintf(outf, "{\\bf %s%s} ", name, colon);
     else /* outstyle == COMPUTER */
       fprintf(outf, "x%s%c%c%s", compsep, name[0], name[1], compsep);
-    j = (od->rawbytes)?0:findbmult(x);
-    printbytes(outf, od, x, j, 0, od->sepchar);
+    bm = (od->rawbytes)?0:findbmult(x);
+    printbytes(outf, od, x, bm, 0, od->sepchar);
     if (outstyle != COMPUTER) {
-      lngs = (j > 0)?lngstr[xbytes_]:lngstr[bytes_];
-      c = NULL;
-      if (j > 0 && (c = strchr(lngs, '?')) != NULL)
-	*c = byteprefix[j];
-      fprintf(outf, " %s", lngs);
-      if (c != NULL)
+      if (bm > 0) {
+	c = strchr(lngstr[xbytes_], '?');
+	*c = '\0';
+	fprintf(outf, " %s%s%s", lngstr[xbytes_], lngstr[byteprefix_ + bm],
+		c + 1);
 	*c = '?';
+      }
+      else
+	fprintf(outf, " %s", lngstr[bytes_]);
     }
     if (x7 != UNSET) {
       if (outstyle == COMPUTER) {
@@ -1765,15 +1861,17 @@ void gensumlineb(FILE *outf, Outchoices *od, char codeletter, int namecode,
       }
       else {
 	fputs(" (", outf);
-	j = (od->rawbytes)?0:findbmult(x7);
-	printbytes(outf, od, x7, j, 0, od->sepchar);
-	lngs = (j > 0)?lngstr[xbytes_]:lngstr[bytes_];
-	c = NULL;
-	if (j > 0 && (c = strchr(lngs, '?')) != NULL)
-	  *c = byteprefix[j];
-	fprintf(outf, " %s)", lngs);
-	if (c != NULL)
+	bm = (od->rawbytes)?0:findbmult(x7);
+	printbytes(outf, od, x7, bm, 0, od->sepchar);
+	if (bm > 0) {
+	  c = strchr(lngstr[xbytes_], '?');
+	  *c = '\0';
+	  fprintf(outf, " %s%s%s)", lngstr[xbytes_], lngstr[byteprefix_ + bm],
+		  c + 1);
 	  *c = '?';
+	}
+	else
+	  fprintf(outf, " %s)", lngstr[bytes_]);
       }
     }
     putc('\n', outf);
@@ -2018,6 +2116,75 @@ FILE *piechart_init(char *filename) {
   return(pieoutf);
 }
 
+void findwedges(Wedge wedge[NO_COLOURS], choice rep, Hashindex *items,
+		choice chartby, Strlist *expandlist, unsigned int level,
+		Strlist *partname, unsigned long tot, double totb,
+		double totb7) {
+  /* Calculate which wedges we actually want, i.e. the ten with the biggest
+     angles. But we also preserve the sort order of the "items" list. (Be
+     careful between > and >= so as to use that order for breaking ties).
+     Construction of name same as in printtree(). */
+  static double smallestangle;
+  static int smallestwedge;
+
+  char *name;
+  double angle;
+  Strlist *pn, s;
+  size_t need = (size_t)level + 3;
+  Hashindex *p;
+  int i;
+
+  if (level == 0) {  /* not recursing: initialise wedges to 0 */
+    for (i = 0; i < NO_COLOURS; i++) {
+      wedge[i].angle = 0.0;
+      wedge[i].name = NULL;
+    }
+    smallestangle = 0.0;
+    smallestwedge = NO_COLOURS - 1;
+  }
+
+  for (pn = partname; pn != NULL; TO_NEXT(pn))
+    need += strlen(pn->name);
+  for (p = items; p != NULL; TO_NEXT(p)) {
+    name = maketreename(partname, p, &pn, &s, need, rep, TRUE);
+    if (incstrlist(name, expandlist) && p->other != NULL &&
+	((Hashtable *)(p->other))->head[0] != NULL)
+      /* then find wedges in lower level of tree instead. ->head[0] != NULL
+	 must come after p->other != NULL; o/wise it may not be a treerep */
+      findwedges(wedge, rep, ((Hashtable *)(p->other))->head[0], chartby,
+		 expandlist, level + 1, pn, tot, totb, totb7);
+    else {
+      if (chartby == BYTES)
+	angle = p->own->bytes / totb;
+      else if (chartby == BYTES7)
+	angle = p->own->bytes7 / totb7;
+      else
+	angle = ((double)(p->own->data[chartby])) / ((double)tot);
+      if (angle > smallestangle) {/* remove smallest, move along, put p last */
+	/* We probably don't do this very often so we don't bother with keeping
+	   hold of the memory and reusing it later. */
+	free(wedge[smallestwedge].name);
+	for (i = smallestwedge; i < NO_COLOURS - 1; i++) {
+	  wedge[i].name = wedge[i + 1].name;
+	  wedge[i].angle = wedge[i + 1].angle;
+	}
+	COPYSTR(wedge[NO_COLOURS - 1].name, name);
+	/* malloc's necessary space. Needed because next call to maketreename()
+	   will overwrite name. */
+	wedge[NO_COLOURS - 1].angle = angle;
+	smallestangle = wedge[0].angle;  /* Recalculate smallest */
+	smallestwedge = 0;
+	for (i = 1; i < NO_COLOURS; i++) {
+	  if (wedge[i].angle <= smallestangle) {
+	    smallestangle = wedge[i].angle;
+	    smallestwedge = i;
+	  }
+	}
+      }
+    }
+  }
+}
+
 void piechart_caption(FILE *outf, choice rep, choice chartby, char **lngstr) {
   extern unsigned int *method2sort;
   static char *caption = NULL;
@@ -2051,11 +2218,11 @@ void piechart_caption(FILE *outf, choice rep, choice chartby, char **lngstr) {
     fprintf(outf, "<p><em>%s</em>\n", caption);
 }
 
-void piechart_wedge(FILE *outf, Outchoices *od, double angle, char *s,
-		    char *extension) {
-  /* the angle is expressed between 0 and 1 */
+int piechart_wedge(FILE *outf, Outchoices *od, double angle, char *s) {
+  /* The angle is expressed between 0 and 1. Returns col if wedge was big
+     enough to be coloured in, NO_COLOURS for grey, else -1. */
   double x, y, newangle, medangle;
-  int colour = black;
+  int colour = black, rc = -1;
   char t[MAXCHARS];
   size_t len;
 
@@ -2064,7 +2231,7 @@ void piechart_wedge(FILE *outf, Outchoices *od, double angle, char *s,
     colour = grey;
   }
   else if (col >= NO_COLOURS)
-    angle = 0;  /* as a signal not to make a wedge */
+    angle = 0;  /* As a signal not to make a wedge. Can this happen? */
   else if (angle >= MINANGLE)
     colour = colours[col];
 
@@ -2084,13 +2251,14 @@ void piechart_wedge(FILE *outf, Outchoices *od, double angle, char *s,
       PIE_EPSILON;
     gdImageLine(im, XCENTRE, YCENTRE, (int)x, (int)y, black);
 
-    /* fill wedge */
+    /* Fill wedge */
     medangle = totangle + angle / 2.;
     x = (double)XCENTRE + (double)DIAMETER * cos(medangle * TWOPI) / 2.5;
     y = (double)YCENTRE + (double)DIAMETER * sin(medangle * TWOPI) / 2.5;
-    if (gdImageGetPixel(im, (int)x, (int)y) == white) {
+    if (gdImageGetPixel(im, (int)x, (int)y) == white) {  /* room to colour */
       gdImageFill(im, (int)x, (int)y, colour);
-      /* make label for wedge */
+      /* Make label for wedge. If !normalchart, this is done in piechart_key()
+	 below instead. (See long comment near bottom of piechart().) */
       if (normalchart) {
 	gdImageFilledRectangle(im, BOXESLEFT, boxesy, BOXESLEFT + BOXESSIZE,
 			       boxesy + BOXESSIZE, colour);
@@ -2110,19 +2278,26 @@ void piechart_wedge(FILE *outf, Outchoices *od, double angle, char *s,
 		      (unsigned char *)t, black);
 	boxesy += TEXTGAP;
       }
-      else {
-	fputs("<br><img src=\"", outf);
-	htmlfprintf(outf, od, FALSE, od->imagedir, FALSE);
-	if (colour == grey)
-	  fprintf(outf, "sqg%c%s", EXTSEP, extension);
-	else
-	  fprintf(outf, "sq%d%c%s", col, EXTSEP, extension);
-	fprintf(outf, "\" alt=\"\"> %s\n", s);
-      }
-    }
+      rc = (colour == grey)?NO_COLOURS:col;
+    }   /* end if (room to colour) */
     totangle = newangle;
     col++;
   }
+  return(rc);
+}
+
+void piechart_key(FILE *outf, Outchoices *od, int col, char *name,
+		  char *extension, Alias *aliashead) {
+  /* Only called if !normalchart and wedge was included on chart */
+  fputs("<br><img src=\"", outf);
+  htmlputs(outf, od, od->imagedir, IN_HREF);
+  if (col == NO_COLOURS)
+    fprintf(outf, "sqg%c%s", EXTSEP, extension);
+  else
+    fprintf(outf, "sq%d%c%s", col, EXTSEP, extension);
+  fputs("\" alt=\"\"> ", outf);
+  do_aliasx(outf, od, name, aliashead);  /* alias and print */
+  fputs("\n", outf);
 }
 
 void piechart_write(FILE *pieoutf, char *filename, logical jpegcharts) {
@@ -2137,6 +2312,14 @@ void piechart_write(FILE *pieoutf, char *filename, logical jpegcharts) {
   gdImageDestroy(im);
 }
 
+void piechart_cleanup(Wedge wedge[NO_COLOURS]) {
+  int i;
+
+  /* free the wedge names allocated in findwedges() */
+  for (i = 0; i < NO_COLOURS; i++)
+    free(wedge[i].name);
+}
+
 void piechart(FILE *outf, Outchoices *od, choice rep, Hashindex *items,
 	      choice requests, choice requests7, choice pages, choice pages7,
 	      unsigned long totr, unsigned long totr7, unsigned long totp,
@@ -2148,14 +2331,14 @@ void piechart(FILE *outf, Outchoices *od, choice rep, Hashindex *items,
 
   char **lngstr = od->lngstr;
   choice chartby = od->chartby[G(rep)];
+  Strlist *expandlist = od->expandhead[G(rep)];
   char gender = lngstr[rep2lng[rep] + 3][0];
   char *extension = (od->jpegcharts)?"jpg":"png";
 
   Wedge wedge[NO_COLOURS];
   FILE *pieoutf;
-  Hashindex *p;
-  double angle, smallestangle, largestangle;
-  int smallestwedge;
+  int key[NO_COLOURS], keyg;
+  double largestangle;
   unsigned long tot = 1;
   char *otherstr;
   int i;
@@ -2185,41 +2368,9 @@ void piechart(FILE *outf, Outchoices *od, choice rep, Hashindex *items,
     return;
   }
 
-  /* We need to calculate which wedges we actually want, i.e. the ten with the
-     biggest angles. But we also preserve the sort order of the "items" list.
-     (And we are careful between > and >= so as to use that order for breaking
-     ties). */
-  for (i = 0; i < NO_COLOURS; i++) {
-    wedge[i].angle = 0.0;
-    wedge[i].name = NULL;
-  }
-  smallestangle = 0.0;
-  smallestwedge = NO_COLOURS - 1;
-
-  for (p = items; p != NULL; TO_NEXT(p)) {
-    if (chartby == BYTES)
-      angle = p->own->bytes / totb;
-    else if (chartby == BYTES7)
-      angle = p->own->bytes7 / totb7;
-    else
-      angle = ((double)(p->own->data[chartby])) / ((double)tot);
-    if (angle > smallestangle) {  /* remove smallest, move along, put p last */
-      for (i = smallestwedge; i < NO_COLOURS - 1; i++) {
-	wedge[i].name = wedge[i + 1].name;
-	wedge[i].angle = wedge[i + 1].angle;
-      }
-      wedge[NO_COLOURS - 1].name = p->name;
-      wedge[NO_COLOURS - 1].angle = angle;
-      smallestangle = wedge[0].angle;  /* Recalculate smallest */
-      smallestwedge = 0;
-      for (i = 1; i < NO_COLOURS; i++) {
-	if (wedge[i].angle <= smallestangle) {
-	  smallestangle = wedge[i].angle;
-	  smallestwedge = i;
-	}
-      }
-    }
-  }
+  /* Calculate which wedges to include */
+  findwedges(wedge, rep, items, chartby, expandlist, 0, NULL, tot, totb,
+	     totb7);
 
   /* Check whether we still want a chart */
   largestangle = wedge[0].angle;
@@ -2260,26 +2411,19 @@ void piechart(FILE *outf, Outchoices *od, choice rep, Hashindex *items,
 	  extension);
   if ((pieoutf = piechart_init(filename)) == NULL)
     return;  /* Warning message is given in piechart_init() */
-  fprintf(outf, "<p><img src=\"%s%s%c%s\" alt=\"\">\n", od->chartdir,
-	  anchorname[rep], EXTSEP, extension);
 
   /* Now we can finally get round to plotting the chart */
-  piechart_caption(outf, rep, od->chartby[G(rep)], lngstr);
   for (i = 0; i < NO_COLOURS; i++) {
+    key[i] = -1;
     if (wedge[i].name != NULL) {
-      /* ugly special cases to construct name lifted from maketreename() */
-      if (rep == REP_TYPE) {
-	strcpy(workspace, ".");
-	strcat(workspace, wedge[i].name);
-      }
-      else
-	strcpy(workspace, wedge[i].name);
-      if (rep == REP_REFSITE || rep == REP_DIR)
-	strcat(workspace, "/");
-      do_aliasx(workspace, od->aliashead[G(rep)]);
-      piechart_wedge(outf, od, wedge[i].angle, workspace, extension);
+      strcpy(workspace, wedge[i].name);
+      do_aliasx(NULL, NULL, workspace, od->aliashead[G(rep)]);
+      key[i] = piechart_wedge(outf, od, wedge[i].angle, workspace);
+      /* retain i -> colour mapping for calling piechart_key() below */
     }
   }
+  if (normalchart)
+    piechart_caption(outf, rep, od->chartby[G(rep)], lngstr);
 
   /* Plot the catch-all wedge and close the file */
   if (gender == 'm')
@@ -2288,7 +2432,26 @@ void piechart(FILE *outf, Outchoices *od, choice rep, Hashindex *items,
     otherstr = lngstr[otherf_];
   else
     otherstr = lngstr[othern_];
-  piechart_wedge(outf, od, -1., otherstr, extension);
+  keyg = piechart_wedge(outf, od, -1., otherstr);
   piechart_write(pieoutf, filename, od->jpegcharts);
+
+  /* Now the text on the page. In CGI mode, this must be done _after_ the image
+     is closed, or the browser may fail to find the image. This is why printing
+     the caption and key must be done twice; above here if normalchart, below
+     here otherwise. */
+  fprintf(outf, "<p><img src=\"%s%s%c%s\" alt=\"\">\n", od->chartdir,
+	  anchorname[rep], EXTSEP, extension);
+  if (!normalchart) {
+    piechart_caption(outf, rep, od->chartby[G(rep)], lngstr);
+    for (i = 0; i < NO_COLOURS; i++) {
+      if (key[i] != -1)
+	piechart_key(outf, od, key[i], wedge[i].name, extension,
+		     od->aliashead[G(rep)]);
+    }
+    if (keyg != -1)
+      piechart_key(outf, od, keyg, otherstr, extension, NULL);
+  }
+
+  piechart_cleanup(wedge);
 }
 #endif  /* NOGRAPHICS */
