@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -79,9 +79,9 @@ void report_compile(void) {
   printf("  No pie charts\n");
 #else
 #ifdef HAVE_GD
-  printf("  Pie charts included -- using your libgd\n");
+  printf("  Pie chart code included -- using your libgd\n");
 #else
-  printf("  Pie charts included\n");
+  printf("  Pie chart code included\n");
 #endif  /* HAVE_GD */
 #endif  /* NOGRAPHICS */
 #ifdef NOPIPES
@@ -92,7 +92,7 @@ void report_compile(void) {
 #ifdef NODNS
   printf("  No DNS lookups\n");
 #else
-  printf("  DNS lookups included\n");
+  printf("  DNS lookup code included\n");
 #ifdef NOALARM
   printf("  No DNS timeouts\n");
 #else
@@ -246,6 +246,7 @@ void report_logformat(FILE *outf, Inputformatlist *format, logical inwarn) {
 	  p->inpfns->code == 'u' || p->inpfns->code == 'f' ||
 	  p->inpfns->code == 'B' || p->inpfns->code == 'S' ||
 	  p->inpfns->code == 'F' || p->inpfns->code == 'q' ||
+	  p->inpfns->code == 'A' || p->inpfns->code == 's' ||
 	  p->inpfns->code == 'C' || p->inpfns->code == 'j' ||
 	  p->inpfns->code == 'v' || p->inpfns->code == '\0') {
 	if (p->sep == '\t')
@@ -546,7 +547,7 @@ void report_outopts(Outchoices *od) {
     else if (od->outstyle == PLAIN)
       printf("PLAIN\n");
     else if (od->outstyle == LATEX)
-      printf("LATEX\n");
+      printf("LATEX (and PDFLATEX is %s)\n", (od->pdflatex)?"ON":"OFF");
     else {  /* OUT_NONE is reported above */
       printf("COMPUTER\n");
       printf("The field separator is \"%s\"\n", od->compsep);
@@ -648,9 +649,7 @@ void report_genrep(Outchoices *od, choice j) {
   report_cols(od->cols[j]);
   report_sortby(od->sortby[G(j)], FALSE);
   report_floor(&(od->floor[G(j)]), FALSE);
-  if (j == REP_REQ || j == REP_REDIR || j == REP_FAIL || j == REP_REF ||
-      j == REP_REDIRREF || j == REP_FAILREF || j == REP_TYPE || j == REP_DIR ||
-      j == REP_DOM || j == REP_REFSITE || j == REP_BROWSUM) {
+  if (od->subsortby[G(j)] != UNSET) {  /* report is hierarchical */
     report_sortby(od->subsortby[G(j)], TRUE);
     report_floor(&(od->subfloor[G(j)]), TRUE);
   }
