@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -224,7 +224,7 @@ Hashindex *sorttree(Outchoices *od, Hashtable *tree, choice rep, Floor *floor,
   unhash(tree, &gooditems, &baditems);
   if (notcorrupt != NULL) {    /* NB notcorrupt only used for doms */
     baditems = (Hashindex *)submalloc(space, sizeof(Hashindex));
-    baditems->name = "\v";
+    baditems->name = LNGSTR_UNKDOMAIN;
     baditems->own = newhashentry(DATA_NUMBER, FALSE);
     baditems->other = NULL;
     baditems->next = NULL;
@@ -337,7 +337,7 @@ char *maketreename(Strlist *pn, Hashindex *p, Strlist **newpn, Strlist *space,
   /* Compile name from strlist. We end up doing the most-significant portion
      several times, but it really is the best way, at least if the name is
      little-endian, otherwise we have to try and work out which portion we
-     want when we go back down a level. */
+     want when we go back down a level. Further calls will overwrite name. */
   static char *name = NULL;
   static size_t len = 0;
   logical back;
@@ -350,9 +350,15 @@ char *maketreename(Strlist *pn, Hashindex *p, Strlist **newpn, Strlist *space,
     return(p->name);
 
   if (rep == REP_TYPE || rep == REP_DOM || rep == REP_ORG) {
-    back = !(rep == REP_DOM && pn != NULL && STREQ(pn->name, "\f"));
-    /* back = TRUE unless numerical subdomains */
     glue = '.';
+    /* back = TRUE unless numerical subdomains */
+    if (rep == REP_DOM && pn != NULL && STREQ(pn->name, LNGSTR_UNRESOLVED))
+      back = FALSE;
+    else if (rep == REP_ORG && pn != NULL &&
+	     ISDIGIT(pn->name[strlen(pn->name) - 1]))
+      back = FALSE;
+    else
+      back = TRUE;
   }
   else {
     back = FALSE;
@@ -385,9 +391,9 @@ char *maketreename(Strlist *pn, Hashindex *p, Strlist **newpn, Strlist *space,
   if (rep == REP_DOM && !back) /* i.e. numerical subdomain: special case */
     TO_NEXT(sp);
   t = name;
-  if (rep == REP_TYPE && delims)
-    /* delimiter at start: replicate in piechart() in output2.c */
-    *(t++) = glue;
+  if (rep == REP_TYPE && delims && !STREQ(p->name, LNGSTR_NOEXT) &&
+      !STREQ(p->name, LNGSTR_BRKDIRS))
+    *(t++) = glue;      /* delimiter at start */
   if (rep == REP_BROWSUM && sp != NULL) {
     /* special case to enable two different delimiters */
     l = strlen(sp->name);
@@ -404,9 +410,9 @@ char *maketreename(Strlist *pn, Hashindex *p, Strlist **newpn, Strlist *space,
     if (sp->next != NULL)
       *(t++) = glue;
   }
-  if (delims && (rep == REP_REFSITE || rep == REP_DIR))
-    /* delimiter at end: replicate in piechart() in output2.c */
-    *(t++) = glue;
+  if (delims && (rep == REP_REFSITE || rep == REP_DIR) &&
+      !STREQ(p->name, LNGSTR_NODIR) && !STREQ(p->name, LNGSTR_ROOTDIR))
+    *(t++) = glue;      /* delimiter at end */
   *t = '\0';
 
   return(name);
@@ -454,7 +460,8 @@ void rnextname(char **name, char **nameend, char *whole, logical build) {
 }
 
 void inextname(char **name, char **nameend, char *whole, logical build) {
-  static char *s = "/";
+  static char *s0 = LNGSTR_NODIR;
+  static char *s1 = LNGSTR_ROOTDIR;
   logical first = FALSE;
 
   if (*name == NULL) {
@@ -463,6 +470,7 @@ void inextname(char **name, char **nameend, char *whole, logical build) {
   }
   else if (IS_EMPTY_STRING(*nameend) || IS_EMPTY_STRING((*nameend) + 1) ||
 	   **nameend == '?' || *((*nameend) + 1) == '?') {
+    /* including s0 and s1 */
     *name = NULL;
     return;
   }
@@ -477,21 +485,23 @@ void inextname(char **name, char **nameend, char *whole, logical build) {
       ;  /* run to last consecutive '/' */
   }
   if ((**nameend == '\0' || **nameend == '?') && !build) {
-    if (first && **name != '/') {
-      *name = s;
-      *nameend = s + 1;
+    if (first) {
+      if (**name != '/')
+	*name = s0;
+      else
+	*name = s1;
+      *nameend = strchr(*name, '\0');
     }
-    else if (first)
-      *name = *nameend;
     else
       *name = NULL;
   }
 }
 
+static logical isnum = FALSE;  /* used in onextname() and Znextname() */
+
 void onextname(char **name, char **nameend, char *whole, logical build) {
-  static char *s0 = "\f";
-  static char *s1 = "\b";
-  static logical isnum = FALSE;
+  static char *s0 = LNGSTR_UNRESOLVED;
+  static char *s1 = LNGSTR_NODOMAIN;
 
   if (*name == NULL) {
     isnum = FALSE;
@@ -499,13 +509,13 @@ void onextname(char **name, char **nameend, char *whole, logical build) {
     if (!build) {
       if (strchr(whole, '.') == NULL) {
 	*name = s1;
-	*nameend = s1 + 1;
+	*nameend = strchr(*name, '\0');
 	debug('V', "%s", whole);
 	return;
       }
       if (ISDIGIT(*(*nameend - 1))) {
 	*name = s0;
-	*nameend = s0 + 1;
+	*nameend = strchr(*name, '\0');
 	isnum = TRUE;
 	return;
       }
@@ -513,7 +523,7 @@ void onextname(char **name, char **nameend, char *whole, logical build) {
     else if (ISDIGIT(*whole) &&  /* test for num. more subtle while building */
 	     (ISDIGIT(*(*nameend - 1)) || *(*nameend - 1) == '*')) {
       *name = s0;
-      *nameend = s0 + 1;
+      *nameend = strchr(*name, '\0');
       isnum = TRUE;
       return;
     }
@@ -546,6 +556,9 @@ void onextname(char **name, char **nameend, char *whole, logical build) {
 }
 
 void tnextname(char **name, char **nameend, char *whole, logical build) {
+  static char *s0 = LNGSTR_NOEXT;
+  static char *s1 = LNGSTR_BRKDIRS;
+
   logical first = FALSE;
 
   if (*name == NULL) {
@@ -553,24 +566,29 @@ void tnextname(char **name, char **nameend, char *whole, logical build) {
       *nameend = strchr(whole, '\0');
     first = TRUE;
   }
-  else if (*name == whole || *name == whole + 1 || **name == '/') {
+  else if (*name == s0 || *name == s1 || *name == whole ||
+	   *name == whole + 1 || **name == '/') {
     *name = NULL;
     return;
   }
   else
     *nameend = *name - 1;
+
   *name = *nameend - 1;
-  if (**name == '/' && first)
+  if (**name == '/' && first) {
+    *name = s1;
+    *nameend = strchr(*name, '\0');
     return;
+  }
   for ( ; **name != '.' && **name != '/' && *name != whole; (*name)--)
     ;
-  if (**name == '.')
+  if (**name == '.' && (*name) + 1 != *nameend)
     (*name)++;
   else if (build)
     return;
   else if (first) {
-    *name = whole;
-    *nameend = whole;
+    *name = s0;
+    *nameend = strchr(*name, '\0');
   }
   else
     *name = NULL;
@@ -598,27 +616,58 @@ void snextname(char **name, char **nameend, char *whole, logical build) {
 }
 
 void Znextname(char **name, char **nameend, char *whole, logical build) {
-  static char *s0 = "\f";
-  static char *s1 = "\b";
-  static char *s2 = "\v";
+  static char *s1 = LNGSTR_NODOMAIN;
+  static char *s2 = LNGSTR_UNKDOMAIN;
   extern Strpair **domlevels;
 
   Strpair *ap;
   unsigned int c;
 
   if (*name == NULL) {
+    isnum = FALSE;
     *nameend = strchr(whole, '\0');
-    if (ISDIGIT(*(*nameend - 1))) {
-      *name = s0;
-      *nameend = s0 + 1;
+    if (ISDIGIT(*(*nameend - 1)) ||
+	(build && ISDIGIT(*whole) && *(*nameend - 1) == '*')) { /* num. addr */
+      *name = whole;
+      for (c = 0, *nameend = *name;
+	   *nameend - *name <= 3 && ISDIGIT(**nameend); (*nameend)++) {
+	c *= 10;
+	c += **nameend - '0';
+      }   /* c is now first component of IP address */
+      if (**nameend != '.' || *nameend == *name || c > 255) {
+	/* coz of this check, Org Rep can get more s1's & s2's than Dom Rep */
+	if (strchr(*name, '.') == NULL)
+	  *name = s1;
+	else
+	  *name = s2;
+	*nameend = strchr(*name, '\0');
+	return;
+      }
+      /* change c to be number of components required */
+      if (c >= 128 || (c >= 61 && c <= 68) || c == 24 || c == 80 || c == 81)
+	c = 2;
+      else
+	c = 1;
+      for ( ; c > 1; c--) {
+	(*nameend)++;
+	while (ISDIGIT(**nameend))
+	  (*nameend)++;
+	if (**nameend != '.' && !build) {
+	  *name = s2;
+	  *nameend = strchr(*name, '\0');
+	  return;
+	}
+      }
+      isnum = TRUE;
       return;
     }
+    /* non-numerical addresses */
     for (*name = *nameend; **name != '.' && *name != whole; (*name)--)
       ;
     if (*name == whole) {
       if (!build) {
 	*name = s1;
-	*nameend = s1 + 1;
+	*nameend = strchr(*name, '\0');
       }
       return;
     }
@@ -633,7 +682,7 @@ void Znextname(char **name, char **nameend, char *whole, logical build) {
       ;
     if (ap == NULL) {  /* domain not found */
       *name = s2;
-      *nameend = s2 + 1;
+      *nameend = strchr(*name, '\0');
       return;
     }
     /* otherwise we've now found the right number of levels */
@@ -656,6 +705,7 @@ void Znextname(char **name, char **nameend, char *whole, logical build) {
 }
 
 void bnextname(char **name, char **nameend, char *whole, logical build) {
+  /* See Bnextname() below for annotation */
 
   if (*name == NULL) {
     *name = whole;
@@ -695,45 +745,54 @@ void pnextname(char **name, char **nameend, char *whole, logical build) {
     *name = NULL;
 }
 
+/* When compiling the Browser Summary, browser names go through three
+   functions. First Bnextname() locates the bit corresponding to the browser
+   name and version (this is most of the work). Then do_aliasb() makes slight
+   adjustments to get it into canonical form (Name/v.v). Then bnextname()
+   does the actual tree-ification of this canonical form.                    */
 void Bnextname(char **name, char **nameend, char *whole, void *arg) {
   /* NB "arg" is ignored */
-  static char *s1 = "Mozilla (compatible)";
-  static char *s2 = "Mosaic";
-  char *temps;
+  static char *s1 = "Netscape (compatible)";
+  size_t len;
 
   if (*name == NULL) {
-    if (strstr(whole, "osaic") != NULL) {
-      *name = s2;
-      *nameend = s2 + 6;
+    /* First recognise some strings anywhere in the name (mostly claiming to be
+       "Mozilla (compatible)"). The order may matter, for example if some
+       browsers claim to both be Mozilla (compatible) and MSIE (compatible).
+       Note that "len =" is simple assignment and will always return true; it's
+       just a way of setting len. */
+    if (((*name = strstr(whole, "Mosaic")) != NULL && (len = 6)) ||
+	((*name = strstr(whole, "mosaic")) != NULL && (len = 6)) ||
+	((*name = strstr(whole, "Konqueror")) != NULL && (len = 9)) ||
+	((*name = strstr(whole, "Netscape6")) != NULL && (len = 9)) ||
+	((*name = strstr(whole, "Galeon")) != NULL && (len = 6)) ||
+	((*name = strstr(whole, "WebTV")) != NULL && (len = 5)) ||
+	((*name = strstr(whole, "Opera")) != NULL && (len = 5)) ||
+	((*name = strstr(whole, "MSIE")) != NULL && (len = 4))) {
+      *nameend = *name + len;
+      if (**nameend == '/' || **nameend == ' ') {
+	for ((*nameend)++; ISALNUM(**nameend) || **nameend == '.' ||
+	       **nameend == '-' || **nameend == '+'; (*nameend)++)
+	  ;  /* run to end of version number */
+      }
+      else if (headmatch(*name, "Galeon") /* Galeon uses "Galeon; v.vv" */ &&
+	       **nameend == ';' && *(*nameend + 1) == ' ' &&
+	       ISDIGIT(*(*nameend + 2))) {
+	for ((*nameend) += 2; ISALNUM(**nameend) || **nameend == '.' ||
+	       **nameend == '-' || **nameend == '+'; (*nameend)++)
+	  ;
+      }
     }
     else if (headmatch(whole, "Mozilla")) {
-      /* try & do something nearly sensible for all the Moz. (compatible)'s */
-      if ((temps = strstr(whole + 8, "WebTV")) != NULL ||
-	  (temps = strstr(whole + 8, "Opera")) != NULL) {
-	*name = temps;
-
-	if (*(temps + 5) == '\0')
-	  *nameend = temps + 5;
-	else for (*nameend = temps + 6; ISALNUM(**nameend) || **nameend == '.';
-		  (*nameend)++)
-	  ;
-      }
-      else if ((temps = strstr(whole + 8, "MSIE")) != NULL) {
-	*name = temps;
-	if (*(temps + 4) == '\0')
-	  *nameend = temps + 4;
-	else for (*nameend = temps + 5; ISALNUM(**nameend) || **nameend == '.';
-		  (*nameend)++)
-	  ;
-      }
-      else if (strstr(whole + 9, "ompatible")) {
+      if (strstr(whole + 9, "compatible") || strstr(whole + 9, "Compatible")) {
 	*name = s1;
-	*nameend = s1 + 20;
+	*nameend = s1 + 21;
       }
-      else {  /* probably genuine Mozilla */
+      else {  /* probably genuine Netscape/Mozilla */
 	*name = whole;
-	*nameend = strchr(whole + 7, ' ');
-	if (*nameend == NULL)
+	/* Mozilla has version number much later so keep the whole string */
+	if (*(*name + 8) == '5' ||
+	    (*nameend = strchr(whole + 7, ' ')) == NULL)
 	  *nameend = strchr(whole + 7, '\0');
       }
     }
@@ -751,8 +810,8 @@ void Pnextname(char **name, char **nameend, char *whole, void *arg) {
 
   if (*name == NULL) {
     if (arg != NULL && included(whole, FALSE, (Include *)arg)) {
-      *name = "\v";
-      *nameend = (*name) + 1;
+      *name = "Robots";
+      *nameend = strchr(*name, '\0');
       return;
     }
     if (headmatch(whole, "Mozilla"))
@@ -770,18 +829,39 @@ void Pnextname(char **name, char **nameend, char *whole, void *arg) {
 	  *name = "Windows:Windows 95";
       else if (*c == '9' && *(c + 1) == '8') {
 	if (strstr(c, "Win 9x 4.9"))
-	  *name = "Windows:Windows Me";
+	  *name = "Windows:Windows ME";
 	else
 	  *name = "Windows:Windows 98";
       }
       else if (*c == 'N' && *(c + 1) == 'T') {
-	if (*(c + 2) == ' ' && *(c + 3) == '5')
-	  *name = "Windows:Windows 2000";
+	/* advance to NT version number, which corresponds to advertised OS */
+	c += 2;
+	if (*c == ' ')
+	  c++;
+	if (*c == '5') {
+	  if (*(c + 1) == '.' && (*(c + 2) == '0'))
+	    *name = "Windows:Windows 2000";
+	  else if (*(c + 1) == '.' && (*(c + 2) == '1'))
+	    *name = "Windows:Windows XP";
+	  else
+	    *name = "Windows:Unknown Windows";
+	}
+	else if (*c >= '6' && *c <= '9')
+	  *name = "Windows:Unknown Windows";
 	else
 	  *name = "Windows:Windows NT";
       }
       else if (*c == 'C' && *(c + 1) == 'E')
 	*name = "Windows:Windows CE";
+      /* next three not MSIE, but some other vendor might use them */
+      else if (*c == 'X' && *(c + 1) == 'P')
+	*name = "Windows:Windows XP";
+      else if (*c == '2' && *(c + 1) == '0' && *(c + 2) == '0' &&
+	       *(c + 3) == '0')
+	*name = "Windows:Windows 2000";
+      else if (*c == 'M' && (*(c + 1) == 'E' || (*(c + 1) == 'e') ||
+			     headmatch(c + 1, "illennium")))
+	*name = "Windows:Windows ME";
       else if (*c == '3' && *(c + 1) == '.' && *(c + 2) == '1')
 	*name = "Windows:Windows 3.1";
       else if ((*c == '1' && *(c + 1) == '6') || strstr(c + 1, "16bit") ||
@@ -791,7 +871,7 @@ void Pnextname(char **name, char **nameend, char *whole, void *arg) {
 	       strstr(c + 1, "32-bit"))
 	*name = "Windows:Windows 32-bit";
       else
-	*name = "Windows:unkwin";
+	*name = "Windows:Unknown Windows";
     }
     else if ((c = strstr(whole, "Mac")) != NULL) {
       if (strstr(c + 3, "68"))
@@ -799,7 +879,7 @@ void Pnextname(char **name, char **nameend, char *whole, void *arg) {
       else if (strstr(c + 3, "PPC") || strstr(c + 3, "PowerPC"))
 	*name = "Macintosh:Macintosh PowerPC";
       else
-	*name = "Macintosh:unkmac";
+	*name = "Macintosh:Unknown Macintosh";
     }
     else if (strstr(whole, "Linux") != NULL || strstr(whole, "linux") != NULL)
       *name = "Unix:Linux";
@@ -819,7 +899,7 @@ void Pnextname(char **name, char **nameend, char *whole, void *arg) {
     else if (strstr(whole, "VMS") != NULL)
       *name = "OpenVMS";
     else if (strstr(whole, "X11") != NULL)
-      *name = "Unix:unkux";
+      *name = "Unix:Other Unix";
     else if (strstr(whole, "WebTV") != NULL)
       *name = "WebTV";
     else if (strstr(whole, "OS/2") != NULL)
@@ -833,7 +913,7 @@ void Pnextname(char **name, char **nameend, char *whole, void *arg) {
     else if (strstr(whole, "Atari") != NULL)
       *name = "Atari";
     else
-      *name = "unkos";
+      *name = "OS unknown";
     *nameend = strchr(*name, '\0');
   }
   else
@@ -848,29 +928,28 @@ void Nnextname(char **name, char **nameend, char *whole, void *arg) {
   logical done;
 
   if (*name == NULL) {
-    if ((c = strchr(whole, '?')) != NULL && *(c + 1) != '\0') {
-      *c = '\0';
-      d = c;
+    if ((d = strchr(whole, '?')) != NULL && *(d + 1) != '\0') {
+      *d = '\0';
       for (al = (Strpair *)arg, done = FALSE; al != NULL && !done;
 	   TO_NEXT(al)) {
 	if (MATCHES(whole, al->name)) {  /* find right engine in list */
-	  for (c = d; c != NULL && !done; c = strchr(c, '&')) {
+	  for (c = d; c != NULL && !done; c = strpbrk(c, "&;")) {
 	    c++;                            /* find right arg */
 	    if (headmatch(c, al->data) && *(c + strlen(al->data)) == '=') {
 	      done = TRUE;
 	      *name = c + strlen(al->data) + 1;
-	      for (*nameend = *name; **nameend != '&' && **nameend != '\0';
-		   (*nameend)++)
-		;  /* run to next & */
+	      for (*nameend = *name; **nameend != '&' && **nameend != ';' &&
+		     **nameend != '\0'; (*nameend)++)
+		;  /* run nameend to next & or ; */
 	      if (*name == *nameend)
 		name = NULL;
-	    }
-	  }
-	}
-      }
+	    }    /* if headmatch */
+	  }    /* for c */
+	}    /* if MATCHES */
+      }    /* for al */
       *d = '?';
-    }
-  }
+    }    /* if d = strchr */
+  }    /* if *name == NULL */
   else
     *name = NULL;
 }
@@ -879,7 +958,7 @@ void nnextname(char **name, char **nameend, char *whole, void *arg) {
 
   if (*name == NULL)
     Nnextname(name, nameend, whole, arg);   /* to find right CGI arg etc. */
-  else if (**nameend == '&' || **nameend == '\0')
+  else if (**nameend == '&' || **nameend == ';' || **nameend == '\0')
     *name = NULL;
   else
     *name = *nameend + 1;
@@ -898,7 +977,7 @@ void nnextname(char **name, char **nameend, char *whole, void *arg) {
 	(*name)++)
     ;  /* run to first wanted character; cf list in do_aliasN() and below */
        /* NB 'good' dots never occur at beginning of word */
-  if (**name == '&' || **name == '\0')
+  if (**name == '&' || **name == ';' || **name == '\0')
     *name = NULL;
   else for (*nameend = *name; **nameend != '+' && **nameend != '&' &&
 	      **nameend != '\0' && **nameend != '"' && **nameend != ',' &&
