@@ -1,5 +1,5 @@
-/***             analog 5.03             http://www.analog.cx/             ***/
-/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
+/***             analog 5.22             http://www.analog.cx/             ***/
+/*** This program is copyright (c) Stephen R. E. Turner 1995 - 2002 except as
  *** stated otherwise. Distribution, usage and modification of this program is
  *** subject to the conditions of the Licence which you should have received
  *** with it. This program comes with no warranty, expressed or implied.   ***/
@@ -542,41 +542,8 @@ void freemm(Memman *m) {
     free(p->pos);
 }
 
-/* The next function is derived from the FreeBSD 2.2.6 source, and comes with
-   the following copyright notice:
- * Copyright (c) 1990, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Chris Torek.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+/* The next function is derived from the FreeBSD 2.2.6 source, and is subject
+   to the copyright notice in docs/LicBSD.txt within the analog distribution.
  */
 
 #ifdef NEED_MEMMOVE
@@ -777,6 +744,7 @@ logical headcasematch(char *s, char *p) {  /* same, only case insensitive */
   return(TRUE);
 }
 
+/* Is an item included according to an INCLUDE/EXCLUDE list? */
 logical included(char *name, logical ispage, Include *listhead) {
   logical is_regex;
   Include *lp, *lastlp;
@@ -802,6 +770,17 @@ logical included(char *name, logical ispage, Include *listhead) {
   return(!IS_INC(lastlp->type));
 }
 
+/* Is an item included in an Strlist? */
+logical incstrlist(char *name, Strlist *listhead) {
+  Strlist *lp;
+
+  for (lp = listhead; lp != NULL; TO_NEXT(lp)) {
+    if (STREQ(name, lp->name))
+      return(TRUE);
+  }
+  return(FALSE);
+}
+
 logical pageq(char *name, Include *ispagehead, choice type) {
   char *c;
   logical ans;
